@@ -14,12 +14,16 @@
 #include "bar_exif.h"
 
 #include "exif.h"
+#include "ui_bookmark.h"
+#include "ui_misc.h"
 
 #include <math.h>
 
 #define EXIF_BAR_SIZE_INCREMENT 48
 #define EXIF_BAR_ARROW_SIZE 7
 
+#define EXIF_BAR_CUSTOM_COUNT 20
+
 static const gchar *bar_exif_key_list[] = {
 	"fCamera",
 	"fDateTime",
@@ -48,14 +52,15 @@ static const gchar *bar_exif_key_list[] = {
  *-------------------------------------------------------------------
  */
 
-static GtkWidget *table_add_line(GtkWidget *table, gint x, gint y,
-				 const gchar *description, const gchar *text)
+static void table_add_line_custom(GtkWidget *table, gint x, gint y,
+                                  const gchar *text1, const gchar *text2,
+                                  GtkWidget **label1, GtkWidget **label2)
 {
 	GtkWidget *label;
 	gchar *buf;
 
-	buf = g_strconcat((description) ? description : "fixme", ":", NULL);
-	if (!text) text = "";
+	buf = g_strconcat((text1) ? text1 : "fixme", ":", NULL);
+	if (!text2) text2 = "";
 
 	label = gtk_label_new(buf);
 	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.0);
@@ -63,20 +68,33 @@ static GtkWidget *table_add_line(GtkWidget *table, gint x, gint y,
 			 x, x + 1, y, y + 1,
 			 GTK_FILL, GTK_FILL,
 			 2, 2);
-	gtk_widget_show(label);
+	*label1 = label;
 
-	label = gtk_label_new(text);
+	label = gtk_label_new(text2);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
 	gtk_table_attach(GTK_TABLE(table), label,
 			 x + 1, x + 2, y, y + 1,
 			 GTK_FILL, GTK_FILL,
 			 2, 2);
-	gtk_widget_show(label);
+	*label2 = label;
 
 	g_free(buf);
+}
+
+static GtkWidget *table_add_line(GtkWidget *table, gint x, gint y,
+				 const gchar *description, const gchar *text)
+{
+	GtkWidget *key;
+	GtkWidget *label;
+
+	table_add_line_custom(table, x, y, description, text, &key, &label);
+	gtk_widget_show(key);
+	gtk_widget_show(label);
+
 	return label;
 }
 
+
 /*
  *-------------------------------------------------------------------
  * EXIF bar
@@ -93,9 +111,24 @@ struct _ExifBar
 	GtkWidget *listview;
 	GtkWidget **labels;
 
+	GtkWidget *custom_sep;
+	GtkWidget *custom_name[EXIF_BAR_CUSTOM_COUNT];
+	GtkWidget *custom_value[EXIF_BAR_CUSTOM_COUNT];
+
 	gchar *path;
 };
 
+enum {
+	EXIF_ADVCOL_ENABLED = 0,
+	EXIF_ADVCOL_TAG,
+	EXIF_ADVCOL_NAME,
+	EXIF_ADVCOL_VALUE,
+	EXIF_ADVCOL_FORMAT,
+	EXIF_ADVCOL_ELEMENTS,
+	EXIF_ADVCOL_DESCRIPTION,
+	EXIF_ADVCOL_COUNT
+};
+
 static gchar *bar_exif_validate_text(gchar *text)
 {
 	if (text && !g_utf8_validate(text, strlen(text), NULL))
@@ -113,6 +146,20 @@ static void bar_exif_sensitive(ExifBar *eb, gint enable)
 	if (eb->advanced_scrolled) gtk_widget_set_sensitive(eb->advanced_scrolled, enable);
 }
 
+static gint bar_exif_row_enabled(const gchar *name)
+{
+	GList *list;
+
+	list = history_list_get_by_key("exif_extras");
+	while (list)
+		{
+		if (name && strcmp(name, (gchar *)(list->data)) == 0) return TRUE;
+		list = list->next;
+	}
+
+	return FALSE;
+}
+
 static void bar_exif_update(ExifBar *eb)
 {
 	ExifData *exif;
@@ -130,6 +177,7 @@ static void bar_exif_update(ExifBar *eb)
 
 	if (GTK_WIDGET_VISIBLE(eb->scrolled))
 		{
+		GList *list;
 		len = bar_exif_key_count;
 		for (i = 0; i < len; i++)
 			{
@@ -139,6 +187,46 @@ static void bar_exif_update(ExifBar *eb)
 			gtk_label_set_text(GTK_LABEL(eb->labels[i]), text);
 			g_free(text);
 			}
+
+		list = g_list_last(history_list_get_by_key("exif_extras"));
+		if (list)
+			{
+			gtk_widget_show(eb->custom_sep);
+			}
+		else
+			{
+			gtk_widget_hide(eb->custom_sep);
+			}
+		i = 0;
+		while (list && i < EXIF_BAR_CUSTOM_COUNT)
+			{
+			gchar *text;
+			gchar *name;
+			gchar *buf;
+
+			name = list->data;
+			list = list->prev;
+
+			text = exif_get_data_as_text(exif, name);
+			text = bar_exif_validate_text(text);
+
+			buf = g_strconcat(name, ":", NULL);
+			gtk_label_set_text(GTK_LABEL(eb->custom_name[i]), buf);
+			g_free(buf);
+			gtk_label_set_text(GTK_LABEL(eb->custom_value[i]), text);
+			g_free(text);
+
+			gtk_widget_show(eb->custom_name[i]);
+			gtk_widget_show(eb->custom_value[i]);
+
+			i++;
+			}
+		while (i < EXIF_BAR_CUSTOM_COUNT)
+			{
+			gtk_widget_hide(eb->custom_name[i]);
+			gtk_widget_hide(eb->custom_value[i]);
+			i++;
+			}
 		}
 
 	if (eb->advanced_scrolled && GTK_WIDGET_VISIBLE(eb->advanced_scrolled))
@@ -173,8 +261,14 @@ static void bar_exif_update(ExifBar *eb)
 			description = exif_item_get_description(item);
 			if (!description) description = "";
 			gtk_list_store_append(store, &iter);
-			gtk_list_store_set(store, &iter, 0, tag, 1, tag_name, 2, text,
-					   3, format, 4, elements, 5, description, -1);
+			gtk_list_store_set(store, &iter,
+					EXIF_ADVCOL_ENABLED, bar_exif_row_enabled(tag_name),
+					EXIF_ADVCOL_TAG, tag,
+					EXIF_ADVCOL_NAME, tag_name,
+					EXIF_ADVCOL_VALUE, text,
+					EXIF_ADVCOL_FORMAT, format,
+					EXIF_ADVCOL_ELEMENTS, elements,
+					EXIF_ADVCOL_DESCRIPTION, description, -1);
 			g_free(tag);
 			g_free(text);
 			g_free(elements);
@@ -196,6 +290,10 @@ static void bar_exif_clear(ExifBar *eb)
 		{
 		gtk_label_set_text(GTK_LABEL(eb->labels[i]), "");
 		}
+	for (i = 0; i < EXIF_BAR_CUSTOM_COUNT; i++)
+		{
+		gtk_label_set_text(GTK_LABEL(eb->custom_value[i]), "");
+		}
 
 	if (eb->listview)
 		{
@@ -221,6 +319,63 @@ void bar_exif_set(GtkWidget *bar, const gchar *path)
 	bar_exif_update(eb);
 }
 
+static void bar_exif_row_toggled_cb(GtkCellRendererToggle *toggle, const gchar *path, gpointer data)
+{
+	GtkWidget *listview = data;
+	GtkTreeModel *store;
+	GtkTreeIter iter;
+	GtkTreePath *tpath;
+	gchar *name = NULL;
+	gboolean active;
+
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(listview));
+
+	tpath = gtk_tree_path_new_from_string(path);
+	gtk_tree_model_get_iter(store, &iter, tpath);
+	gtk_tree_path_free(tpath);
+
+	gtk_tree_model_get(store, &iter, EXIF_ADVCOL_ENABLED, &active,
+					 EXIF_ADVCOL_NAME, &name, -1);
+	active = (!active);
+
+	if (active &&
+	    g_list_length(history_list_get_by_key("exif_extras")) >= EXIF_BAR_CUSTOM_COUNT)
+		{
+		active = FALSE;
+		}
+
+	gtk_list_store_set(GTK_LIST_STORE(store), &iter, EXIF_ADVCOL_ENABLED, active, -1);
+
+	if (active)
+		{
+		history_list_add_to_key("exif_extras", name, EXIF_BAR_CUSTOM_COUNT);
+		}
+	else
+		{
+		history_list_item_change("exif_extras", name, NULL);
+		}
+
+	g_free(name);
+}
+
+static void bar_exif_add_column_check(GtkWidget *listview, const gchar *title, gint n)
+{
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+
+	column = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(column, title);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+
+	renderer = gtk_cell_renderer_toggle_new();
+	gtk_tree_view_column_pack_start(column, renderer, TRUE);
+	gtk_tree_view_column_add_attribute(column, renderer, "active", n);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(listview), column);
+
+	g_signal_connect(G_OBJECT(renderer), "toggled",
+			 G_CALLBACK(bar_exif_row_toggled_cb), listview);
+}
+
 static void bar_exif_add_column(GtkWidget *listview, const gchar *title, gint n)
 {
 	GtkTreeViewColumn *column;
@@ -242,19 +397,23 @@ static void bar_exif_advanced_build_view(ExifBar *eb)
 
 	if (eb->listview) return;
 
-	store = gtk_list_store_new(6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+	store = gtk_list_store_new(7, G_TYPE_BOOLEAN,
+				      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
 				      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 	eb->listview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(eb->listview), TRUE);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(eb->listview), TRUE);
 
-	bar_exif_add_column(eb->listview, _("Tag"), 0);
-	bar_exif_add_column(eb->listview, _("Name"), 1);
-	bar_exif_add_column(eb->listview, _("Value"), 2);
-	bar_exif_add_column(eb->listview, _("Format"), 3);
-	bar_exif_add_column(eb->listview, _("Elements"), 4);
-	bar_exif_add_column(eb->listview, _("Description"), 5);
+	bar_exif_add_column_check(eb->listview, "", EXIF_ADVCOL_ENABLED);
+
+	bar_exif_add_column(eb->listview, _("Tag"), EXIF_ADVCOL_TAG);
+	bar_exif_add_column(eb->listview, _("Name"), EXIF_ADVCOL_NAME);
+	bar_exif_add_column(eb->listview, _("Value"), EXIF_ADVCOL_VALUE);
+	bar_exif_add_column(eb->listview, _("Format"), EXIF_ADVCOL_FORMAT);
+	bar_exif_add_column(eb->listview, _("Elements"), EXIF_ADVCOL_ELEMENTS);
+	bar_exif_add_column(eb->listview, _("Description"), EXIF_ADVCOL_DESCRIPTION);
 
 	eb->advanced_scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(eb->advanced_scrolled), GTK_SHADOW_IN);
@@ -340,7 +499,7 @@ static void bar_exif_destroy(GtkWidget *widget, gpointer data)
 	g_free(eb);
 }
 
-GtkWidget *bar_exif_new(gint show_title, const gchar *path, gint advanced)
+GtkWidget *bar_exif_new(gint show_title, const gchar *path, gint advanced, GtkWidget *bounding_widget)
 {
 	ExifBar *eb;
 	GtkWidget *table;
@@ -367,6 +526,11 @@ GtkWidget *bar_exif_new(gint show_title, const gchar *path, gint advanced)
 
 		box = gtk_hbox_new(FALSE, 0);
 
+		label = sizer_new(eb->vbox, bounding_widget, SIZER_POS_LEFT);
+		sizer_set_limits(label, EXIF_BAR_SIZE_INCREMENT * 2, -1, -1 , -1);
+		gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
+		gtk_widget_show(label);
+
 		label = gtk_label_new(_("Exif"));
 		gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
 		gtk_widget_show(label);
@@ -397,7 +561,7 @@ GtkWidget *bar_exif_new(gint show_title, const gchar *path, gint advanced)
 		gtk_widget_show(box);
 		}
 
-	table = gtk_table_new(2, 9, FALSE);
+	table = gtk_table_new(2, bar_exif_key_count + 1 + EXIF_BAR_CUSTOM_COUNT, FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 
 	eb->table = table;
@@ -411,6 +575,17 @@ GtkWidget *bar_exif_new(gint show_title, const gchar *path, gint advanced)
 		eb->labels[i] = table_add_line(table, 0, i, text, NULL);
 		}
 
+	eb->custom_sep = gtk_hseparator_new();
+	gtk_table_attach(GTK_TABLE(table), eb->custom_sep, 0, 1,
+					   bar_exif_key_count, bar_exif_key_count + 1,
+					   GTK_FILL, GTK_FILL, 2, 2);
+
+	for (i = 0; i < EXIF_BAR_CUSTOM_COUNT; i++)
+		{
+		table_add_line_custom(table, 0, bar_exif_key_count + 1 + i,
+				      "", "",  &eb->custom_name[i], &eb->custom_value[i]);
+		}
+
 	eb->scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(eb->scrolled),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
