@@ -615,28 +615,28 @@ static void collection_table_popup_copy_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 
-	file_util_copy(NULL, collection_table_popup_file_list(ct), NULL);
+	file_util_copy(NULL, collection_table_popup_file_list(ct), NULL, ct->listview);
 }
 
 static void collection_table_popup_move_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 
-	file_util_move(NULL, collection_table_popup_file_list(ct), NULL);
+	file_util_move(NULL, collection_table_popup_file_list(ct), NULL, ct->listview);
 }
 
 static void collection_table_popup_rename_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 
-	file_util_rename(NULL, collection_table_popup_file_list(ct));
+	file_util_rename(NULL, collection_table_popup_file_list(ct), ct->listview);
 }
 
 static void collection_table_popup_delete_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
 
-	file_util_delete(NULL, collection_table_popup_file_list(ct));
+	file_util_delete(NULL, collection_table_popup_file_list(ct), ct->listview);
 }
 
 static void collection_table_popup_sort_cb(GtkWidget *widget, gpointer data)
@@ -1972,13 +1972,13 @@ static GtkWidget *collection_table_drop_menu(CollectTable *ct)
 	g_signal_connect(G_OBJECT(menu), "destroy",
 			 G_CALLBACK(collection_table_popup_destroy_cb), ct);
 
-	menu_item_add_stock(menu, _("Dropped list includes directories."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
+	menu_item_add_stock(menu, _("Dropped list includes folders."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("_Add contents"), GTK_STOCK_OK,
 			    G_CALLBACK(confirm_dir_list_add), ct);
 	menu_item_add_stock(menu, _("Add contents _recursive"), GTK_STOCK_ADD,
 			    G_CALLBACK(confirm_dir_list_recurse), ct);
-	menu_item_add_stock(menu, _("_Skip directories"), GTK_STOCK_REMOVE,
+	menu_item_add_stock(menu, _("_Skip folders"), GTK_STOCK_REMOVE,
 			    G_CALLBACK(confirm_dir_list_skip), ct);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("Cancel"), GTK_STOCK_CANCEL, NULL, ct);
