@@ -257,7 +257,7 @@ static gint collection_list_find_index(GList *list, gchar *path)
 }
 #endif
 
-static CollectWindow *collection_window_find(CollectionData *cd)
+CollectWindow *collection_window_find(CollectionData *cd)
 {
 	GList *work;
 
@@ -318,7 +318,7 @@ CollectionData *collection_new(const gchar *path)
 			}
 		else
 			{
-			cd->name = g_strdup_printf(_("Untitled (%d)"), untitled_counter);
+			cd->name = g_strdup_printf(_("Untitled (%d)"), untitled_counter + 1);
 			}
 
 		untitled_counter++;
@@ -778,25 +778,29 @@ static gint collection_window_keypress(GtkWidget *widget, GdkEventKey *event, gp
 				stop_signal = TRUE;
 				break;
 			case 'C': case 'c':
-				file_util_copy(NULL, collection_table_selection_get_list(cw->table), NULL);
+				file_util_copy(NULL, collection_table_selection_get_list(cw->table), NULL, cw->window);
 				stop_signal = TRUE;
 				break;
 			case 'M': case 'm':
-				file_util_move(NULL, collection_table_selection_get_list(cw->table), NULL);
+				file_util_move(NULL, collection_table_selection_get_list(cw->table), NULL, cw->window);
 				stop_signal = TRUE;
 				break;
 			case 'R': case 'r':
-				file_util_rename(NULL, collection_table_selection_get_list(cw->table));
+				file_util_rename(NULL, collection_table_selection_get_list(cw->table), cw->window);
 				stop_signal = TRUE;
 				break;
 			case 'D': case 'd':
-				file_util_delete(NULL, collection_table_selection_get_list(cw->table));
+				file_util_delete(NULL, collection_table_selection_get_list(cw->table), cw->window);
 				stop_signal = TRUE;
 				break;
 			case 'P': case 'p':
 				info_window_new(NULL, collection_table_selection_get_list(cw->table));
 				stop_signal = TRUE;
 				break;
+			case 'S': case 's':
+				collection_dialog_save_as(NULL, cw->cd);
+				stop_signal = TRUE;
+				break;
 			case 'W': case 'w':
 				collection_window_close(cw);
 				stop_signal = TRUE;
@@ -820,20 +824,13 @@ static gint collection_window_keypress(GtkWidget *widget, GdkEventKey *event, gp
 				stop_signal = TRUE;
 				break;
 			case 'S': case 's':
-				if (event->state & GDK_CONTROL_MASK)
+				if (!cw->cd->path)
 					{
 					collection_dialog_save_as(NULL, cw->cd);
-					}
-				else
+			                }
+				else if (!collection_save(cw->cd, cw->cd->path))
 					{
-					if (!cw->cd->path)
-						{
-						collection_dialog_save_as(NULL, cw->cd);
-				                }
-					else if (!collection_save(cw->cd, cw->cd->path))
-						{
-						printf("failed saving to collection path: %s\n", cw->cd->path);
-						}
+					printf("failed saving to collection path: %s\n", cw->cd->path);
 					}
 				stop_signal = TRUE;
 				break;
@@ -1007,7 +1004,7 @@ static void collection_close_save_cb(GenericDialog *gd, gpointer data)
 		{
 		gchar *buf;
 		buf = g_strdup_printf(_("Failed to save the collection:\n%s"), cw->cd->path);
-		warning_dialog(_("Save Failed"), buf);
+		warning_dialog(_("Save Failed"), buf, cw->window);
 		g_free(buf);
 		return;
 		}
@@ -1044,7 +1041,7 @@ static void collection_close_dlg_show(CollectWindow *cw)
 		}
 
 	gd = generic_dialog_new(_("Close collection"), _("Collection has been modified.\nSave first?"),
-				"GQview", "close_collection", FALSE,
+				"GQview", "close_collection", cw->window, FALSE,
 				collection_close_cancel_cb, cw);
 
 	generic_dialog_add_stock(gd, _("Save"), GTK_STOCK_SAVE, collection_close_save_cb, TRUE);
