@@ -24,6 +24,7 @@
 #include "info.h"
 #include "layout.h"
 #include "layout_image.h"
+#include "md5-util.h"
 #include "menu.h"
 #include "thumb.h"
 #include "utilops.h"
@@ -36,7 +37,6 @@
 
 
 #include <math.h>
-#include <utime.h>
 
 
 #define DUPE_DEF_WIDTH 600
@@ -113,62 +113,88 @@ static void dupe_window_update_count(DupeWindow *dw, gint count_only)
 	g_free(text);
 }
 
+static guint64 msec_time(void)
+{
+	struct timeval tv;
+
+	if (gettimeofday(&tv, NULL) == -1) return 0;
+
+	return (guint64)tv.tv_sec * 1000000 + (guint64)tv.tv_usec;
+}
+
 static gint dupe_iterations(gint n)
 {
 	return (n * ((n + 1) / 2));
 }
 
-static void dupe_window_update_progress(DupeWindow *dw, const gchar *status, gfloat value)
+static void dupe_window_update_progress(DupeWindow *dw, const gchar *status, gfloat value, gint force)
 {
 	const gchar *status_text;
 
 	if (status)
 		{
-		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(dw->extra_label), value);
+		guint64 new_time = 0;
 
-		if (dw->setup_count > 0)
+		if (dw->setup_n % 10 == 0)
 			{
-			if (dw->setup_n % 10 == 0)
-				{
-				dw->setup_time_count = (gint)time(NULL) - dw->setup_time;
-				}
+			new_time = msec_time() - dw->setup_time;
+			}
+		
+		if (!force &&
+		    value != 0.0 &&
+		    dw->setup_count > 0 &&
+		    new_time > 2000000)
+			{
+			gchar *buf;
+			gint t;
+			gint d;
+			guint32 rem;
 
-			if (dw->setup_time_count > 3)
-				{
-				gchar *buf;
-				gint t;
-				gint d;
-				guint32 rem;
+			if (new_time - dw->setup_time_count < 250000) return;
+			dw->setup_time_count = new_time;
 
-				if (dw->setup_done)
-					{
-					if (dw->second_set)
-						{
-						t = dw->setup_count;
-						d = dw->setup_count - dw->setup_n;
-						}
-					else
-						{
-						t = dupe_iterations(dw->setup_count);
-						d = dupe_iterations(dw->setup_count - dw->setup_n);
-						}
-					}
-				else
+			if (dw->setup_done)
+				{
+				if (dw->second_set)
 					{
 					t = dw->setup_count;
 					d = dw->setup_count - dw->setup_n;
 					}
-
-				rem = (t - d) ? ((float)dw->setup_time_count / (t - d)) * d : 0;
-
-				buf = g_strdup_printf("%s %d:%02d ", status, rem / 60, rem % 60);
-
-				gtk_progress_bar_set_text(GTK_PROGRESS_BAR(dw->extra_label), buf);
-				g_free(buf);
-				return;
+				else
+					{
+					t = dupe_iterations(dw->setup_count);
+					d = dupe_iterations(dw->setup_count - dw->setup_n);
+					}
 				}
+			else
+				{
+				t = dw->setup_count;
+				d = dw->setup_count - dw->setup_n;
+				}
+
+			rem = (t - d) ? ((float)(dw->setup_time_count / 1000000) / (t - d)) * d : 0;
+
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(dw->extra_label), value);
+
+			buf = g_strdup_printf("%s %d:%02d ", status, rem / 60, rem % 60);
+			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(dw->extra_label), buf);
+			g_free(buf);
+
+			return;
+			}
+		else if (force ||
+			 value == 0.0 ||
+			 dw->setup_count == 0 ||
+			 (new_time > 0 && new_time - dw->setup_time_count >= 250000))
+			{
+			if (new_time > 0) dw->setup_time_count = new_time;
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(dw->extra_label), value);
+			status_text = status;
+			}
+		else
+			{
+			status_text = NULL;
 			}
-		status_text = status;
 		}	
 	else
 		{
@@ -176,7 +202,7 @@ static void dupe_window_update_progress(DupeWindow *dw, const gchar *status, gfl
 		status_text = " ";
 		}
 
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(dw->extra_label), status_text);
+	if (status_text) gtk_progress_bar_set_text(GTK_PROGRESS_BAR(dw->extra_label), status_text);
 }
 
 static void widget_set_cursor(GtkWidget *widget, gint icon)
@@ -263,6 +289,7 @@ static DupeItem *dupe_item_new(const gchar *path, gint64 size, time_t date)
 
 	di->simd = NULL;
 	di->checksum = 0;
+	di->md5sum = NULL;
 	di->width = 0;
 	di->height = 0;
 
@@ -275,6 +302,7 @@ static void dupe_item_free(DupeItem *di)
 {
 	g_free(di->path);
 	image_sim_free(di->simd);
+	g_free(di->md5sum);
 	if (di->pixbuf) g_object_unref(di->pixbuf);
 
 	g_free(di);
@@ -329,7 +357,7 @@ static void dupe_item_read_cache(DupeItem *di)
 
 	if (!di) return;
 
-	path = cache_find_location(di->path, GQVIEW_CACHE_SIMIL_EXT);
+	path = cache_find_location(CACHE_TYPE_SIM, di->path);
 	if (!path) return;
 
 	if (filetime(di->path) != filetime(path))
@@ -357,6 +385,10 @@ static void dupe_item_read_cache(DupeItem *di)
 			{
 			di->checksum = cd->checksum;
 			}
+		if (!di->md5sum && cd->have_md5sum)
+			{
+			di->md5sum = md5_digest_to_text(cd->md5sum);
+			}
 		cache_sim_data_free(cd);
 		}
 }
@@ -368,16 +400,21 @@ static void dupe_item_write_cache(DupeItem *di)
 
 	if (!di) return;
 
-	base = cache_get_location(di->path, FALSE, NULL, &mode);
+	base = cache_get_location(CACHE_TYPE_SIM, di->path, FALSE, &mode);
 	if (cache_ensure_dir_exists(base, mode))
 		{
 		CacheData *cd;
 
 		cd = cache_sim_data_new();
-		cd->path = cache_get_location(di->path, TRUE, GQVIEW_CACHE_SIMIL_EXT, NULL);
+		cd->path = cache_get_location(CACHE_TYPE_SIM, di->path, TRUE, NULL);
 
 		if (di->width != 0) cache_sim_data_set_dimensions(cd, di->width, di->height);
 		if (di->checksum != 0) cache_sim_data_set_checksum(cd, di->checksum);
+		if (di->md5sum)
+			{
+			guchar digest[16];
+			if (md5_digest_from_text(di->md5sum, digest)) cache_sim_data_set_md5sum(cd, digest);
+			}
 		if (di->simd) cache_sim_data_set_similarity(cd, di->simd);
 
 		if (cache_sim_data_save(cd))
@@ -1053,9 +1090,11 @@ static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask, gfloat *ran
 		}
 	if (mask & DUPE_MATCH_SUM)
 		{
-		if (a->checksum == 0) a->checksum = checksum_simple(a->path);
-		if (b->checksum == 0) b->checksum = checksum_simple(b->path);
-		if (a->checksum != b->checksum) return FALSE;
+		if (!a->md5sum) a->md5sum = md5_text_from_file_utf8(a->path, "");
+		if (!b->md5sum) b->md5sum = md5_text_from_file_utf8(b->path, "");
+		if (a->md5sum[0] == '\0' ||
+		    b->md5sum[0] == '\0' ||
+		    strcmp(a->md5sum, b->md5sum) != 0) return FALSE;
 		}
 	if (mask & DUPE_MATCH_DIM)
 		{
@@ -1223,12 +1262,12 @@ static void dupe_thumb_step(DupeWindow *dw)
 		thumb_loader_free(dw->thumb_loader);
 		dw->thumb_loader = NULL;
 
-		dupe_window_update_progress(dw, NULL, 0.0);
+		dupe_window_update_progress(dw, NULL, 0.0, FALSE);
 		return;
 		}
 
 	dupe_window_update_progress(dw, _("Loading thumbs..."),
-				    length == 0 ? 0.0 : (float)(row) / length);
+				    length == 0 ? 0.0 : (float)(row) / length, FALSE);
 
 	dw->thumb_item = di;
 	thumb_loader_free(dw->thumb_loader);
@@ -1259,7 +1298,7 @@ static void dupe_check_stop(DupeWindow *dw)
 		{
 		gtk_idle_remove(dw->idle_id);
 		dw->idle_id = -1;
-		dupe_window_update_progress(dw, NULL, 0.0);
+		dupe_window_update_progress(dw, NULL, 0.0, FALSE);
 		widget_set_cursor(dw->listview, -1);
 		}
 
@@ -1313,7 +1352,7 @@ static void dupe_setup_reset(DupeWindow *dw)
 {
 	dw->setup_point = NULL;
 	dw->setup_n = 0;
-	dw->setup_time = (gint)time(NULL);
+	dw->setup_time = msec_time();
 	dw->setup_time_count = 0;
 }
 
@@ -1341,25 +1380,25 @@ static gint dupe_check_cb(gpointer data)
 			{
 			if (!dw->setup_point) dw->setup_point = dw->list;
 
-			while(dw->setup_point)
+			while (dw->setup_point)
 				{
 				DupeItem *di = dw->setup_point->data;
 
 				dw->setup_point = dupe_setup_point_step(dw, dw->setup_point);
 				dw->setup_n++;
 
-				if (di->checksum == 0)
+				if (!di->md5sum)
 					{
 					dupe_window_update_progress(dw, _("Reading checksums..."),
-						dw->setup_count == 0 ? 0.0 : (float)dw->setup_n / dw->setup_count);
+						dw->setup_count == 0 ? 0.0 : (float)(dw->setup_n - 1) / dw->setup_count, FALSE);
 
 					if (enable_thumb_caching)
 						{
 						dupe_item_read_cache(di);
-						if (di->checksum != 0) return TRUE;
+						if (di->md5sum) return TRUE;
 						}
 
-					di->checksum = checksum_simple(di->path);
+					di->md5sum = md5_text_from_file_utf8(di->path, "");
 					if (enable_thumb_caching)
 						{
 						dupe_item_write_cache(di);
@@ -1375,15 +1414,16 @@ static gint dupe_check_cb(gpointer data)
 			{
 			if (!dw->setup_point) dw->setup_point = dw->list;
 
-			while(dw->setup_point)
+			while (dw->setup_point)
 				{
 				DupeItem *di = dw->setup_point->data;
 
 				dw->setup_point = dupe_setup_point_step(dw, dw->setup_point);
+				dw->setup_n++;
 				if (di->width == 0 && di->height == 0)
 					{
 					dupe_window_update_progress(dw, _("Reading dimensions..."),
-						dw->setup_count == 0 ? 0.0 : (float)dw->setup_n / dw->setup_count);
+						dw->setup_count == 0 ? 0.0 : (float)(dw->setup_n - 1) / dw->setup_count, FALSE);
 
 					if (enable_thumb_caching)
 						{
@@ -1398,7 +1438,6 @@ static gint dupe_check_cb(gpointer data)
 						}
 					return TRUE;
 					}
-				dw->setup_n++;
 				}
 			dw->setup_mask |= DUPE_MATCH_DIM;
 			dupe_setup_reset(dw);
@@ -1411,14 +1450,14 @@ static gint dupe_check_cb(gpointer data)
 			{
 			if (!dw->setup_point) dw->setup_point = dw->list;
 
-			while(dw->setup_point)
+			while (dw->setup_point)
 				{
 				DupeItem *di = dw->setup_point->data;
 
 				if (!di->simd)
 					{
 					dupe_window_update_progress(dw, _("Reading similarity data..."),
-						dw->setup_count == 0 ? 0.0 : (float)dw->setup_n / dw->setup_count);
+						dw->setup_count == 0 ? 0.0 : (float)dw->setup_n / dw->setup_count, FALSE);
 
 					if (enable_thumb_caching)
 						{
@@ -1452,7 +1491,7 @@ static gint dupe_check_cb(gpointer data)
 			dw->setup_mask |= DUPE_MATCH_SIM_MED;
 			dupe_setup_reset(dw);
 			}
-		dupe_window_update_progress(dw, _("Comparing..."), 0.0);
+		dupe_window_update_progress(dw, _("Comparing..."), 0.0, FALSE);
 		dw->setup_done = TRUE;
 		dupe_setup_reset(dw);
 		dw->setup_count = g_list_length(dw->list);
@@ -1463,11 +1502,11 @@ static gint dupe_check_cb(gpointer data)
 		if (dw->setup_count > 0)
 			{
 			dw->setup_count = 0;
-			dupe_window_update_progress(dw, _("Sorting..."), 1.0);
+			dupe_window_update_progress(dw, _("Sorting..."), 1.0, TRUE);
 			return TRUE;
 			}
 		dw->idle_id = -1;
-		dupe_window_update_progress(dw, NULL, 0.0);
+		dupe_window_update_progress(dw, NULL, 0.0, FALSE);
 
 		dupe_match_rank(dw);
 		dupe_window_update_count(dw, FALSE);
@@ -1483,7 +1522,7 @@ static gint dupe_check_cb(gpointer data)
 		}
 
 	dupe_list_check_match(dw, (DupeItem *)dw->working->data, dw->working);
-	dupe_window_update_progress(dw, _("Comparing..."), dw->setup_count == 0 ? 0.0 : (float) dw->setup_n / dw->setup_count);
+	dupe_window_update_progress(dw, _("Comparing..."), dw->setup_count == 0 ? 0.0 : (float) dw->setup_n / dw->setup_count, FALSE);
 	dw->setup_n++;
 
 	dw->working = dw->working->prev;
@@ -1790,7 +1829,8 @@ static void dupe_display_stats(DupeWindow *dw, DupeItem *di)
 	
 	if (!di) return;
 
-	gd = file_util_gen_dlg("Image thumbprint debug info", NULL, "GQview", "thumbprint", TRUE,
+	gd = file_util_gen_dlg("Image thumbprint debug info", NULL, "GQview", "thumbprint",
+			       dw->window, TRUE,
 			       NULL, NULL);
 	generic_dialog_add_stock(gd, _("Close"), GTK_STOCK_CLOSE, NULL, TRUE);
 
@@ -1802,9 +1842,7 @@ static void dupe_display_stats(DupeWindow *dw, DupeItem *di)
 	buf = g_strdup_printf("%d x %d", di->width, di->height);
 	dupe_display_label(gd->vbox, "dimensions:", buf);
 	g_free(buf);
-	buf = g_strdup_printf("%016ld", di->checksum);
-	dupe_display_label(gd->vbox, "checksum:", buf);
-	g_free(buf);
+	dupe_display_label(gd->vbox, "md5sum:", (di->md5sum) ? di->md5sum : "not generated");
 
 	dupe_display_label(gd->vbox, "thumbprint:", (di->simd) ? "" : "not generated");
 	if (di->simd)
@@ -2050,28 +2088,28 @@ static void dupe_menu_copy_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
-	file_util_copy(NULL, dupe_listview_get_selection(dw, dw->listview), NULL);
+	file_util_copy(NULL, dupe_listview_get_selection(dw, dw->listview), NULL, dw->window);
 }
 
 static void dupe_menu_move_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
-	file_util_move(NULL, dupe_listview_get_selection(dw, dw->listview), NULL);
+	file_util_move(NULL, dupe_listview_get_selection(dw, dw->listview), NULL, dw->window);
 }
 
 static void dupe_menu_rename_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
-	file_util_rename(NULL, dupe_listview_get_selection(dw, dw->listview));
+	file_util_rename(NULL, dupe_listview_get_selection(dw, dw->listview), dw->window);
 }
 
 static void dupe_menu_delete_cb(GtkWidget *widget, gpointer data)
 {
 	DupeWindow *dw = data;
 
-	file_util_delete(NULL, dupe_listview_get_selection(dw, dw->listview));
+	file_util_delete(NULL, dupe_listview_get_selection(dw, dw->listview), dw->window);
 }
 
 static void dupe_menu_remove_cb(GtkWidget *widget, gpointer data)
@@ -2552,7 +2590,7 @@ static void dupe_menu_setup(DupeWindow *dw)
 
 #define CELL_HEIGHT_OVERRIDE 512
                                                                                                                                
-static void cell_renderer_height_override(GtkCellRenderer *renderer)
+void cell_renderer_height_override(GtkCellRenderer *renderer)
 {
 	GParamSpec *spec;
 
@@ -2694,7 +2732,7 @@ static void dupe_window_show_thumb_cb(GtkWidget *widget, gpointer data)
 			gtk_list_store_set(GTK_LIST_STORE(store), &iter, DUPE_COLUMN_THUMB, NULL, -1);
 			valid = gtk_tree_model_iter_next(store, &iter);
 			}
-		dupe_window_update_progress(dw, NULL, 0.0);
+		dupe_window_update_progress(dw, NULL, 0.0, FALSE);
 		}
 
 	dupe_listview_set_height(dw->listview, dw->show_thumbs);
@@ -2806,18 +2844,18 @@ static gint dupe_window_keypress_cb(GtkWidget *widget, GdkEventKey *event, gpoin
 				break;
 			case 'C': case 'c':
 				stop_signal = TRUE;
-				file_util_copy(NULL, dupe_listview_get_selection(dw, listview), NULL);
+				file_util_copy(NULL, dupe_listview_get_selection(dw, listview), NULL, dw->window);
 				break;
 			case 'M': case 'm':
-				file_util_move(NULL, dupe_listview_get_selection(dw, listview), NULL);
+				file_util_move(NULL, dupe_listview_get_selection(dw, listview), NULL, dw->window);
 				stop_signal = TRUE;
 				break;
 			case 'R': case 'r':
-				file_util_rename(NULL, dupe_listview_get_selection(dw, listview));
+				file_util_rename(NULL, dupe_listview_get_selection(dw, listview), dw->window);
 				stop_signal = TRUE;
 				break;
 			case 'D': case 'd':
-				file_util_delete(NULL, dupe_listview_get_selection(dw, listview));
+				file_util_delete(NULL, dupe_listview_get_selection(dw, listview), dw->window);
 				stop_signal = TRUE;
 				break;
 			case 'P': case 'p':
@@ -2857,6 +2895,11 @@ static gint dupe_window_keypress_cb(GtkWidget *widget, GdkEventKey *event, gpoin
 				dupe_window_append_file_list(dw, FALSE);
 				stop_signal = TRUE;
 				break;
+			case 'T': case 't':
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dw->button_thumbs),
+					!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dw->button_thumbs)));
+				stop_signal = TRUE;
+				break;
 			case 'W': case 'w':
 				dupe_window_close(dw);
 				stop_signal = TRUE;
@@ -2949,7 +2992,7 @@ void dupe_window_clear(DupeWindow *dw)
 	dupe_match_reset_list(dw->second_list);
 
 	dupe_window_update_count(dw, FALSE);
-	dupe_window_update_progress(dw, NULL, 0.0);
+	dupe_window_update_progress(dw, NULL, 0.0, FALSE);
 }
 
 void dupe_window_close(DupeWindow *dw)
@@ -3110,16 +3153,16 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 	gtk_box_pack_start(GTK_BOX(status_box), dw->dupe_omenu, FALSE, FALSE, 0);
 	gtk_widget_show(dw->dupe_omenu);
 
-	button = gtk_check_button_new_with_label(_("Thumbnails"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), dw->show_thumbs);
-	g_signal_connect(G_OBJECT(button), "clicked",
+	dw->button_thumbs = gtk_check_button_new_with_label(_("Thumbnails"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dw->button_thumbs), dw->show_thumbs);
+	g_signal_connect(G_OBJECT(dw->button_thumbs), "toggled",
 			 G_CALLBACK(dupe_window_show_thumb_cb), dw);
-	gtk_box_pack_start(GTK_BOX(status_box), button, FALSE, FALSE, 5);
-	gtk_widget_show(button);
+	gtk_box_pack_start(GTK_BOX(status_box), dw->button_thumbs, FALSE, FALSE, 5);
+	gtk_widget_show(dw->button_thumbs);
 
 	button = gtk_check_button_new_with_label(_("Compare two file sets"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), dw->second_set);
-	g_signal_connect(G_OBJECT(button), "clicked",
+	g_signal_connect(G_OBJECT(button), "toggled",
 			 G_CALLBACK(dupe_second_set_toggle_cb), dw);
 	gtk_box_pack_end(GTK_BOX(status_box), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
@@ -3159,7 +3202,7 @@ DupeWindow *dupe_window_new(DupeMatchType match_mask)
 	gtk_widget_show(dw->window);
 
 	dupe_window_update_count(dw, TRUE);
-	dupe_window_update_progress(dw, NULL, 0.0);
+	dupe_window_update_progress(dw, NULL, 0.0, FALSE);
 
 	dupe_window_list = g_list_append(dupe_window_list, dw);
 
@@ -3241,11 +3284,11 @@ static GtkWidget *dupe_confirm_dir_list(DupeWindow *dw, GList *list)
 	g_signal_connect(G_OBJECT(menu), "destroy",
 			 G_CALLBACK(confirm_dir_list_destroy), d);
 
-	menu_item_add_stock(menu, _("Dropped list includes directories."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
+	menu_item_add_stock(menu, _("Dropped list includes folders."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("_Add contents"), GTK_STOCK_OK, G_CALLBACK(confirm_dir_list_add), d);
 	menu_item_add_stock(menu, _("Add contents _recursive"), GTK_STOCK_ADD, G_CALLBACK(confirm_dir_list_recurse), d);
-	menu_item_add_stock(menu, _("_Skip directories"), GTK_STOCK_REMOVE, G_CALLBACK(confirm_dir_list_skip), d);
+	menu_item_add_stock(menu, _("_Skip folders"), GTK_STOCK_REMOVE, G_CALLBACK(confirm_dir_list_skip), d);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("Cancel"), GTK_STOCK_CANCEL, G_CALLBACK(confirm_dir_list_cancel), d);
 
