@@ -45,6 +45,7 @@ struct _DupeItem
 	time_t date;
 
 	long checksum;
+	gchar *md5sum;
 	gint width;
 	gint height;
 
@@ -79,6 +80,7 @@ struct _DupeWindow
 	GtkWidget *dupe_omenu;
 	GtkWidget *status_label;
 	GtkWidget *extra_label;
+	GtkWidget *button_thumbs;
 
 	gint show_thumbs;
 
@@ -89,8 +91,8 @@ struct _DupeWindow
 	gint setup_n;			/* these are merely for speed optimization */
 	GList *setup_point;		/* ditto */
 	DupeMatchType setup_mask;	/* ditto */
-	guint32 setup_time;
-	guint32 setup_time_count;
+	guint64 setup_time;
+	guint64 setup_time_count;
 
 	DupeItem *click_item;		/* for popup menu */
 
@@ -125,6 +127,10 @@ void dupe_maint_removed(const gchar *path);
 void dupe_maint_renamed(const gchar *source, const gchar *dest);
 
 
+/* cell max with/height hack utility */
+void cell_renderer_height_override(GtkCellRenderer *renderer);
+
+
 #endif
 
 
