@@ -129,8 +129,9 @@ static EditorVerboseData *editor_verbose_window(const gchar *template, const gch
 	vd->fd = -1;
 
 	buf = g_strdup_printf(_("Output of %s"), text);
-	vd->gd = file_util_gen_dlg(_("Edit command results"), buf, "GQview", "editor_results", FALSE,
-			       NULL, vd);
+	vd->gd = file_util_gen_dlg(_("Edit command results"), buf, "GQview", "editor_results",
+				   NULL, FALSE,
+				   NULL, vd);
 	g_free(buf);
 	vd->button_stop = generic_dialog_add_stock(vd->gd, NULL, GTK_STOCK_STOP,
 						   editor_verbose_window_stop, FALSE);
