@@ -18,8 +18,11 @@
 #include "ui_menu.h"
 
 
-/* see fullscreen.h for possible values and their meaning */
-#define FULLSCREEN_SCREEN 0
+enum {
+	FULLSCREEN_CURSOR_HIDDEN = 1 << 0,
+	FULLSCREEN_CURSOR_NORMAL = 1 << 1,
+	FULLSCREEN_CURSOR_BUSY   = 1 << 2
+};
 
 
 /*
@@ -28,9 +31,23 @@
  *----------------------------------------------------------------------------
  */
 
-static void clear_mouse_cursor(GtkWidget *widget, gint clear)
+static void clear_mouse_cursor(GtkWidget *widget, gint state)
 {
-	if (clear)
+	if (!widget->window) return;
+
+	if (state & FULLSCREEN_CURSOR_BUSY)
+		{
+		GdkCursor *cursor;
+
+		cursor = gdk_cursor_new(GDK_WATCH);
+		gdk_window_set_cursor (widget->window, cursor);
+		gdk_cursor_unref(cursor);
+		}
+	else if (state & FULLSCREEN_CURSOR_NORMAL)
+		{
+		gdk_window_set_cursor (widget->window, NULL);
+		}
+	else
 		{
 		GdkCursor *cursor;
 		GdkPixmap *p;
@@ -47,10 +64,6 @@ static void clear_mouse_cursor(GtkWidget *widget, gint clear)
 		gdk_cursor_unref(cursor);
 		g_object_unref(p);
 		}
-	else
-		{
-		gdk_window_set_cursor (widget->window, NULL);
-		}
 }
 
 static gint fullscreen_hide_mouse_cb(gpointer data)
@@ -59,7 +72,8 @@ static gint fullscreen_hide_mouse_cb(gpointer data)
 
 	if (fs->hide_mouse_id == -1) return FALSE;
 
-	clear_mouse_cursor(fs->window, TRUE);
+	fs->cursor_state &= ~FULLSCREEN_CURSOR_NORMAL;
+	if (!(fs->cursor_state & FULLSCREEN_CURSOR_BUSY)) clear_mouse_cursor(fs->window, fs->cursor_state);
 
 	fs->hide_mouse_id = -1;
 	return FALSE;
@@ -84,12 +98,136 @@ static gint fullscreen_mouse_moved(GtkWidget *widget, GdkEventButton *bevent, gp
 {
 	FullScreenData *fs = data;
 
-	clear_mouse_cursor(fs->window, FALSE);
+	if (!(fs->cursor_state & FULLSCREEN_CURSOR_NORMAL))
+		{
+		fs->cursor_state |= FULLSCREEN_CURSOR_NORMAL;
+		if (!(fs->cursor_state & FULLSCREEN_CURSOR_BUSY)) clear_mouse_cursor(fs->window, fs->cursor_state);
+		}
 	fullscreen_hide_mouse_reset(fs);
 
 	return FALSE;
 }
 
+static void fullscreen_busy_mouse_disable(FullScreenData *fs)
+{
+	if (fs->busy_mouse_id != -1)
+		{
+		gtk_timeout_remove(fs->busy_mouse_id);
+		fs->busy_mouse_id = -1;
+		}
+}
+
+static void fullscreen_mouse_set_busy(FullScreenData *fs, gint busy)
+{
+	fullscreen_busy_mouse_disable(fs);
+
+	if ((fs->cursor_state & FULLSCREEN_CURSOR_BUSY) == (busy)) return;
+
+	if (busy)
+		{
+		fs->cursor_state |= FULLSCREEN_CURSOR_BUSY;
+		}
+	else
+		{
+		fs->cursor_state &= ~FULLSCREEN_CURSOR_BUSY;
+		}
+
+	clear_mouse_cursor(fs->window, fs->cursor_state);
+}
+
+static gboolean fullscreen_mouse_set_busy_cb(gpointer data)
+{
+	FullScreenData *fs = data;
+
+	fs->busy_mouse_id = -1;
+	fullscreen_mouse_set_busy(fs, TRUE);
+	return FALSE;
+}
+
+static void fullscreen_mouse_set_busy_idle(FullScreenData *fs)
+{
+	if (fs->busy_mouse_id == -1)
+		{
+		fs->busy_mouse_id = gtk_timeout_add(FULL_SCREEN_BUSY_MOUSE_DELAY,
+						    fullscreen_mouse_set_busy_cb, fs);
+		}
+}
+
+static void fullscreen_image_update_cb(ImageWindow *imd, gpointer data)
+{
+	FullScreenData *fs = data;
+
+	if (fs->imd->il &&
+	    fs->imd->pixbuf != fs->imd->il->pixbuf)
+		{
+		fullscreen_mouse_set_busy_idle(fs);
+		}
+}
+
+static void fullscreen_image_complete_cb(ImageWindow *imd, gint preload, gpointer data)
+{
+	FullScreenData *fs = data;
+
+	if (!preload) fullscreen_mouse_set_busy(fs, FALSE);
+}
+
+#define XSCREENSAVER_BINARY	"xscreensaver-command"
+#define XSCREENSAVER_COMMAND	"xscreensaver-command -deactivate >&- 2>&- &"
+
+static void fullscreen_saver_deactivate(void)
+{
+	static gint checked = FALSE;
+	static gint found = FALSE;
+
+	if (!checked)
+		{
+		checked = TRUE;
+		found = file_in_path(XSCREENSAVER_BINARY);
+		}
+
+	if (found)
+		{
+		system (XSCREENSAVER_COMMAND);
+		}
+}
+
+static gboolean fullscreen_saver_block_cb(gpointer data)
+{
+	if (fullscreen_disable_saver)
+		{
+		fullscreen_saver_deactivate();
+		}
+
+	return TRUE;
+}
+
+#if 0
+static gint fullscreen_map_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
+{
+	GdkRectangle *rect = data;
+	gint ox, oy;
+
+	gdk_window_get_origin(widget->window, &ox, &oy);
+	if (rect->x != ox || rect->y != oy)
+		{
+		/* If we get to here, then the window manager is not listening to us,
+		 * we want no borders and no titles (example twm)... or at least it should be
+		 * honoring the STATIC gravity, but it isn't ...To work around a deaf
+		 * window manager we move the window where we want it once it is mapped.
+		 */
+
+		if (debug) printf("window manager is not listening to our request for no decorations!\n"
+				  "requested position: %d,%d actual: %d,%d\n", rect->x, rect->y, ox, oy);
+
+		gtk_window_move(GTK_WINDOW(widget), rect->x, rect->y);
+		}
+
+	g_signal_handlers_disconnect_by_func(widget, fullscreen_map_cb, data);
+
+	return FALSE;
+}
+#endif
+
 static gint fullscreen_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
 {
 	FullScreenData *fs = data;
@@ -106,35 +244,86 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 	gint same;
 	gint x, y;
 	gint w, h;
+	GdkGeometry geometry;
 
 	if (!window || !imd) return NULL;
 
 	fs = g_new0(FullScreenData, 1);
 
 	fs->hide_mouse_id = -1;
+	fs->busy_mouse_id = -1;
+	fs->cursor_state = FULLSCREEN_CURSOR_HIDDEN;
+
 	fs->normal_window = window;
 	fs->normal_imd = imd;
 
 	fs->stop_func = stop_func;
 	fs->stop_data = stop_data;
 
-	if (debug) printf("full screen requests screen %d\n", FULLSCREEN_SCREEN);
-	fullscreen_prefs_get_geometry(FULLSCREEN_SCREEN, window, &x, &y, &w, &h,
+	if (debug) printf("full screen requests screen %d\n", fullscreen_screen);
+	fullscreen_prefs_get_geometry(fullscreen_screen, window, &x, &y, &w, &h,
 				      &screen, &same);
 
 	fs->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+
+#if 0
+	/* This is not what we want, it does not allow setting window position and size.
+	 * GQview needs control of which monitor(s) to use for full screen.
+	 */
 	gtk_window_fullscreen(GTK_WINDOW(fs->window));
+#endif
+
+	/* this requests no decorations, if you still have them complain to the window manager author(s) */
 	gtk_window_set_decorated(GTK_WINDOW(fs->window), FALSE);
 
+	gtk_window_set_resizable(GTK_WINDOW(fs->window), FALSE);
+
 	gtk_window_set_screen(GTK_WINDOW(fs->window), screen);
 	gtk_container_set_border_width(GTK_CONTAINER(fs->window), 0);
 	g_signal_connect(G_OBJECT(fs->window), "delete_event",
 			 G_CALLBACK(fullscreen_delete_cb), fs);
 
 	gtk_window_set_title(GTK_WINDOW(fs->window), _("GQview full screen"));
+
+	geometry.min_width = w;
+	geometry.min_height = h;
+	geometry.max_width = w;
+	geometry.max_height = h;
+	geometry.base_width = w;
+	geometry.base_height = h;
+	geometry.win_gravity = GDK_GRAVITY_STATIC;
+	/* By setting USER_POS and USER_SIZE, most window managers will
+	 * not request positioning of the full screen window (for example twm).
+	 *
+	 * In addition, setting gravity to STATIC will result in the
+	 * decorations of twm to not effect the requested window position,
+	 * the decorations will simply be off screen, except in multi monitor setups :-/
+	 */
+	gtk_window_set_geometry_hints(GTK_WINDOW(fs->window), fs->window, &geometry,
+				      GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_BASE_SIZE |
+				      GDK_HINT_WIN_GRAVITY |
+				      GDK_HINT_USER_POS | GDK_HINT_USER_SIZE);
+
 	gtk_window_set_default_size(GTK_WINDOW(fs->window), w, h);
 	gtk_window_move(GTK_WINDOW(fs->window), x, y);
 
+#if 0
+        /* on_map decorations hack check */
+	if (TRUE)
+		{
+		GdkRectangle *rect;
+
+		rect = g_new0(GdkRectangle, 1);
+		rect->x = x;
+		rect->y = y;
+		rect->width = w;
+		rect->height = h;
+		g_signal_connect_data(G_OBJECT(fs->window), "map_event",
+				      G_CALLBACK(fullscreen_map_cb), rect,
+				      (GClosureNotify)g_free, G_CONNECT_AFTER);
+		}
+#endif
+
 	fs->imd = image_new(FALSE);
 
 	gtk_container_add(GTK_CONTAINER(fs->window), fs->imd->widget);
@@ -145,8 +334,15 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 		image_background_set_black(fs->imd, TRUE);
 		}
 
+	image_set_delay_flip(fs->imd, fullscreen_clean_flip);
 	image_auto_refresh(fs->imd, fs->normal_imd->auto_refresh_interval);
 
+	if (fullscreen_clean_flip)
+		{
+		image_set_update_func(fs->imd, fullscreen_image_update_cb, fs);
+		image_set_complete_func(fs->imd, fullscreen_image_complete_cb, fs);
+		}
+
 	gtk_widget_show(fs->imd->widget);
 
 	image_change_from_image(fs->imd, fs->normal_imd);
@@ -156,10 +352,10 @@ FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 	/* for hiding the mouse */
 	g_signal_connect(G_OBJECT(fs->imd->image), "motion_notify_event",
 			   G_CALLBACK(fullscreen_mouse_moved), fs);
-	clear_mouse_cursor(fs->window, TRUE);
-#if 0
-	fullscreen_hide_mouse_reset(fs);
-#endif
+	clear_mouse_cursor(fs->window, fs->cursor_state);
+
+	/* set timer to block screen saver */
+	fs->saver_block_id = gtk_timeout_add(60 * 1000, fullscreen_saver_block_cb, fs);
 
 	/* hide normal window
 	 * FIXME: properly restore this window on show
@@ -176,7 +372,10 @@ void fullscreen_stop(FullScreenData *fs)
 {
 	if (!fs) return;
 
+	gtk_timeout_remove(fs->saver_block_id);
+
 	fullscreen_hide_mouse_disable(fs);
+	fullscreen_busy_mouse_disable(fs);
 	gdk_keyboard_ungrab(GDK_CURRENT_TIME);
 
 	image_change_from_image(fs->normal_imd, fs->imd);
