@@ -11,6 +11,10 @@
 
 
 #include "gqview.h"
+#include "globals.h"
+
+#include "icons_inline.h"
+
 #include "logo.xpm"
 #include "icons/img_unknown.xpm"
 
@@ -22,6 +26,7 @@ gint main_window_w = 500;
 gint main_window_h = 400;
 gint main_window_x = 0;
 gint main_window_y = 0;
+gint main_window_maximized = FALSE;
 
 gint float_window_w = 260;
 gint float_window_h = 450;
@@ -48,6 +53,7 @@ gint safe_delete_size = 20;
 gint restore_tool = FALSE;
 gint zoom_mode = ZOOM_RESET_ORIGINAL;
 gint two_pass_zoom = TRUE;
+gint scroll_reset_method = SCROLL_RESET_TOPLEFT;
 gint fit_window = FALSE;
 gint limit_window_size = FALSE;
 gint zoom_to_fit_expands = TRUE;
@@ -66,7 +72,7 @@ gint thumbnails_enabled = FALSE;
 SortType file_sort_method = SORT_NAME;
 gint file_sort_ascending = TRUE;
 
-gint slideshow_delay = 15;
+gint slideshow_delay = 150;
 gint slideshow_random = FALSE;
 gint slideshow_repeat = FALSE;
 
@@ -90,6 +96,10 @@ gint place_dialogs_under_mouse = FALSE;
 
 gint black_window_background = FALSE;
 
+gint fullscreen_screen = 1;
+gint fullscreen_clean_flip = FALSE;
+gint fullscreen_disable_saver = TRUE;
+
 gint dupe_custom_threshold = 99;
 
 gint debug = FALSE;
