@@ -62,9 +62,8 @@
  */
 
 #define GQVIEW_RC_DIR             ".gqview"
-#define GQVIEW_RC_DIR_THUMBS      ".gqview/thumbnails"
-#define GQVIEW_RC_DIR_COLLECTIONS ".gqview/collections"
-#define GQVIEW_RC_DIR_TRASH       ".gqview/trash"
+#define GQVIEW_RC_DIR_COLLECTIONS GQVIEW_RC_DIR"/collections"
+#define GQVIEW_RC_DIR_TRASH       GQVIEW_RC_DIR"/trash"
 
 #define RC_FILE_NAME "gqviewrc"
 
@@ -72,6 +71,10 @@
 #define ZOOM_RESET_FIT_WINDOW 1
 #define ZOOM_RESET_NONE 2
 
+#define SCROLL_RESET_TOPLEFT 0
+#define SCROLL_RESET_CENTER 1
+#define SCROLL_RESET_NOCHANGE 2
+
 #define MOUSEWHEEL_SCROLL_SIZE 20
 
 #define GQVIEW_EDITOR_SLOTS 10
@@ -94,6 +97,7 @@ extern gint main_window_w;
 extern gint main_window_h;
 extern gint main_window_x;
 extern gint main_window_y;
+extern gint main_window_maximized;
 
 extern gint float_window_w;
 extern gint float_window_h;
@@ -120,6 +124,7 @@ extern gint safe_delete_size;
 extern gint restore_tool;
 extern gint zoom_mode;
 extern gint two_pass_zoom;
+extern gint scroll_reset_method;
 extern gint fit_window;
 extern gint limit_window_size;
 extern gint zoom_to_fit_expands;
@@ -138,7 +143,7 @@ extern gint thumbnails_enabled;
 extern SortType file_sort_method;
 extern gint file_sort_ascending;
 
-extern gint slideshow_delay;	/* in seconds */
+extern gint slideshow_delay;	/* in tenths of a second */
 extern gint slideshow_random;
 extern gint slideshow_repeat;
 
@@ -147,6 +152,10 @@ extern gint enable_in_place_rename;
 
 extern gint black_window_background;
 
+extern gint fullscreen_screen;
+extern gint fullscreen_clean_flip;
+extern gint fullscreen_disable_saver;
+
 extern gint dupe_custom_threshold;
 
 extern gint debug;
@@ -166,10 +175,6 @@ extern gint enable_read_ahead;
 
 extern gint place_dialogs_under_mouse;
 
-/* logo & images */
-extern const char **logo;
-extern const char **img_unknown;
-
 /* layout */
 extern gchar *layout_order;
 extern gint layout_style;
@@ -196,6 +201,7 @@ extern gint exif_rotate_enable;
  */
 
 void window_set_icon(GtkWidget *window, const char **icon, const gchar *file);
+gint window_maximized(GtkWidget *window);
 
 gfloat get_zoom_increment(void);
 
