@@ -300,6 +300,27 @@ GdkPixbuf *image_loader_get_pixbuf(ImageLoader *il)
 	return il->pixbuf;
 }
 
+gchar *image_loader_get_format(ImageLoader *il)
+{
+	GdkPixbufFormat *format;
+	gchar **mimev;
+	gchar *mime;
+
+	if (!il || !il->loader) return NULL;
+
+	format = gdk_pixbuf_loader_get_format(il->loader);
+	if (!format) return NULL;
+
+	mimev = gdk_pixbuf_format_get_mime_types(format);
+	if (!mimev) return NULL;
+
+	/* return first member of mimev, as GdkPixbufLoader has no way to tell us which exact one ? */
+	mime = g_strdup(mimev[0]);
+	g_strfreev(mimev);
+
+	return mime;
+}
+
 void image_loader_set_area_ready_func(ImageLoader *il,
 				      void (*func_area_ready)(ImageLoader *, guint, guint, guint, guint, gpointer),
 				      gpointer data_area_ready)
