@@ -14,6 +14,7 @@
 #include "image.h"
 
 
+#include "globals.h"
 #include "image-load.h"
 #include "collect.h"
 #include "exif.h"
@@ -104,6 +105,21 @@ struct _CacheData
 	guint size;
 };
 
+typedef struct _OverlayData OverlayData;
+struct _OverlayData
+{
+	gint id;
+
+	GdkPixbuf *pixbuf;
+
+	gint x;
+	gint y;
+	gint relative;	/* x,y coordinates are relative, negative values start bottom right */
+
+	gint visible;
+	gint always;	/* hide temporarily when scrolling */
+};
+
 
 static void image_queue_clear(ImageWindow *imd);
 
@@ -113,6 +129,33 @@ static void image_complete_util(ImageWindow *imd, gint preload);
 
 static void image_button_do(ImageWindow *imd, GdkEventButton *bevent);
 
+static void image_overlay_draw(ImageWindow *imd, gint x, gint y, gint w, gint h);
+static void image_overlay_queue_all(ImageWindow *imd);
+
+static void image_scroller_timer_set(ImageWindow *imd, gint start);
+
+
+static gint util_clip_region(gint x, gint y, gint w, gint h,
+			     gint clip_x, gint clip_y, gint clip_w, gint clip_h,
+			     gint *rx, gint *ry, gint *rw, gint *rh)
+{
+	if (clip_x + clip_w <= x ||
+	    clip_x >= x + w ||
+	    clip_y + clip_h <= y ||
+	    clip_y >= y + h)
+		{
+		return FALSE;
+		}
+
+	*rx = MAX(x, clip_x);
+	*rw = MIN((x + w), (clip_x + clip_w)) - *rx;
+
+	*ry = MAX(y, clip_y);
+	*rh = MIN((y + h), (clip_y + clip_h)) - *ry;
+
+	return TRUE;
+}
+
 
 /*
  *-------------------------------------------------------------------
@@ -464,6 +507,13 @@ static void image_tile_expose(ImageWindow *imd, ImageTile *it,
 	gdk_draw_drawable(imd->image->window, imd->image->style->fg_gc[GTK_WIDGET_STATE(imd->image)],
 			  it->pixmap, x, y,
 			  imd->x_offset + (it->x - imd->x_scroll) + x, imd->y_offset + (it->y - imd->y_scroll) + y, w, h);
+
+	if (imd->overlay_list)
+		{
+		image_overlay_draw(imd, imd->x_offset + (it->x - imd->x_scroll) + x,
+					imd->y_offset + (it->y - imd->y_scroll) + y,
+					w, h);
+		}
 }
 
 static gint image_tile_is_visible(ImageWindow *imd, ImageTile *it)
@@ -767,6 +817,7 @@ static gint image_top_window_sizable(ImageWindow *imd)
 	if (!fit_window) return FALSE;
 	if (!imd->top_window_sync) return FALSE;
 	if (!imd->eventbox->window) return FALSE;
+	if (window_maximized(imd->top_window)) return FALSE;
 
 	return TRUE;
 }
@@ -803,42 +854,61 @@ static void image_redraw(ImageWindow *imd, gint new_data)
 	image_queue(imd, 0, 0, imd->width, imd->height, TRUE, TILE_RENDER_ALL, new_data);
 }
 
-static void image_border_clear(ImageWindow *imd)
+static void image_border_draw(ImageWindow *imd, gint x, gint y, gint w, gint h)
 {
+	gint rx, ry, rw, rh;
+
 	if (!imd->image->window) return;
 
 	if (imd->vis_width < imd->window_width)
 		{
-		if (imd->x_offset > 0)
+		if (imd->x_offset > 0 &&
+		    util_clip_region(x, y, w, h,
+			0, 0,
+			imd->x_offset, imd->window_height,
+			&rx, &ry, &rw, &rh))
 			{
-			gdk_window_clear_area(imd->image->window,
-					      0, 0,
-					      imd->x_offset, imd->window_height);
+			gdk_window_clear_area(imd->image->window, rx, ry, rw, rh);
+			image_overlay_draw(imd, rx, ry, rw, rh);
 			}
-		if (imd->window_width - imd->vis_width - imd->x_offset > 0)
+		if (imd->window_width - imd->vis_width - imd->x_offset > 0 &&
+		    util_clip_region(x, y, w, h,
+			imd->x_offset + imd->vis_width, 0,
+			imd->window_width - imd->vis_width - imd->x_offset, imd->window_height,
+			&rx, &ry, &rw, &rh))
 			{
-			gdk_window_clear_area(imd->image->window,
-					      imd->vis_width + imd->x_offset, 0,
-					      imd->window_width - imd->vis_width - imd->x_offset, imd->window_height);
+			gdk_window_clear_area(imd->image->window, rx, ry, rw, rh);
+			image_overlay_draw(imd, rx, ry, rw, rh);
 			}
 		}
 	if (imd->vis_height < imd->window_height)
 		{
-		if (imd->y_offset > 0)
+		if (imd->y_offset > 0 &&
+		    util_clip_region(x, y, w, h,
+			imd->x_offset, 0,
+			imd->vis_width, imd->y_offset,
+			&rx, &ry, &rw, &rh))
 			{
-			gdk_window_clear_area(imd->image->window,
-					      imd->x_offset, 0,
-					      imd->vis_width, imd->y_offset);
+			gdk_window_clear_area(imd->image->window, rx, ry, rw, rh);
+			image_overlay_draw(imd, rx, ry, rw, rh);
 			}
-		if (imd->window_height - imd->vis_height - imd->y_offset > 0)
+		if (imd->window_height - imd->vis_height - imd->y_offset > 0 &&
+		    util_clip_region(x, y, w, h,
+			imd->x_offset, imd->y_offset + imd->vis_height,
+			imd->vis_width, imd->window_height - imd->vis_height - imd->y_offset,
+			&rx, &ry, &rw, &rh))
 			{
-			gdk_window_clear_area(imd->image->window,
-					      imd->x_offset, imd->vis_height + imd->y_offset,
-					      imd->vis_width, imd->window_height - imd->vis_height - imd->y_offset);
+			gdk_window_clear_area(imd->image->window, rx, ry, rw, rh);
+			image_overlay_draw(imd, rx, ry, rw, rh);
 			}
 		}
 }
 
+static void image_border_clear(ImageWindow *imd)
+{
+	image_border_draw(imd, 0, 0, imd->window_width, imd->window_height);
+}
+
 static gint image_scroll_clamp(ImageWindow *imd)
 {
 	gint old_xs;
@@ -1007,6 +1077,21 @@ static void image_size_sync(ImageWindow *imd, gint new_width, gint new_height)
 
 	gtk_widget_set_size_request(imd->image, imd->window_width, imd->window_height);
 
+	/* ensure scroller remains visible */
+	if (imd->scroller_id != -1)
+		{
+		if (imd->scroller_x > new_width)
+			{
+			imd->scroller_x = new_width;
+			imd->scroller_xpos = new_width;
+			}
+		if (imd->scroller_y > new_height)
+			{
+			imd->scroller_y = new_height;
+			imd->scroller_ypos = new_height;
+			}
+		}
+
 	/* clear any borders */
 	image_border_clear(imd);
 	
@@ -1071,6 +1156,8 @@ static void image_update_util(ImageWindow *imd)
 
 static void image_complete_util(ImageWindow *imd, gint preload)
 {
+	if (imd->il && imd->pixbuf != image_loader_get_pixbuf(imd->il)) return;
+
 	if (debug) printf("image load completed \"%s\" (%s)\n",
 			  (preload) ? imd->read_ahead_path : imd->image_path,
 			  (preload) ? "preload" : "current");
@@ -1079,6 +1166,11 @@ static void image_complete_util(ImageWindow *imd, gint preload)
 	if (imd->func_complete) imd->func_complete(imd, preload, imd->data_complete);
 }
 
+static void image_new_util(ImageWindow *imd)
+{
+	if (imd->func_new) imd->func_new(imd, imd->data_new);
+}
+
 static void image_scroll_real(ImageWindow *imd, gint x, gint y)
 {
 	gint old_x, old_y;
@@ -1094,6 +1186,19 @@ static void image_scroll_real(ImageWindow *imd, gint x, gint y)
 	image_scroll_clamp(imd);
 	if (imd->x_scroll == old_x && imd->y_scroll == old_y) return;
 
+	if (imd->overlay_list)
+		{
+		gint new_x, new_y;
+
+		new_x = imd->x_scroll;
+		new_y = imd->y_scroll;
+		imd->x_scroll = old_x;
+		imd->y_scroll = old_y;
+		image_overlay_queue_all(imd);
+		imd->x_scroll = new_x;
+		imd->y_scroll = new_y;
+		}
+
 	x_off = imd->x_scroll - old_x;
 	y_off = imd->y_scroll - old_y;
 
@@ -1142,6 +1247,11 @@ static void image_scroll_real(ImageWindow *imd, gint x, gint y)
 				  x1 + imd->x_offset, y1 + imd->y_offset, w, h);
 		g_object_unref(gc);
 
+		if (imd->overlay_list)
+			{
+			image_overlay_queue_all(imd);
+			}
+
 		w = imd->vis_width - w;
 		h = imd->vis_height - h;
 
@@ -1222,7 +1332,28 @@ static void image_zoom_sync(ImageWindow *imd, gfloat zoom,
 		sized = image_size_top_window(imd, imd->width, imd->height);
 		}
 
-	if (!force)
+	if (force)
+		{
+		/* force means new image, so update scroll offset per options */
+		switch (scroll_reset_method)
+			{
+			case SCROLL_RESET_NOCHANGE:
+				/* maintain old scroll position, do nothing */
+				break;
+			case SCROLL_RESET_CENTER:
+				/* center new image */
+				imd->x_scroll = ((double)imd->image_width / 2.0 * imd->scale) - imd->vis_width / 2;
+				imd->y_scroll = ((double)imd->image_height / 2.0 * imd->scale) - imd->vis_height / 2;
+				break;
+			case SCROLL_RESET_TOPLEFT:
+			default:
+				/* reset to upper left */
+				imd->x_scroll = 0;
+				imd->y_scroll = 0;
+				break;
+			}
+		}
+	else
 		{
 		/* user zoom does not force, so keep visible center point */
 		if (center_point)
@@ -1259,7 +1390,11 @@ static void image_pixbuf_sync(ImageWindow *imd, gfloat zoom, gint blank, gint ne
 		imd->image_height = 0;
 		imd->scale = 1.0;
 
-		if (imd->image->window) gdk_window_clear(imd->image->window);
+		if (imd->image->window)
+			{
+			gdk_window_clear(imd->image->window);
+			image_overlay_draw(imd, 0, 0, imd->window_width, imd->window_height);
+			}
 
 		image_update_util(imd);
 		
@@ -1269,9 +1404,11 @@ static void image_pixbuf_sync(ImageWindow *imd, gfloat zoom, gint blank, gint ne
 	imd->image_width = gdk_pixbuf_get_width(imd->pixbuf);
 	imd->image_height = gdk_pixbuf_get_height(imd->pixbuf);
 
+#if 0
 	/* reset scrolling */
 	imd->x_scroll = 0;
 	imd->y_scroll = 0;
+#endif
 
 	image_zoom_sync(imd, zoom, TRUE, blank, new, FALSE, 0, 0);
 }
@@ -1526,9 +1663,10 @@ static gint image_post_buffer_get(ImageWindow *imd)
 {
 	gint success;
 
-	if (!imd->pixbuf && imd->prev_pixbuf &&
+	if (imd->prev_pixbuf &&
 	    imd->image_path && imd->prev_path && strcmp(imd->image_path, imd->prev_path) == 0)
 		{
+		if (imd->pixbuf) g_object_unref(imd->pixbuf);
 		imd->pixbuf = imd->prev_pixbuf;
 		success = TRUE;
 		}
@@ -1567,6 +1705,12 @@ static void image_load_area_cb(ImageLoader *il, guint x, guint y, guint w, guint
 {
 	ImageWindow *imd = data;
 
+	if (imd->delay_flip &&
+	    imd->pixbuf != image_loader_get_pixbuf(il))
+		{
+		return;
+		}
+
 	if (!imd->pixbuf) image_load_pixbuf_ready(imd);
 
 	if (imd->scale != 1.0)
@@ -1600,6 +1744,15 @@ static void image_load_done_cb(ImageLoader *il, gpointer data)
 
 	if (debug) printf ("image done\n");
 
+	if (imd->delay_flip &&
+	    imd->pixbuf != image_loader_get_pixbuf(imd->il))
+		{
+		if (imd->pixbuf) g_object_unref(imd->pixbuf);
+		imd->pixbuf = image_loader_get_pixbuf(imd->il);
+		if (imd->pixbuf) g_object_ref(imd->pixbuf);
+		image_pixbuf_sync(imd, imd->zoom, FALSE, TRUE);
+		}
+
 	image_loader_free(imd->il);
 	imd->il = NULL;
 
@@ -1637,7 +1790,7 @@ static void image_load_buffer_throttle(ImageLoader *il)
 static gint image_read_ahead_check(ImageWindow *imd)
 {
 	if (!imd->read_ahead_path) return FALSE;
-	if (imd->il || imd->pixbuf) return FALSE;
+	if (imd->il) return FALSE;
 
 	if (!imd->image_path || strcmp(imd->read_ahead_path, imd->image_path) != 0)
 		{
@@ -1662,14 +1815,19 @@ static gint image_read_ahead_check(ImageWindow *imd)
 		/* do this one directly (probably should add a set func) */
 		imd->il->func_done = image_load_done_cb;
 
-		imd->pixbuf = image_loader_get_pixbuf(imd->il);
-		if (imd->pixbuf) g_object_ref(imd->pixbuf);
+		if (!imd->delay_flip)
+			{
+			if (imd->pixbuf) g_object_unref(imd->pixbuf);
+			imd->pixbuf = image_loader_get_pixbuf(imd->il);
+			if (imd->pixbuf) g_object_ref(imd->pixbuf);
+			}
 
 		image_read_ahead_cancel(imd);
 		return TRUE;
 		}
 	else if (imd->read_ahead_pixbuf)
 		{
+		if (imd->pixbuf) g_object_unref(imd->pixbuf);
 		imd->pixbuf = imd->read_ahead_pixbuf;
 		imd->read_ahead_pixbuf = NULL;
 
@@ -1703,10 +1861,16 @@ static gint image_load_begin(ImageWindow *imd, const gchar *path)
 		{
 		if (debug) printf("from read ahead buffer: %s\n", imd->image_path);
 
-		image_pixbuf_sync(imd, imd->zoom, FALSE, TRUE);
+		if (!imd->delay_flip || !imd->il) image_pixbuf_sync(imd, imd->zoom, FALSE, TRUE);
 		return TRUE;
 		}
 
+	if (!imd->delay_flip && imd->pixbuf)
+		{
+		g_object_unref(imd->pixbuf);
+		imd->pixbuf = NULL;
+		}
+
 	imd->il = image_loader_new(path);
 
 	image_loader_set_area_ready_func(imd->il, image_load_area_cb, imd);
@@ -1729,7 +1893,7 @@ static gint image_load_begin(ImageWindow *imd, const gchar *path)
 	image_load_buffer_throttle(imd->il);
 #endif
 
-	if (!imd->pixbuf) image_load_pixbuf_ready(imd);
+	if (!imd->delay_flip && !imd->pixbuf) image_load_pixbuf_ready(imd);
 
 	return TRUE;
 }
@@ -1757,23 +1921,20 @@ static void image_change_complete(ImageWindow *imd, gfloat zoom, gint new)
 {
 	gint sync = TRUE;
 
-	if (imd->pixbuf) g_object_unref(imd->pixbuf);
-	imd->pixbuf = NULL;
-
 	imd->zoom = zoom;	/* store the zoom, needed by the loader */
 
 	image_reset(imd);
 
 	if (imd->image_path && isfile(imd->image_path))
 		{
-		image_load_begin(imd, imd->image_path);
-		if (imd->pixbuf)
+		if (image_load_begin(imd, imd->image_path))
 			{
 			imd->unknown = FALSE;
 			sync = FALSE;
 			}
 		else
 			{
+			if (imd->pixbuf) g_object_unref(imd->pixbuf);
 			imd->pixbuf = gdk_pixbuf_new_from_xpm_data(img_unknown);
 			imd->unknown = TRUE;
 			}
@@ -1782,6 +1943,9 @@ static void image_change_complete(ImageWindow *imd, gfloat zoom, gint new)
 		}
 	else
 		{
+		if (imd->pixbuf) g_object_unref(imd->pixbuf);
+		imd->pixbuf = NULL;
+
 		if (imd->image_path)
 			{
 			imd->pixbuf = gdk_pixbuf_new_from_xpm_data(img_unknown);
@@ -1815,9 +1979,6 @@ static void image_change_real(ImageWindow *imd, const gchar *path,
 
 	imd->collection = cd;
 	imd->collection_info = info;
-#if 0
-	if (!path) zoom = 1.0;
-#endif
 
 	if (enable_read_ahead && imd->image_path && imd->pixbuf)
 		{
@@ -1834,7 +1995,10 @@ static void image_change_real(ImageWindow *imd, const gchar *path,
 			}
 		}
 
-	image_set_path(imd, path);
+	g_free(imd->image_path);
+	imd->image_path = g_strdup(path);
+	imd->image_name = filename_from_path(imd->image_path);
+
 	image_change_complete(imd, zoom, TRUE);
 
 	if (prev_pixbuf)
@@ -1847,6 +2011,9 @@ static void image_change_real(ImageWindow *imd, const gchar *path,
 		{
 		image_post_buffer_set(imd, NULL, NULL);
 		}
+
+	image_update_title(imd);
+	image_new_util(imd);
 }
 
 /*
@@ -1861,10 +2028,21 @@ static gint image_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer d
 
 	ImageWindow *imd = data;
 
-	if (!imd->pixbuf) return FALSE;
+	if (!imd->pixbuf)
+		{
+		image_overlay_draw(imd, event->area.x, event->area.y,
+					event->area.width, event->area.height);
+		return TRUE;
+		}
 
 	/* border */
 
+	if (imd->overlay_list)
+		{
+		image_border_draw(imd, event->area.x, event->area.y,
+				  event->area.width, event->area.height);
+		}
+
 	/* image */
 	x = MAX(0, (gint)event->area.x - imd->x_offset + imd->x_scroll);
 	y = MAX(0, (gint)event->area.y - imd->y_offset + imd->y_scroll);
@@ -1928,10 +2106,6 @@ static gint image_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, gpointer
 	GTK_WIDGET_SET_FLAGS(imd->widget, GTK_HAS_FOCUS);
 	image_focus_paint(imd, TRUE, NULL);
 
-#if 0
-	gtk_frame_set_shadow_type(GTK_FRAME(imd->widget), GTK_SHADOW_NONE);
-#endif
-
 	return TRUE;
 }
 
@@ -1942,13 +2116,352 @@ static gint image_focus_out_cb(GtkWidget *widget, GdkEventFocus *event, gpointer
 	GTK_WIDGET_UNSET_FLAGS(imd->widget, GTK_HAS_FOCUS);
 	image_focus_paint(imd, FALSE, NULL);
 
-#if 0
-	gtk_frame_set_shadow_type(GTK_FRAME(imd->widget), GTK_SHADOW_IN);
-#endif
+	return TRUE;
+}
+
+
+/*
+ *-------------------------------------------------------------------
+ * overlays
+ *-------------------------------------------------------------------
+ */
+
+static void image_overlay_draw(ImageWindow *imd, gint x, gint y, gint w, gint h)
+{
+	GList *work;
+
+	work = imd->overlay_list;
+	while (work)
+		{
+		OverlayData *od;
+		gint px, py, pw, ph;
+		gint rx, ry, rw, rh;
+
+		od = work->data;
+		work = work->next;
+
+		if (!od->visible) continue;
+
+		pw = gdk_pixbuf_get_width(od->pixbuf);
+		ph = gdk_pixbuf_get_height(od->pixbuf);
+		px = od->x;
+		py = od->y;
+
+	        if (od->relative)
+			{
+			if (px < 0) px = imd->window_width - pw + px;
+			if (py < 0) py = imd->window_height - ph + py;
+			}
+
+		if (util_clip_region(x, y, w, h, px, py, pw, ph, &rx, &ry, &rw, &rh))
+			{
+			gdk_pixbuf_render_to_drawable(od->pixbuf, imd->image->window,
+						      imd->image->style->fg_gc[GTK_WIDGET_STATE(imd->image)],
+						      rx - px, ry - py,
+						      rx, ry, rw, rh,
+						      (GdkRgbDither)dither_quality, rx, ry);
+			}
+		}
+}
+
+static void image_overlay_queue_draw(ImageWindow *imd, OverlayData *od, gint hidden)
+{
+	gint x, y, w, h;
+	gint old_vis;
+
+	w = gdk_pixbuf_get_width(od->pixbuf);
+	h = gdk_pixbuf_get_height(od->pixbuf);
+	x = od->x;
+	y = od->y;
+
+	if (od->relative)
+		{
+		if (x < 0) x = imd->window_width - w + x;
+		if (y < 0) y = imd->window_height - h + y;
+		}
+
+	image_queue(imd, imd->x_scroll - imd->x_offset + x,
+			 imd->y_scroll - imd->y_offset + y,
+			 w, h,
+			 FALSE, TILE_RENDER_ALL, FALSE);
+
+	old_vis = od->visible;
+	if (hidden) od->visible = FALSE;
+	image_border_draw(imd, x, y, w, h);
+	od->visible = old_vis;
+}
+
+static void image_overlay_queue_all(ImageWindow *imd)
+{
+	GList *work;
+
+	work = imd->overlay_list;
+	while (work)
+		{
+		OverlayData *od = work->data;
+		work = work->next;
+
+		image_overlay_queue_draw(imd, od, FALSE);
+		}
+}
+
+static OverlayData *image_overlay_find(ImageWindow *imd, gint id)
+{
+	GList *work;
+
+	work = imd->overlay_list;
+	while (work)
+		{
+		OverlayData *od = work->data;
+		work = work->next;
+
+		if (od->id == id) return od;
+		}
+
+	return NULL;
+}
+
+gint image_overlay_add(ImageWindow *imd, GdkPixbuf *pixbuf, gint x, gint y,
+		       gint relative, gint always)
+{
+	OverlayData *od;
+	gint id;
+
+	if (!imd || !pixbuf) return -1;
+
+	id = 1;
+	while (image_overlay_find(imd, id)) id++;
+
+	od = g_new0(OverlayData, 1);
+	od->id = id;
+	od->pixbuf = pixbuf;
+	g_object_ref(G_OBJECT(od->pixbuf));
+	od->x = x;
+	od->y = y;
+	od->relative = relative;
+	od->visible = TRUE;
+	od->always = always;
+
+	imd->overlay_list = g_list_append(imd->overlay_list, od);
+
+	image_overlay_queue_draw(imd, od, FALSE);
+
+	return od->id;
+}
+
+static void image_overlay_free(ImageWindow *imd, OverlayData *od)
+{
+	imd->overlay_list = g_list_remove(imd->overlay_list, od);
+
+	if (od->pixbuf) g_object_unref(G_OBJECT(od->pixbuf));
+	g_free(od);
+}
+
+static void image_overlay_list_clear(ImageWindow *imd)
+{
+	while (imd->overlay_list)
+		{
+		OverlayData *od;
+
+		od = imd->overlay_list->data;
+		image_overlay_free(imd, od);
+		}
+}
+
+void image_overlay_set(ImageWindow *imd, gint id, GdkPixbuf *pixbuf, gint x, gint y)
+{
+	OverlayData *od;
+
+	if (!imd) return;
+
+	od = image_overlay_find(imd, id);
+	if (!od) return;
+
+	if (pixbuf)
+		{
+		image_overlay_queue_draw(imd, od, TRUE);
+
+		g_object_ref(G_OBJECT(pixbuf));
+		g_object_unref(G_OBJECT(od->pixbuf));
+		od->pixbuf = pixbuf;
+
+		od->x = x;
+		od->y = y;
+
+		image_overlay_queue_draw(imd, od, FALSE);
+		}
+	else
+		{
+		image_overlay_queue_draw(imd, od, TRUE);
+		image_overlay_free(imd, od);
+		}
+}
+
+gint image_overlay_get(ImageWindow *imd, gint id, GdkPixbuf **pixbuf, gint *x, gint *y)
+{
+	OverlayData *od;
+
+	if (!imd) return FALSE;
+
+	od = image_overlay_find(imd, id);
+	if (!od) return FALSE;
+
+	if (pixbuf) *pixbuf = od->pixbuf;
+	if (x) *x = od->x;
+	if (y) *y = od->y;
 
 	return TRUE;
 }
 
+void image_overlay_remove(ImageWindow *imd, gint id)
+{
+	image_overlay_set(imd, id, NULL, 0, 0);
+}
+
+/*
+ *-------------------------------------------------------------------
+ * scroller
+ *-------------------------------------------------------------------
+ */
+
+#define SCROLLER_UPDATES_PER_SEC 30
+#define SCROLLER_DEAD_ZONE 6
+
+
+static gboolean image_scroller_update_cb(gpointer data)
+{
+	ImageWindow *imd = data;
+	gint x, y;
+	gint xinc, yinc;
+
+	/* this was a simple scroll by difference between scroller and mouse position,
+	 * but all this math results in a smoother result and accounts for a dead zone.
+	 */
+
+	if (abs(imd->scroller_xpos - imd->scroller_x) < SCROLLER_DEAD_ZONE)
+		{
+		x = 0;
+		}
+	else
+		{
+		gint shift = SCROLLER_DEAD_ZONE / 2 * SCROLLER_UPDATES_PER_SEC;
+		x = (imd->scroller_xpos - imd->scroller_x) / 2 * SCROLLER_UPDATES_PER_SEC;
+		x += (x > 0) ? -shift : shift;
+		}
+
+	if (abs(imd->scroller_ypos - imd->scroller_y) < SCROLLER_DEAD_ZONE)
+		{
+		y = 0;
+		}
+	else
+		{
+		gint shift = SCROLLER_DEAD_ZONE / 2 * SCROLLER_UPDATES_PER_SEC;
+		y = (imd->scroller_ypos - imd->scroller_y) / 2 * SCROLLER_UPDATES_PER_SEC;
+		y += (y > 0) ? -shift : shift;
+		}
+
+	if (abs(x) < SCROLLER_DEAD_ZONE * SCROLLER_UPDATES_PER_SEC)
+		{
+		xinc = x;
+		}
+	else
+		{
+		xinc = imd->scroller_xinc;
+
+		if (x >= 0)
+			{
+			if (xinc < 0) xinc = 0;
+			if (x < xinc) xinc = x;
+			if (x > xinc) xinc = MIN(xinc + x / SCROLLER_UPDATES_PER_SEC, x);
+			}
+		else
+			{
+			if (xinc > 0) xinc = 0;
+			if (x > xinc) xinc = x;
+			if (x < xinc) xinc = MAX(xinc + x / SCROLLER_UPDATES_PER_SEC, x);
+			}
+		}
+
+	if (abs(y) < SCROLLER_DEAD_ZONE * SCROLLER_UPDATES_PER_SEC)
+		{
+		yinc = y;
+		}
+	else
+		{
+		yinc = imd->scroller_yinc;
+
+		if (y >= 0)
+			{
+			if (yinc < 0) yinc = 0;
+			if (y < yinc) yinc = y;
+			if (y > yinc) yinc = MIN(yinc + y / SCROLLER_UPDATES_PER_SEC, y);
+			}
+		else
+			{
+			if (yinc > 0) yinc = 0;
+			if (y > yinc) yinc = y;
+			if (y < yinc) yinc = MAX(yinc + y / SCROLLER_UPDATES_PER_SEC, y);
+			}
+		}
+
+	imd->scroller_xinc = xinc;
+	imd->scroller_yinc = yinc;
+
+	xinc = xinc / SCROLLER_UPDATES_PER_SEC;
+	yinc = yinc / SCROLLER_UPDATES_PER_SEC;
+
+	image_scroll(imd, xinc, yinc);
+
+	return TRUE;
+}
+
+static void image_scroller_timer_set(ImageWindow *imd, gint start)
+{
+	if (imd->scroller_id != -1)
+		{
+		gtk_timeout_remove(imd->scroller_id);
+		imd->scroller_id = -1;
+		}
+
+	if (start)
+		{
+		imd->scroller_id = gtk_timeout_add(1000 / SCROLLER_UPDATES_PER_SEC,
+						   image_scroller_update_cb, imd);
+		}
+}
+
+static void image_scroller_start(ImageWindow *imd, gint x, gint y)
+{
+	if (imd->scroller_overlay == -1)
+		{
+		GdkPixbuf *pixbuf;
+		gint w, h;
+
+		pixbuf = gdk_pixbuf_new_from_inline(-1, icon_scroller, FALSE, NULL);
+		w = gdk_pixbuf_get_width(pixbuf);
+		h = gdk_pixbuf_get_height(pixbuf);
+
+		imd->scroller_overlay = image_overlay_add(imd, pixbuf, x - w / 2, y - h / 2, FALSE, TRUE);
+		}
+
+	imd->scroller_x = x;
+	imd->scroller_y = y;
+	imd->scroller_xpos = x;
+	imd->scroller_ypos = y;
+
+	image_scroller_timer_set(imd, TRUE);
+}
+
+static void image_scroller_stop(ImageWindow *imd)
+{
+	if (imd->scroller_id == -1) return;
+
+	image_overlay_remove(imd, imd->scroller_overlay);
+	imd->scroller_overlay = -1;
+
+	image_scroller_timer_set(imd, FALSE);
+}
+
 /*
  *-------------------------------------------------------------------
  * mouse stuff
@@ -1959,6 +2472,12 @@ static gint image_mouse_motion_cb(GtkWidget *widget, GdkEventButton *bevent, gpo
 {
 	ImageWindow *imd = data;
 
+	if (imd->scroller_id != -1)
+		{
+		imd->scroller_xpos = bevent->x;
+		imd->scroller_ypos = bevent->y;
+		}
+
 	if (!imd->in_drag || !gdk_pointer_is_grabbed()) return FALSE;
 
 	if (imd->drag_moved < IMAGE_DRAG_SCROLL_THRESHHOLD)
@@ -1983,6 +2502,8 @@ static gint image_mouse_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpoi
 {
 	ImageWindow *imd = data;
 
+	if (imd->scroller_id != -1) return TRUE;
+
 	switch (bevent->button)
 		{
 		case 1:
@@ -2014,6 +2535,12 @@ static gint image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 {
 	ImageWindow *imd = data;
 
+	if (imd->scroller_id != -1)
+		{
+		image_scroller_stop(imd);
+		return TRUE;
+		}
+
 	if (gdk_pointer_is_grabbed() && GTK_WIDGET_HAS_GRAB(imd->image))
 		{
 		gtk_grab_remove(imd->image);
@@ -2021,7 +2548,11 @@ static gint image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 		widget_set_cursor(imd->image, -1);
 		}
 
-	if (bevent->button == 1 || bevent->button == 2)
+	if (bevent->button == 1 && (bevent->state & GDK_SHIFT_MASK))
+		{
+		image_scroller_start(imd, bevent->x, bevent->y);
+		}
+	else if (bevent->button == 1 || bevent->button == 2)
 		{
 		if (imd->drag_moved < IMAGE_DRAG_SCROLL_THRESHHOLD) image_button_do(imd, bevent);
 		}
@@ -2031,6 +2562,21 @@ static gint image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 	return FALSE;
 }
 
+static gint image_mouse_leave_cb(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
+{
+	ImageWindow *imd = data;
+
+	if (imd->scroller_id != -1)
+		{
+		imd->scroller_xpos = imd->scroller_x;
+		imd->scroller_ypos = imd->scroller_y;
+		imd->scroller_xinc = 0;
+		imd->scroller_yinc = 0;
+		}
+
+	return FALSE;
+}
+
 static gint image_scroll_cb(GtkWidget *widget, GdkEventScroll *event, gpointer data)
 {
 	ImageWindow *imd = data;
@@ -2094,6 +2640,14 @@ void image_set_complete_func(ImageWindow *imd,
 	imd->data_complete = data;
 }
 
+void image_set_new_func(ImageWindow *imd,
+			void (*func)(ImageWindow *, gpointer),
+			gpointer data)
+{
+	imd->func_new = func;
+	imd->data_new = data;
+}
+
 
 static void image_button_do(ImageWindow *imd, GdkEventButton *bevent)
 {
@@ -2142,6 +2696,7 @@ void image_set_path(ImageWindow *imd, const gchar *newpath)
 	imd->image_name = filename_from_path(imd->image_path);
 
 	image_update_title(imd);
+	image_new_util(imd);
 }
 
 /* load a new image */
@@ -2157,6 +2712,7 @@ void image_change_path(ImageWindow *imd, const gchar *path, gfloat zoom)
 void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gfloat zoom)
 {
 	image_set_pixbuf(imd, pixbuf, zoom, TRUE);
+	image_new_util(imd);
 }
 
 void image_change_from_collection(ImageWindow *imd, CollectionData *cd, CollectInfo *info, gfloat zoom)
@@ -2248,6 +2804,8 @@ void image_change_from_image(ImageWindow *imd, ImageWindow *source)
 	imd->prev_path = source->prev_path;
 	source->prev_path = NULL;
 
+	imd->completed = source->completed;
+
 	imd->x_scroll = source->x_scroll;
 	imd->y_scroll = source->y_scroll;
 
@@ -2256,6 +2814,18 @@ void image_change_from_image(ImageWindow *imd, ImageWindow *source)
 
 /* manipulation */
 
+void image_area_changed(ImageWindow *imd, gint x, gint y, gint width, gint height)
+{
+	gint sx, sy, sw, sh;
+
+	sx = (gint)floor((float)x * imd->scale);
+	sy = (gint)floor((float)y * imd->scale);
+	sw = (gint)ceil((float)width * imd->scale);
+	sh = (gint)ceil((float)height * imd->scale);
+
+	image_queue(imd, sx, sy, sw, sh, FALSE, TILE_RENDER_AREA, TRUE);
+}
+
 void image_reload(ImageWindow *imd)
 {
 	image_change_complete(imd, imd->zoom, FALSE);
@@ -2340,6 +2910,29 @@ void image_zoom_set(ImageWindow *imd, gfloat zoom)
 	image_zoom_sync(imd, zoom, FALSE, FALSE, FALSE, FALSE, 0, 0);
 }
 
+void image_zoom_set_fill_geometry(ImageWindow *imd, gint vertical)
+{
+	gfloat zoom;
+
+	if (!imd->pixbuf || imd->image_width < 1 || imd->image_height < 1) return;
+
+	if (vertical)
+		{
+		zoom = (gfloat)imd->window_height / imd->image_height;
+		}
+	else
+		{
+		zoom = (gfloat)imd->window_width / imd->image_width;
+		}
+
+	if (zoom < 1.0)
+		{
+		zoom = 0.0 - 1.0 / zoom;
+		}
+
+	image_zoom_set(imd, zoom);
+}
+
 gfloat image_zoom_get(ImageWindow *imd)
 {
 	return imd->zoom;
@@ -2521,6 +3114,23 @@ void image_background_set_color(ImageWindow *imd, GdkColor *color)
 	if (GTK_WIDGET_VISIBLE(imd->widget)) image_border_clear(imd);
 }
 
+void image_set_delay_flip(ImageWindow *imd, gint delay)
+{
+	if (!imd ||
+	    imd->delay_flip == delay) return;
+
+	imd->delay_flip = delay;
+	if (!imd->delay_flip && imd->il)
+		{
+		if (imd->pixbuf) g_object_unref(imd->pixbuf);
+		imd->pixbuf = NULL;
+		image_load_pixbuf_ready(imd);
+
+		image_queue_clear(imd);
+		image_queue(imd, 0, 0, imd->width, imd->height, FALSE, TILE_RENDER_AREA, TRUE);
+		}
+}
+
 /* wallpaper util */
 
 void image_to_root_window(ImageWindow *imd, gint scaled)
@@ -2576,6 +3186,10 @@ static void image_free(ImageWindow *imd)
 	image_tile_sync_count(imd, 0);
 	if (imd->pixbuf) g_object_unref(imd->pixbuf);
 
+	image_scroller_timer_set(imd, FALSE);
+
+	image_overlay_list_clear(imd);
+
 	g_free(imd);
 }
 
@@ -2624,12 +3238,17 @@ ImageWindow *image_new(gint frame)
 	imd->auto_refresh_id = -1;
 	imd->auto_refresh_interval = -1;
 
+	imd->delay_flip = FALSE;
+
 	imd->func_update = NULL;
 	imd->func_complete = NULL;
 
 	imd->func_button = NULL;
 	imd->func_scroll = NULL;
 
+	imd->scroller_id = -1;
+	imd->scroller_overlay = -1;
+
 	imd->eventbox = gtk_event_box_new();
 
 	g_signal_connect_after(G_OBJECT(imd->eventbox), "size_allocate",
@@ -2664,7 +3283,11 @@ ImageWindow *image_new(gint frame)
 			 G_CALLBACK(image_mouse_press_cb), imd);
 	g_signal_connect(G_OBJECT(imd->image), "button_release_event",
 			 G_CALLBACK(image_mouse_release_cb), imd);
-	gtk_widget_set_events(imd->image, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK);
+	g_signal_connect(G_OBJECT(imd->image), "leave_notify_event",
+			 G_CALLBACK(image_mouse_leave_cb), imd);
+	gtk_widget_set_events(imd->image, GDK_POINTER_MOTION_MASK |
+					  GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK |
+					  GDK_LEAVE_NOTIFY_MASK);
 
 	g_signal_connect(G_OBJECT(imd->image), "expose_event",
 			 G_CALLBACK(image_expose_cb), imd);
