@@ -31,6 +31,9 @@ void image_set_scroll_func(ImageWindow *imd,
 void image_set_complete_func(ImageWindow *imd,
 			     void (*func)(ImageWindow *, gint preload, gpointer),
 			     gpointer data);
+void image_set_new_func(ImageWindow *imd,
+			void (*func)(ImageWindow *, gpointer),
+			gpointer data);
 
 /* path, name */
 const gchar *image_get_path(ImageWindow *imd);
@@ -47,6 +50,7 @@ CollectionData *image_get_collection(ImageWindow *imd, CollectInfo **info);
 void image_change_from_image(ImageWindow *imd, ImageWindow *source);
 
 /* manipulation */
+void image_area_changed(ImageWindow *imd, gint x, gint y, gint width, gint height);
 void image_reload(ImageWindow *imd);
 void image_scroll(ImageWindow *imd, gint x, gint y);
 void image_alter(ImageWindow *imd, AlterType type);
@@ -55,6 +59,7 @@ void image_alter(ImageWindow *imd, AlterType type);
 void image_zoom_adjust(ImageWindow *imd, gfloat increment);
 void image_zoom_adjust_at_point(ImageWindow *imd, gfloat increment, gint x, gint y);
 void image_zoom_set(ImageWindow *imd, gfloat zoom);
+void image_zoom_set_fill_geometry(ImageWindow *imd, gint vertical);
 gfloat image_zoom_get(ImageWindow *imd);
 gfloat image_zoom_get_real(ImageWindow *imd);
 gchar *image_zoom_get_as_text(ImageWindow *imd);
@@ -73,9 +78,19 @@ void image_top_window_set_sync(ImageWindow *imd, gint allow_sync);
 void image_background_set_black(ImageWindow *imd, gint black);
 void image_background_set_color(ImageWindow *imd, GdkColor *color);
 
+/* set delayed page flipping */
+void image_set_delay_flip(ImageWindow *imd, gint delay);
+
 /* wallpaper util */
 void image_to_root_window(ImageWindow *imd, gint scaled);
 
+/* overlays */
+gint image_overlay_add(ImageWindow *imd, GdkPixbuf *pixbuf, gint x, gint y,
+		       gint relative, gint always);
+void image_overlay_set(ImageWindow *imd, gint id, GdkPixbuf *pixbuf, gint x, gint y);
+gint image_overlay_get(ImageWindow *imd, gint id, GdkPixbuf **pixbuf, gint *x, gint *y);
+void image_overlay_remove(ImageWindow *imd, gint id);
+
 
 #endif
 
