@@ -14,11 +14,13 @@
 #include "img-view.h"
 
 #include "collect.h"
+#include "collect-io.h"
 #include "dnd.h"
 #include "editors.h"
 #include "filelist.h"
 #include "fullscreen.h"
 #include "image.h"
+#include "image-overlay.h"
 #include "info.h"
 #include "menu.h"
 #include "slideshow.h"
@@ -42,6 +44,8 @@ struct _ViewWindow
 
 	GList *list;
 	GList *list_pointer;
+
+	gint overlay_id;
 };
 
 
@@ -50,6 +54,7 @@ static GList *view_window_list = NULL;
 
 static GtkWidget *view_popup_menu(ViewWindow *vw);
 static void view_fullscreen_toggle(ViewWindow *vw, gint force_off);
+static void view_overlay_toggle(ViewWindow *vw);
 
 static void view_slideshow_next(ViewWindow *vw);
 static void view_slideshow_prev(ViewWindow *vw);
@@ -214,9 +219,9 @@ static void view_list_step(ViewWindow *vw, gint next)
 		}
 	if (!work) return;
 
+	vw->list_pointer = work;
 	path = work->data;
 	image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
-	vw->list_pointer = work;
 
 	if (enable_read_ahead && work_ahead)
 		{
@@ -245,9 +250,9 @@ static void view_list_step_to_end(ViewWindow *vw, gint last)
 		work_ahead = work->next;
 		}
 
+	vw->list_pointer = work;
 	path = work->data;
 	image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
-	vw->list_pointer = work;
 
 	if (enable_read_ahead && work_ahead)
 		{
@@ -396,6 +401,12 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 		case '9':
 			image_zoom_set(imd, -2.0);
 			break;
+		case 'W': case 'w':
+			image_zoom_set_fill_geometry(imd, FALSE);
+			break;
+		case 'H': case 'h':
+			image_zoom_set_fill_geometry(imd, TRUE);
+			break;
 		case 'R': case 'r':
 			image_reload(imd);
 			break;
@@ -414,6 +425,9 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 			view_fullscreen_toggle(vw, FALSE);
 			stop_signal = TRUE;
 			break;
+		case 'I': case 'i':
+			view_overlay_toggle(vw);
+			break;
 		case ']':
 			image_alter(imd, ALTER_ROTATE_90);
 			stop_signal = TRUE;
@@ -425,7 +439,7 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 		case GDK_Delete: case GDK_KP_Delete:
 			if (enable_delete_key)
 				{
-				file_util_delete(image_get_path(imd), NULL);
+				file_util_delete(image_get_path(imd), NULL, imd->widget);
 				stop_signal = TRUE;
 				}
 			break;
@@ -484,19 +498,19 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 				n = 9;
 				break;
 			case 'C': case 'c':
-				file_util_copy(image_get_path(imd), NULL, NULL);
+				file_util_copy(image_get_path(imd), NULL, NULL, imd->widget);
 				stop_signal = TRUE;
 				break;
 			case 'M': case 'm':
-				file_util_move(image_get_path(imd), NULL, NULL);
+				file_util_move(image_get_path(imd), NULL, NULL, imd->widget);
 				stop_signal = TRUE;
 				break;
 			case 'R': case 'r':
-				file_util_rename(image_get_path(imd), NULL);
+				file_util_rename(image_get_path(imd), NULL, imd->widget);
 				stop_signal = TRUE;
 				break;
 			case 'D': case 'd':
-				file_util_delete(image_get_path(imd), NULL);
+				file_util_delete(image_get_path(imd), NULL, imd->widget);
 				stop_signal = TRUE;
 				break;
 			case 'P': case 'p':
@@ -651,6 +665,8 @@ static void view_fullscreen_toggle(ViewWindow *vw, gint force_off)
 	if (vw->fs)
 		{
 		fullscreen_stop(vw->fs);
+
+		if (vw->overlay_id != -1) vw->overlay_id = image_overlay_info_enable(vw->imd);
 		}
 	else
 		{
@@ -661,6 +677,29 @@ static void view_fullscreen_toggle(ViewWindow *vw, gint force_off)
 				 G_CALLBACK(view_window_key_press_cb), vw);
 
 		if (vw->ss) vw->ss->imd = vw->fs->imd;
+
+		if (vw->overlay_id != -1)
+			{
+			image_overlay_info_disable(vw->imd, vw->overlay_id);
+			vw->overlay_id = image_overlay_info_enable(vw->fs->imd);
+			}
+		}
+}
+
+static void view_overlay_toggle(ViewWindow *vw)
+{
+	ImageWindow *imd;
+
+	imd = view_window_active_image(vw);
+
+	if (vw->overlay_id == -1)
+		{
+		vw->overlay_id = image_overlay_info_enable(imd);
+		}
+	else
+		{
+		image_overlay_info_disable(imd, vw->overlay_id);
+		vw->overlay_id = -1;
 		}
 }
 
@@ -730,14 +769,14 @@ static gint view_window_delete_cb(GtkWidget *w, GdkEventAny *event, gpointer dat
 	return TRUE;
 }
 
-static void real_view_window_new(const gchar *path, GList *list, CollectionData *cd, CollectInfo *info)
+static ViewWindow *real_view_window_new(const gchar *path, GList *list, CollectionData *cd, CollectInfo *info)
 {
 	ViewWindow *vw;
 	GtkAllocation req_size;
 	GdkGeometry geometry;
 	gint w, h;
 
-	if (!path && !list && (!cd || !info)) return;
+	if (!path && !list && (!cd || !info)) return NULL;
 
 	vw = g_new0(ViewWindow, 1);
 	vw->fs = NULL;
@@ -745,6 +784,8 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 	vw->list = NULL;
 	vw->list_pointer = NULL;
 
+	vw->overlay_id = -1;
+
 	vw->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
 	geometry.min_width = 8;
@@ -791,8 +832,8 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 	else if (list)
 		{
 		view_window_set_list(vw, list);
-		image_change_path(vw->imd, (gchar *)vw->list->data, image_zoom_get_default(NULL, zoom_mode));
 		vw->list_pointer = vw->list;
+		image_change_path(vw->imd, (gchar *)vw->list->data, image_zoom_get_default(NULL, zoom_mode));
 
 		if (enable_read_ahead)
 			{
@@ -835,11 +876,59 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 	gtk_widget_show(vw->window);
 
 	view_window_list = g_list_append(view_window_list, vw);
+
+	return vw;
+}
+
+static void view_window_collection_unref_cb(GtkWidget *widget, gpointer data)
+{
+	CollectionData *cd = data;
+
+	collection_unref(cd);
 }
 
 void view_window_new(const gchar *path)
 {
-	real_view_window_new(path, NULL, NULL, NULL);
+	if (file_extension_match(path, ".gqv"))
+		{
+		ViewWindow *vw;
+		CollectionData *cd;
+		CollectInfo *info;
+
+		cd = collection_new(path);
+		if (collection_load(cd, path, FALSE))
+			{
+			info = collection_get_first(cd);
+			}
+		else
+			{
+			collection_unref(cd);
+			cd = NULL;
+			info = NULL;
+			}
+		vw = real_view_window_new(NULL, NULL, cd, info);
+		if (vw && cd)
+			{
+			g_signal_connect(G_OBJECT(vw->window), "destroy",
+					 G_CALLBACK(view_window_collection_unref_cb), cd);
+			}
+		}
+	else if (isdir(path))
+		{
+		GList *list = NULL;
+
+		if (path_list(path, &list, NULL))
+			{
+			list = path_list_sort(list);
+			list = path_list_filter(list, FALSE);
+			}
+		real_view_window_new(NULL, list, NULL, NULL);
+		path_list_free(list);
+		}
+	else
+		{
+		real_view_window_new(path, NULL, NULL, NULL);
+		}
 }
 
 void view_window_new_from_list(GList *list)
@@ -872,6 +961,38 @@ void view_window_colors_update(void)
 		}
 }
 
+gint view_window_find_image(ImageWindow *imd, gint *index, gint *total)
+{
+	GList *work;
+
+	work = view_window_list;
+	while (work)
+		{
+		ViewWindow *vw = work->data;
+		work = work->next;
+
+		if (vw->imd == imd ||
+		    (vw->fs && vw->fs->imd == imd))
+			{
+			if (vw->ss)
+				{
+				gint n;
+				n = g_list_length(vw->ss->list_done);
+				if (index) *index = n - 1;
+				if (total) *total = n + g_list_length(vw->ss->list);
+				}
+			else
+				{
+				if (index) *index = g_list_position(vw->list, vw->list_pointer);
+				if (total) *total = g_list_length(vw->list);
+				}
+			return TRUE;
+			}
+		}
+
+	return FALSE;
+}
+
 /*
  *-----------------------------------------------------------------------------
  * view window menu routines and callbacks
@@ -973,7 +1094,7 @@ static void view_copy_cb(GtkWidget *widget, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
-	file_util_copy(image_get_path(imd), NULL, NULL);
+	file_util_copy(image_get_path(imd), NULL, NULL, imd->widget);
 }
 
 static void view_move_cb(GtkWidget *widget, gpointer data)
@@ -982,7 +1103,7 @@ static void view_move_cb(GtkWidget *widget, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
-	file_util_move(image_get_path(imd), NULL, NULL);
+	file_util_move(image_get_path(imd), NULL, NULL, imd->widget);
 }
 
 static void view_rename_cb(GtkWidget *widget, gpointer data)
@@ -991,7 +1112,7 @@ static void view_rename_cb(GtkWidget *widget, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
-	file_util_rename(image_get_path(imd), NULL);
+	file_util_rename(image_get_path(imd), NULL, imd->widget);
 }
 
 static void view_delete_cb(GtkWidget *widget, gpointer data)
@@ -1000,7 +1121,7 @@ static void view_delete_cb(GtkWidget *widget, gpointer data)
 	ImageWindow *imd;
 
 	imd = view_window_active_image(vw);
-	file_util_delete(image_get_path(imd), NULL);
+	file_util_delete(image_get_path(imd), NULL, imd->widget);
 }
 
 static void view_fullscreen_cb(GtkWidget *widget, gpointer data)
@@ -1164,9 +1285,9 @@ static void view_dir_list_do(ViewWindow *vw, GList *list, gint skip, gint recurs
 		{
 		gchar *path;
 
+		vw->list_pointer = vw->list;
 		path = vw->list->data;
 		image_change_path(vw->imd, path, image_zoom_get_default(vw->imd, zoom_mode));
-		vw->list_pointer = vw->list;
 
 		work = vw->list->next;
 		if (enable_read_ahead && work)
@@ -1219,11 +1340,11 @@ static GtkWidget *view_confirm_dir_list(ViewWindow *vw, GList *list)
 	g_signal_connect(G_OBJECT(menu), "destroy",
 			 G_CALLBACK(view_dir_list_destroy), d);
 
-	menu_item_add_stock(menu, _("Dropped list includes directories."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
+	menu_item_add_stock(menu, _("Dropped list includes folders."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("_Add contents"), GTK_STOCK_OK, G_CALLBACK(view_dir_list_add), d);
 	menu_item_add_stock(menu, _("Add contents _recursive"), GTK_STOCK_ADD, G_CALLBACK(view_dir_list_recurse), d);
-	menu_item_add_stock(menu, _("_Skip directories"), GTK_STOCK_REMOVE, G_CALLBACK(view_dir_list_skip), d);
+	menu_item_add_stock(menu, _("_Skip folders"), GTK_STOCK_REMOVE, G_CALLBACK(view_dir_list_skip), d);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("Cancel"), GTK_STOCK_CANCEL, G_CALLBACK(view_dir_list_cancel), d);
 
