@@ -13,10 +13,12 @@
 #include "gqview.h"
 #include "info.h"
 
+#include "bar_info.h"
 #include "bar_exif.h"
 #include "dnd.h"
 #include "filelist.h"
 #include "image.h"
+#include "image-load.h"
 #include "ui_bookmark.h"
 #include "ui_fileops.h"
 
@@ -28,7 +30,7 @@
 #define IMAGE_SIZE_H 200
 
 #define DEF_PROPERTY_WIDTH  500
-#define DEF_PROPERTY_HEIGHT 300
+#define DEF_PROPERTY_HEIGHT 350
 
 typedef struct _TabData TabData;
 struct _TabData
@@ -46,8 +48,8 @@ struct _TabData
  *-------------------------------------------------------------------
  */
 
-static GtkWidget *table_add_line(GtkWidget *table, gint x, gint y,
-				 const gchar *description, const gchar *text)
+GtkWidget *table_add_line(GtkWidget *table, gint x, gint y,
+			  const gchar *description, const gchar *text)
 {
 	GtkWidget *label;
 
@@ -107,7 +109,7 @@ static TabData *info_tab_exif_new(InfoData *id)
 	GtkWidget *bar;
 	GtkWidget *label;
 
-	bar = bar_exif_new(FALSE, NULL, FALSE);
+	bar = bar_exif_new(FALSE, NULL, FALSE, NULL);
 
 	label = gtk_label_new(_("Exif"));
 	gtk_notebook_append_page(GTK_NOTEBOOK(id->notebook), bar, label);
@@ -129,116 +131,53 @@ static TabData *info_tab_exif_new(InfoData *id)
  *-------------------------------------------------------------------
  */
 
-typedef struct _InfoTabPerm InfoTabPerm;
-struct _InfoTabPerm
+typedef struct _InfoTabMeta InfoTabMeta;
+struct _InfoTabMeta
 {
-	GtkWidget *label_user;
-	GtkWidget *label_group;
-	GtkWidget *label_perms;
+	GtkWidget *bar_info;
 };
 
-static void info_tab_perm_free(gpointer data)
+static void info_tab_meta_free(gpointer data)
 {
-	InfoTabPerm *tab = data;
+	InfoTabMeta *tab = data;
 
 	g_free(tab);
 }
 
-static gchar *mode_number(mode_t m)
+static void info_tab_meta_sync(InfoData *id, gpointer data)
 {
-	int mb, mu, mg, mo;
+	InfoTabMeta *tab = data;
 
-	mb = mu = mg = mo = 0;
-
-	if (m & S_ISUID) mb |= 4;
-	if (m & S_ISGID) mb |= 2;
-	if (m & S_ISVTX) mb |= 1;
-
-	if (m & S_IRUSR) mu |= 4;
-	if (m & S_IWUSR) mu |= 2;
-	if (m & S_IXUSR) mu |= 1;
-
-	if (m & S_IRGRP) mg |= 4;
-	if (m & S_IWGRP) mg |= 2;
-	if (m & S_IXGRP) mg |= 1;
-
-	if (m & S_IROTH) mo |= 4;
-	if (m & S_IWOTH) mo |= 2;
-	if (m & S_IXOTH) mo |= 1;
-
-	return g_strdup_printf("%d%d%d%d", mb, mu, mg, mo);
+	bar_info_set(tab->bar_info, id->path);
 }
 
-static void info_tab_perm_sync(InfoData *id, gpointer data)
+static GList *info_tab_meta_list_cb(gpointer data)
 {
-	InfoTabPerm *tab = data;
-	struct stat st;
+	InfoData *id = data;
 
-	if (!stat_utf8(id->path, &st))
-		{
-		gtk_label_set_text(GTK_LABEL(tab->label_user), "");
-		gtk_label_set_text(GTK_LABEL(tab->label_group), "");
-		gtk_label_set_text(GTK_LABEL(tab->label_perms), "");
-		}
-	else
-		{
-		struct passwd *user;
-		struct group *grp;
-		gchar pbuf[12];
-		gchar *pmod;
-		gchar *buf;
-
-		user = getpwuid(st.st_uid);
-		gtk_label_set_text(GTK_LABEL(tab->label_user), (user) ? user->pw_name : "");
-
-		grp = getgrgid(st.st_gid);
-		gtk_label_set_text(GTK_LABEL(tab->label_group), (grp) ? grp->gr_name : "");
-
-		pbuf[0] = (st.st_mode & S_IRUSR) ? 'r' : ' ';
-		pbuf[1] = (st.st_mode & S_IWUSR) ? 'w' : ' ';
-		pbuf[2] = (st.st_mode & S_IXUSR) ? 'x' : ' ';
-		pbuf[3] = '-';
-		pbuf[4] = (st.st_mode & S_IRGRP) ? 'r' : ' ';
-		pbuf[5] = (st.st_mode & S_IWGRP) ? 'w' : ' ';
-		pbuf[6] = (st.st_mode & S_IXGRP) ? 'x' : ' ';
-		pbuf[7] = '-';
-		pbuf[8] = (st.st_mode & S_IROTH) ? 'r' : ' ';
-		pbuf[9] = (st.st_mode & S_IWOTH) ? 'w' : ' ';
-		pbuf[10] = (st.st_mode & S_IXOTH) ? 'x' : ' ';
-		pbuf[11] = '\0';
-
-		pmod = mode_number(st.st_mode);
-		buf = g_strdup_printf("%s (%s)", pbuf, pmod);
-		gtk_label_set_text(GTK_LABEL(tab->label_perms), buf);
-		g_free(buf);
-		g_free(pmod);
-		}
+	return path_list_copy(id->list);
 }
 
-static TabData *info_tab_perm_new(InfoData *id)
+static TabData *info_tab_meta_new(InfoData *id)
 {
 	TabData *td;
-	InfoTabPerm *tab;
-	GtkWidget *table;
+	InfoTabMeta *tab;
 	GtkWidget *label;
 
-	tab = g_new0(InfoTabPerm, 1);
+	tab = g_new0(InfoTabMeta, 1);
 
-	table = gtk_table_new(6, 8, FALSE);
-	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
+	tab->bar_info = bar_info_new(NULL, TRUE, NULL);
+	bar_info_set_selection_func(tab->bar_info, info_tab_meta_list_cb, id);
+	bar_info_selection(tab->bar_info, g_list_length(id->list) - 1);
 
-	tab->label_user = table_add_line(table, 0, 0, _("Owner:"), NULL);
-	tab->label_group = table_add_line(table, 0, 1, _("Group:"), NULL);
-	tab->label_perms = table_add_line(table, 0, 2, "", NULL);
-
-	label = gtk_label_new(_("Misc."));
-	gtk_notebook_append_page(GTK_NOTEBOOK(id->notebook), table, label);
-	gtk_widget_show(table);
+	label = gtk_label_new(_("Keywords"));
+	gtk_notebook_append_page(GTK_NOTEBOOK(id->notebook), tab->bar_info, label);
+	gtk_widget_show(tab->bar_info);
 
 	/* register */
 	td = g_new0(TabData, 1);
-	td->func_free = info_tab_perm_free;
-	td->func_sync = info_tab_perm_sync;
+	td->func_free = info_tab_meta_free;
+	td->func_sync = info_tab_meta_sync;
 	td->func_image = NULL;
 	td->data = tab;
 
@@ -260,6 +199,12 @@ struct _InfoTabGeneral
 	GtkWidget *label_transparent;
 	GtkWidget *label_image_size;
 	GtkWidget *label_compression;
+	GtkWidget *label_mime_type;
+
+	GtkWidget *label_user;
+	GtkWidget *label_group;
+	GtkWidget *label_perms;
+
 	gint compression_done;
 	gint64 byte_size;
 };
@@ -300,6 +245,79 @@ static void info_tab_general_image(InfoData *id, gpointer data)
 
 		tab->compression_done = TRUE;
 		}
+
+	buf = image_loader_get_format(id->image->il);
+	if (buf)
+	gtk_label_set_text(GTK_LABEL(tab->label_mime_type), buf);
+	g_free(buf);
+}
+
+static gchar *mode_number(mode_t m)
+{
+	int mb, mu, mg, mo;
+
+	mb = mu = mg = mo = 0;
+
+	if (m & S_ISUID) mb |= 4;
+	if (m & S_ISGID) mb |= 2;
+	if (m & S_ISVTX) mb |= 1;
+
+	if (m & S_IRUSR) mu |= 4;
+	if (m & S_IWUSR) mu |= 2;
+	if (m & S_IXUSR) mu |= 1;
+
+	if (m & S_IRGRP) mg |= 4;
+	if (m & S_IWGRP) mg |= 2;
+	if (m & S_IXGRP) mg |= 1;
+
+	if (m & S_IROTH) mo |= 4;
+	if (m & S_IWOTH) mo |= 2;
+	if (m & S_IXOTH) mo |= 1;
+
+	return g_strdup_printf("%d%d%d%d", mb, mu, mg, mo);
+}
+
+static void info_tab_general_sync_perm(InfoTabGeneral *tab, InfoData *id)
+{
+	struct stat st;
+
+	if (!stat_utf8(id->path, &st))
+		{
+		gtk_label_set_text(GTK_LABEL(tab->label_user), "");
+		gtk_label_set_text(GTK_LABEL(tab->label_group), "");
+		gtk_label_set_text(GTK_LABEL(tab->label_perms), "");
+		}
+	else
+		{
+		struct passwd *user;
+		struct group *grp;
+		gchar pbuf[12];
+		gchar *pmod;
+		gchar *buf;
+
+		user = getpwuid(st.st_uid);
+		gtk_label_set_text(GTK_LABEL(tab->label_user), (user) ? user->pw_name : "");
+
+		grp = getgrgid(st.st_gid);
+		gtk_label_set_text(GTK_LABEL(tab->label_group), (grp) ? grp->gr_name : "");
+
+		pbuf[0] = (st.st_mode & S_IRUSR) ? 'r' : '-';
+		pbuf[1] = (st.st_mode & S_IWUSR) ? 'w' : '-';
+		pbuf[2] = (st.st_mode & S_IXUSR) ? 'x' : '-';
+		pbuf[3] = (st.st_mode & S_IRGRP) ? 'r' : '-';
+		pbuf[4] = (st.st_mode & S_IWGRP) ? 'w' : '-';
+		pbuf[5] = (st.st_mode & S_IXGRP) ? 'x' : '-';
+		pbuf[6] = (st.st_mode & S_IROTH) ? 'r' : '-';
+		pbuf[7] = (st.st_mode & S_IWOTH) ? 'w' : '-';
+		pbuf[8] = (st.st_mode & S_IXOTH) ? 'x' : '-';
+		pbuf[9] = '\0';
+
+		pmod = mode_number(st.st_mode);
+		buf = g_strdup_printf("%s (%s)", pbuf, pmod);
+		gtk_label_set_text(GTK_LABEL(tab->label_perms), buf);
+		g_free(buf);
+		g_free(pmod);
+		}
 }
 
 static void info_tab_general_sync(InfoData *id, gpointer data)
@@ -320,6 +338,9 @@ static void info_tab_general_sync(InfoData *id, gpointer data)
 	gtk_label_set_text(GTK_LABEL(tab->label_image_size), "");
 
 	gtk_label_set_text(GTK_LABEL(tab->label_compression), "");
+	gtk_label_set_text(GTK_LABEL(tab->label_mime_type), "");
+
+	info_tab_general_sync_perm(tab, id);
 
 	tab->compression_done = FALSE;
 }
@@ -340,7 +361,7 @@ static TabData *info_tab_general_new(InfoData *id)
 
 	tab = g_new0(InfoTabGeneral, 1);
 
-	table = gtk_table_new(2, 8, FALSE);
+	table = gtk_table_new(2, 11, FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 
 	tab->label_file_time = table_add_line(table, 0, 0, _("File date:"), NULL);
@@ -351,6 +372,11 @@ static TabData *info_tab_general_new(InfoData *id)
 	tab->label_image_size = table_add_line(table, 0, 4, _("Image size:"), NULL);
 
 	tab->label_compression = table_add_line(table, 0, 5, _("Compress ratio:"), NULL);
+	tab->label_mime_type = table_add_line(table, 0, 6, _("File type:"), NULL);
+
+	tab->label_user = table_add_line(table, 0, 7, _("Owner:"), NULL);
+	tab->label_group = table_add_line(table, 0, 8, _("Group:"), NULL);
+	tab->label_perms = table_add_line(table, 0, 9, "", NULL);
 
 	label = gtk_label_new(_("General"));
 	gtk_notebook_append_page(GTK_NOTEBOOK(id->notebook), table, label);
@@ -413,7 +439,7 @@ static void info_tabs_free(InfoData *id)
 static void info_tabs_init(InfoData *id)
 {
 	id->tab_list = g_list_append(id->tab_list, info_tab_general_new(id));
-	id->tab_list = g_list_append(id->tab_list, info_tab_perm_new(id));
+	id->tab_list = g_list_append(id->tab_list, info_tab_meta_new(id));
 	id->tab_list = g_list_append(id->tab_list, info_tab_exif_new(id));
 }
 
