@@ -12,6 +12,7 @@
 #include "gqview.h"
 #include "layout.h"
 
+#include "globals.h"
 #include "image.h"
 #include "layout_config.h"
 #include "layout_image.h"
@@ -65,6 +66,22 @@ gint layout_valid(LayoutWindow **lw)
 	return (g_list_find(layout_window_list, *lw) != NULL);
 }
 
+LayoutWindow *layout_find_by_image(ImageWindow *imd)
+{
+	GList *work;
+
+	work = layout_window_list;
+	while (work)
+		{
+		LayoutWindow *lw = work->data;
+		work = work->next;
+
+		if (lw->image == imd) return lw;
+		}
+
+	return NULL;
+}
+
 /*
  *-----------------------------------------------------------------------------
  * menu, toolbar, and dir view
@@ -337,6 +354,8 @@ void layout_status_update_info(LayoutWindow *lw, const gchar *text)
 		n = layout_list_count(lw, &n_bytes);
 		s = layout_selection_count(lw, &s_bytes);
 
+		layout_bars_new_selection(lw, s);
+
 		if (s > 0)
 			{
 			b = text_from_size_abrev(n_bytes);
@@ -1060,18 +1079,28 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 	if (!lw->tools)
 		{
 		GdkGeometry geometry;
+		GdkWindowHints hints;
 
 		lw->tools = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 		g_signal_connect(G_OBJECT(lw->tools), "delete_event",
 				 G_CALLBACK(layout_tools_delete_cb), lw);
 		layout_keyboard_init(lw, lw->tools);
 
+		if (save_window_positions)
+			{
+			hints = GDK_HINT_USER_POS | GDK_HINT_USER_SIZE;
+			}
+		else
+			{
+			hints = 0;
+			}
+
 		geometry.min_width = 32;
 		geometry.min_height = 32;
 		geometry.base_width = TOOLWINDOW_DEF_WIDTH;
 		geometry.base_height = TOOLWINDOW_DEF_HEIGHT;
 		gtk_window_set_geometry_hints(GTK_WINDOW(lw->tools), NULL, &geometry,
-					      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE);
+					      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE | hints);
 
 
 	        gtk_window_set_resizable(GTK_WINDOW(lw->tools), TRUE);
@@ -1464,7 +1493,7 @@ void layout_tools_hide_toggle(LayoutWindow *lw)
 
 void layout_tools_float_set(LayoutWindow *lw, gint popped, gint hidden)
 {
-	if (!lw) return;
+	if (!layout_valid(&lw)) return;
 
 	if (lw->tools_float == popped && lw->tools_hidden == hidden) return;
 
@@ -1566,6 +1595,7 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 {
 	LayoutWindow *lw;
 	GdkGeometry geometry;
+	GdkWindowHints hints;
 
 	lw = g_new0(LayoutWindow, 1);
 
@@ -1586,6 +1616,9 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 	lw->bar_exif_size = -1;
 	lw->bar_exif_advanced = FALSE;
 
+	lw->full_screen_overlay_id = -1;
+	lw->full_screen_overlay_on = FALSE;
+
 	/* default layout */
 
 	layout_config_parse(layout_style, layout_order,
@@ -1619,12 +1652,21 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 
 	window_set_icon(lw->window, NULL, NULL);
 
+	if (save_window_positions)
+		{
+		hints = GDK_HINT_USER_POS | GDK_HINT_USER_SIZE;
+		}
+	else
+		{
+		hints = 0;
+		}
+
 	geometry.min_width = 32;
 	geometry.min_height = 32;
 	geometry.base_width = MAINWINDOW_DEF_WIDTH;
 	geometry.base_height = MAINWINDOW_DEF_HEIGHT;
 	gtk_window_set_geometry_hints(GTK_WINDOW(lw->window), NULL, &geometry,
-				      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE);
+				      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE | hints);
 
 	if (save_window_positions)
 		{
@@ -1632,6 +1674,7 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 		if (!layout_window_list)
 			{
 			gtk_window_move(GTK_WINDOW(lw->window), main_window_x, main_window_y);
+			if (main_window_maximized) gtk_window_maximize(GTK_WINDOW(lw->window));
 			}
 		}
 	else
