@@ -15,13 +15,16 @@
 #include "collect.h"
 #include "dnd.h"
 #include "editors.h"
+#include "filelist.h"
 #include "fullscreen.h"
 #include "image.h"
+#include "image-overlay.h"
 #include "img-view.h"
 #include "info.h"
 #include "layout.h"
 #include "layout_util.h"
 #include "menu.h"
+#include "pixbuf_util.h"
 #include "utilops.h"
 #include "slideshow.h"
 #include "ui_bookmark.h"
@@ -34,6 +37,35 @@
 static GtkWidget *layout_image_pop_menu(LayoutWindow *lw);
 static void layout_image_set_buttons(LayoutWindow *lw);
 
+/*
+ *----------------------------------------------------------------------------
+ * full screen overlay
+ *----------------------------------------------------------------------------
+ */
+
+static void layout_image_overlay_set(LayoutWindow *lw, gint enable)
+{
+	lw->full_screen_overlay_on = enable;
+
+	if (!lw->full_screen) return;
+
+	if (enable)
+		{
+		if (lw->full_screen_overlay_id == -1)
+			{
+			lw->full_screen_overlay_id = image_overlay_info_enable(lw->image);
+			}
+		}
+	else
+		{
+		if (lw->full_screen_overlay_id != -1)
+			{
+			image_overlay_info_disable(lw->image, lw->full_screen_overlay_id);
+			lw->full_screen_overlay_id = -1;
+			}
+		}
+}
+
 
 /*
  *----------------------------------------------------------------------------
@@ -116,6 +148,12 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 		case '9':
 			layout_image_zoom_set(lw, -2.0);
 			break;
+		case 'W': case 'w':
+			layout_image_zoom_set_fill_geometry(lw, FALSE);
+			break;
+		case 'H': case 'h':
+			layout_image_zoom_set_fill_geometry(lw, TRUE);
+			break;
 		case GDK_Page_Up: case GDK_KP_Page_Up:
 		case GDK_BackSpace:
 		case 'B': case 'b':
@@ -147,7 +185,7 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 		case GDK_Delete: case GDK_KP_Delete:
 			if (enable_delete_key)
 				{
-				file_util_delete(layout_image_get_path(lw), NULL);
+				file_util_delete(layout_image_get_path(lw), NULL, widget);
 				stop_signal = TRUE;
 				}
 			break;
@@ -173,6 +211,9 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 			layout_image_full_screen_menu_popup(lw);
 			stop_signal = TRUE;
 			break;
+		case 'I': case 'i':
+			layout_image_overlay_set(lw, !(lw->full_screen_overlay_on));
+			break;
 		}
 
 	if (event->state & GDK_CONTROL_MASK)
@@ -211,16 +252,16 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 				n = 9;
 				break;
 			case 'C': case 'c':
-				file_util_copy(layout_image_get_path(lw), NULL, NULL);
+				file_util_copy(layout_image_get_path(lw), NULL, NULL, widget);
 				break;
 			case 'M': case 'm':
-				file_util_move(layout_image_get_path(lw), NULL, NULL);
+				file_util_move(layout_image_get_path(lw), NULL, NULL, widget);
 				break;
 			case 'R': case 'r':
-				file_util_rename(layout_image_get_path(lw), NULL);
+				file_util_rename(layout_image_get_path(lw), NULL, widget);
 				break;
 			case 'D': case 'd':
-				file_util_delete(layout_image_get_path(lw), NULL);
+				file_util_delete(layout_image_get_path(lw), NULL, widget);
 				break;
 			case 'P': case 'p':
 				info_window_new(layout_image_get_path(lw), NULL);
@@ -281,6 +322,7 @@ static void layout_image_full_screen_stop_func(FullScreenData *fs, gpointer data
 		}
 
 	lw->full_screen = NULL;
+	lw->full_screen_overlay_id = -1;
 }
 
 void layout_image_full_screen_start(LayoutWindow *lw)
@@ -309,6 +351,8 @@ void layout_image_full_screen_start(LayoutWindow *lw)
 	gtk_widget_set_sensitive(lw->window, FALSE);
 	if (lw->tools) gtk_widget_set_sensitive(lw->tools, FALSE);
 #endif
+
+	layout_image_overlay_set(lw, lw->full_screen_overlay_on);
 }
 
 void layout_image_full_screen_stop(LayoutWindow *lw)
@@ -552,32 +596,54 @@ static void li_pop_menu_new_cb(GtkWidget *widget, gpointer data)
 	view_window_new(layout_image_get_path(lw));
 }
 
+static GtkWidget *li_pop_menu_click_parent(GtkWidget *widget, LayoutWindow *lw)
+{
+	GtkWidget *menu;
+	GtkWidget *parent;
+
+	menu = gtk_widget_get_toplevel(widget);
+	if (!menu) return NULL;
+
+	parent = g_object_get_data(G_OBJECT(menu), "click_parent");
+
+	if (!parent && lw->full_screen)
+		{
+		parent = lw->full_screen->imd->widget;
+		}
+
+	return parent;
+}
+
 static void li_pop_menu_copy_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
-	file_util_copy(layout_image_get_path(lw), NULL, NULL);
+	file_util_copy(layout_image_get_path(lw), NULL, NULL,
+		       li_pop_menu_click_parent(widget, lw));
 }
 
 static void li_pop_menu_move_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
-	file_util_move(layout_image_get_path(lw), NULL, NULL);
+	file_util_move(layout_image_get_path(lw), NULL, NULL,
+		       li_pop_menu_click_parent(widget, lw));
 }
 
 static void li_pop_menu_rename_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
-	file_util_rename(layout_image_get_path(lw), NULL);
+	file_util_rename(layout_image_get_path(lw), NULL,
+			 li_pop_menu_click_parent(widget, lw));
 }
 
 static void li_pop_menu_delete_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
-	file_util_delete(layout_image_get_path(lw), NULL);
+	file_util_delete(layout_image_get_path(lw), NULL,
+			 li_pop_menu_click_parent(widget, lw));
 }
 
 static void li_pop_menu_slide_start_cb(GtkWidget *widget, gpointer data)
@@ -916,6 +982,13 @@ void layout_image_zoom_set(LayoutWindow *lw, gfloat zoom)
 	image_zoom_set(lw->image, zoom);
 }
 
+void layout_image_zoom_set_fill_geometry(LayoutWindow *lw, gint vertical)
+{
+	if (!layout_valid(&lw)) return;
+
+	image_zoom_set_fill_geometry(lw->image, vertical);
+}
+
 void layout_image_alter(LayoutWindow *lw, AlterType type)
 {
 	if (!layout_valid(&lw)) return;
@@ -1199,6 +1272,10 @@ static void layout_image_button_cb(ImageWindow *imd, gint button, guint32 time,
 			break;
 		case 3:
 			menu = layout_image_pop_menu(lw);
+			if (imd == lw->image)
+				{
+				g_object_set_data(G_OBJECT(menu), "click_parent", imd->widget);
+				}
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, time);
 			break;
 		default:
