@@ -13,6 +13,7 @@
 #include "gqview.h"
 #include "layout_util.h"
 
+#include "bar_info.h"
 #include "bar_exif.h"
 #include "bar_sort.h"
 #include "cache_maint.h"
@@ -23,6 +24,8 @@
 #include "info.h"
 #include "layout_image.h"
 #include "preferences.h"
+#include "print.h"
+#include "search.h"
 #include "utilops.h"
 #include "ui_bookmark.h"
 #include "ui_fileops.h"
@@ -95,6 +98,11 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 		{
 		return TRUE;
 		}
+	if (lw->bar_info &&
+	    bar_info_event(lw->bar_info, (GdkEvent *)event))
+		{
+		return TRUE;
+		}
 
 	if (lw->image &&
 	    (GTK_WIDGET_HAS_FOCUS(lw->image->widget) || (lw->tools && widget == lw->window)) )
@@ -178,6 +186,12 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 			layout_image_zoom_set(lw, -2.0);
 			stop_signal = TRUE;
 			break;
+		case 'W': case 'w':
+			layout_image_zoom_set_fill_geometry(lw, FALSE);
+			break;
+		case 'H': case 'h':
+			layout_image_zoom_set_fill_geometry(lw, TRUE);
+			break;
 		case GDK_Page_Up: case GDK_KP_Page_Up:
 			layout_image_prev(lw);
 			stop_signal = TRUE;
@@ -197,7 +211,7 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 		case GDK_Delete: case GDK_KP_Delete:
 			if (enable_delete_key)
 				{
-				file_util_delete(NULL, layout_selection_list(lw));
+				file_util_delete(NULL, layout_selection_list(lw), widget);
 				stop_signal = TRUE;
 				}
 			break;
@@ -265,44 +279,58 @@ static void layout_menu_open_cb(gpointer data, guint action, GtkWidget *widget)
 	collection_dialog_load(NULL);
 }
 
+static void layout_menu_search_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	search_new(lw->path, NULL);
+}
+
 static void layout_menu_dupes_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	dupe_window_new(DUPE_MATCH_NAME);
 }
 
+static void layout_menu_print_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	print_window_new(layout_image_get_path(lw), layout_selection_list(lw), layout_list(lw), lw->window);
+}
+
 static void layout_menu_dir_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
 
-	file_util_create_dir(lw->path);
+	file_util_create_dir(lw->path, lw->window);
 }
 
 static void layout_menu_copy_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
 
-	file_util_copy(NULL, layout_selection_list(lw), NULL);
+	file_util_copy(NULL, layout_selection_list(lw), NULL, lw->window);
 }
 
 static void layout_menu_move_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
 
-	file_util_move(NULL, layout_selection_list(lw), NULL);
+	file_util_move(NULL, layout_selection_list(lw), NULL, lw->window);
 }
 
 static void layout_menu_rename_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
 
-	file_util_rename(NULL, layout_selection_list(lw));
+	file_util_rename(NULL, layout_selection_list(lw), lw->window);
 }
 
 static void layout_menu_delete_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
 
-	file_util_delete(NULL, layout_selection_list(lw));
+	file_util_delete(NULL, layout_selection_list(lw), lw->window);
 }
 
 static void layout_menu_close_cb(gpointer data, guint action, GtkWidget *widget)
@@ -361,7 +389,6 @@ static void layout_menu_info_cb(gpointer data, guint action, GtkWidget *widget)
 	list = layout_selection_list(lw);
 	if (!list) path = layout_image_get_path(lw);
 
-	layout_image_full_screen_stop(lw);
 	info_window_new(path, list);
 }
 
@@ -497,6 +524,16 @@ static void layout_menu_toolbar_cb(gpointer data, guint action, GtkWidget *widge
 		}
 }
 
+static void layout_menu_bar_info_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	if (lw->bar_info_enabled != GTK_CHECK_MENU_ITEM(widget)->active)
+		{
+		layout_bar_info_toggle(lw);
+		}
+}
+
 static void layout_menu_bar_exif_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
@@ -525,6 +562,11 @@ static void layout_menu_slideshow_cb(gpointer data, guint action, GtkWidget *wid
 }
 
 static void layout_menu_help_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	help_window_show("html_contents");
+}
+
+static void layout_menu_help_keys_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	help_window_show("documentation");
 }
@@ -700,8 +742,10 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 	{ N_("/File/_Open collection..."),	"O",		layout_menu_open_cb,		0, "<StockItem>", GTK_STOCK_OPEN },
 	{ N_("/File/Open _recent"),		NULL,		NULL,				0, NULL },
 	{ N_("/File/sep1"),			NULL,		NULL,				0, "<Separator>" },
+	{ N_("/File/_Search..."),		"F3",		layout_menu_search_cb,		0, "<StockItem>", GTK_STOCK_FIND },
 	{ N_("/File/_Find duplicates..."),	"D",		layout_menu_dupes_cb,		0, "<StockItem>", GTK_STOCK_FIND },
 	{ N_("/File/sep2"),			NULL,		NULL,				0, "<Separator>" },
+	{ N_("/File/_Print..."),		"<shift>P",		layout_menu_print_cb,		0, "<StockItem>", GTK_STOCK_PRINT },
 	{ N_("/File/N_ew folder..."),		"<control>F",	layout_menu_dir_cb,		0, NULL },
 	{ N_("/File/sep3"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/File/_Copy..."),			"<control>C",	layout_menu_copy_cb,		0, "<StockItem>", GTK_STOCK_COPY },
@@ -760,9 +804,10 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 
 	{ N_("/View/sep3"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/View/_Float file list"),		"L",		layout_menu_float_cb,		0, "<CheckItem>" },
-	{ N_("/View/_Hide file list"),		"H",		layout_menu_hide_cb,		0, NULL },
+	{ N_("/View/_Hide file list"),		"<control>H",	layout_menu_hide_cb,		0, NULL },
 	{ N_("/View/Hide tool_bar"),		NULL,		layout_menu_toolbar_cb,		0, "<CheckItem>" },
 	{ N_("/View/sep4"),			NULL,		NULL,				0, "<Separator>" },
+	{ N_("/View/_Keywords"),		"<control>K",	layout_menu_bar_info_cb,	0, "<CheckItem>" },
 	{ N_("/View/E_xif data"),		"<control>E",	layout_menu_bar_exif_cb,	0, "<CheckItem>" },
 	{ N_("/View/Sort _manager"),		"<control>S",	layout_menu_bar_sort_cb,	0, "<CheckItem>" },
 	{ N_("/View/sep5"),			NULL,		NULL,				0, "<Separator>" },
@@ -771,8 +816,9 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 
 	{ N_("/_Help"),				NULL,		NULL,				0, "<Branch>" },
 	{ N_("/Help/tear1"),			NULL,		NULL,				0, "<Tearoff>" },
-	{ N_("/Help/_Keyboard shortcuts"),	"F1",		layout_menu_help_cb,		0, "<StockItem>", GTK_STOCK_HELP },
-	{ N_("/Help/_Release notes"),		NULL,		layout_menu_notes_cb,		0, NULL},
+	{ N_("/Help/_Contents"),		"F1",		layout_menu_help_cb,		0, "<StockItem>", GTK_STOCK_HELP },
+	{ N_("/Help/_Keyboard shortcuts"),	NULL,		layout_menu_help_keys_cb,	0, NULL },
+	{ N_("/Help/_Release notes"),		NULL,		layout_menu_notes_cb,		0, NULL },
 	{ N_("/Help/sep1"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/Help/_About"),			NULL,		layout_menu_about_cb,		0, NULL },
 };
@@ -916,7 +962,7 @@ GtkWidget *layout_button_bar(LayoutWindow *lw)
 	lw->thumb_button = layout_button(box, (gchar **)icon_thumb_xpm, NULL, TRUE,
 		      tooltips, _("Show thumbnails"), G_CALLBACK(layout_button_thumb_cb), lw);
 	layout_button(box, NULL, GTK_STOCK_HOME, FALSE,
-		      tooltips, _("Change to home directory"), G_CALLBACK(layout_button_home_cb), lw);
+		      tooltips, _("Change to home folder"), G_CALLBACK(layout_button_home_cb), lw);
 	layout_button(box, NULL, GTK_STOCK_REFRESH, FALSE,
 		      tooltips, _("Refresh file list"), G_CALLBACK(layout_button_refresh_cb), lw);
 	layout_button(box, NULL, GTK_STOCK_ZOOM_IN, FALSE,
@@ -956,6 +1002,9 @@ static void layout_util_sync_views(LayoutWindow *lw)
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Float file list");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->tools_float);
 
+	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Keywords");
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->bar_info_enabled);
+
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Exif data");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->bar_exif_enabled);
 
@@ -1038,6 +1087,79 @@ void folder_icons_free(PixmapFolders *pf)
 
 #define SIDEBAR_WIDTH 288
 
+static void layout_bar_info_destroyed(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	lw->bar_info = NULL;
+
+	if (lw->utility_box)
+		{
+		/* destroyed from within itself */
+		lw->bar_info_enabled = FALSE;
+		layout_util_sync_views(lw);
+		}
+}
+
+static GList *layout_bar_info_list_cb(gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	return layout_selection_list(lw);
+}
+
+static void layout_bar_info_new(LayoutWindow *lw)
+{
+	if (!lw->utility_box) return;
+                                                                                                                    
+	lw->bar_info = bar_info_new(layout_image_get_path(lw), FALSE, lw->utility_box);
+	bar_info_set_selection_func(lw->bar_info, layout_bar_info_list_cb, lw);
+	bar_info_selection(lw->bar_info, layout_selection_count(lw, NULL) - 1);
+	bar_info_size_request(lw->bar_info, SIDEBAR_WIDTH * 3 / 4);
+	g_signal_connect(G_OBJECT(lw->bar_info), "destroy",
+			 G_CALLBACK(layout_bar_info_destroyed), lw);
+	lw->bar_info_enabled = TRUE;
+
+	gtk_box_pack_start(GTK_BOX(lw->utility_box), lw->bar_info, FALSE, FALSE, 0);
+	gtk_widget_show(lw->bar_info);
+}
+                                                                                                                    
+static void layout_bar_info_close(LayoutWindow *lw)
+{
+	if (lw->bar_info)
+		{
+		bar_info_close(lw->bar_info);
+		lw->bar_info = NULL;
+		}
+	lw->bar_info_enabled = FALSE;
+}
+
+void layout_bar_info_toggle(LayoutWindow *lw)
+{
+	if (lw->bar_info_enabled)
+		{
+		layout_bar_info_close(lw);
+		}
+	else
+		{
+		layout_bar_info_new(lw);
+		}
+}
+
+static void layout_bar_info_new_image(LayoutWindow *lw)
+{
+	if (!lw->bar_info || !lw->bar_info_enabled) return;
+
+	bar_info_set(lw->bar_info, layout_image_get_path(lw));
+}
+
+static void layout_bar_info_new_selection(LayoutWindow *lw, gint count)
+{
+	if (!lw->bar_info || !lw->bar_info_enabled) return;
+
+	bar_info_selection(lw->bar_info, count - 1);
+}
+
 static void layout_bar_exif_destroyed(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
@@ -1066,11 +1188,12 @@ static void layout_bar_exif_sized(GtkWidget *widget, GtkAllocation *allocation,
 		}
 }
 
-void layout_bar_exif_new(LayoutWindow *lw)
+static void layout_bar_exif_new(LayoutWindow *lw)
 {
 	if (!lw->utility_box) return;
 
-	lw->bar_exif = bar_exif_new(TRUE, layout_image_get_path(lw), lw->bar_exif_advanced);
+	lw->bar_exif = bar_exif_new(TRUE, layout_image_get_path(lw),
+				    lw->bar_exif_advanced, lw->utility_box);
 	g_signal_connect(G_OBJECT(lw->bar_exif), "destroy",
 			 G_CALLBACK(layout_bar_exif_destroyed), lw);
 	g_signal_connect(G_OBJECT(lw->bar_exif), "size_allocate",
@@ -1080,10 +1203,11 @@ void layout_bar_exif_new(LayoutWindow *lw)
 	if (lw->bar_exif_size < 1) lw->bar_exif_size = SIDEBAR_WIDTH;
 	gtk_widget_set_size_request(lw->bar_exif, lw->bar_exif_size, -1);
 	gtk_box_pack_start(GTK_BOX(lw->utility_box), lw->bar_exif, FALSE, FALSE, 0);
+	if (lw->bar_info) gtk_box_reorder_child(GTK_BOX(lw->utility_box), lw->bar_exif, 1);
 	gtk_widget_show(lw->bar_exif);
 }
 
-void layout_bar_exif_close(LayoutWindow *lw)
+static void layout_bar_exif_close(LayoutWindow *lw)
 {
 	if (lw->bar_exif)
 		{
@@ -1127,7 +1251,7 @@ static void layout_bar_sort_destroyed(GtkWidget *widget, gpointer data)
 		}
 }
 
-void layout_bar_sort_new(LayoutWindow *lw)
+static void layout_bar_sort_new(LayoutWindow *lw)
 {
 	if (!lw->utility_box) return;
 
@@ -1140,7 +1264,7 @@ void layout_bar_sort_new(LayoutWindow *lw)
 	gtk_widget_show(lw->bar_sort);
 }
 
-void layout_bar_sort_close(LayoutWindow *lw)
+static void layout_bar_sort_close(LayoutWindow *lw)
 {
 	if (lw->bar_sort)
 		{
@@ -1164,9 +1288,15 @@ void layout_bar_sort_toggle(LayoutWindow *lw)
 
 void layout_bars_new_image(LayoutWindow *lw)
 {
+	layout_bar_info_new_image(lw);
 	layout_bar_exif_new_image(lw);
 }
 
+void layout_bars_new_selection(LayoutWindow *lw, gint count)
+{
+	layout_bar_info_new_selection(lw, count);
+}
+
 GtkWidget *layout_bars_prepare(LayoutWindow *lw, GtkWidget *image)
 {
 	lw->utility_box = gtk_hbox_new(FALSE, 0);
@@ -1178,6 +1308,11 @@ GtkWidget *layout_bars_prepare(LayoutWindow *lw, GtkWidget *image)
 		layout_bar_sort_new(lw);
 		}
 
+	if (lw->bar_info_enabled)
+		{
+		layout_bar_info_new(lw);
+		}
+
 	if (lw->bar_exif_enabled)
 		{
 		layout_bar_exif_new(lw);
@@ -1190,4 +1325,6 @@ void layout_bars_close(LayoutWindow *lw)
 {
 	layout_bar_sort_close(lw);
 	layout_bar_exif_close(lw);
+	layout_bar_info_close(lw);
 }
+
