@@ -40,15 +40,13 @@ PixmapFolders *folder_icons_new(void);
 void folder_icons_free(PixmapFolders *pf);
 
 
-void layout_bar_exif_new(LayoutWindow *lw);
-void layout_bar_exif_close(LayoutWindow *lw);
+void layout_bar_info_toggle(LayoutWindow *lw);
 void layout_bar_exif_toggle(LayoutWindow *lw);
-
-void layout_bar_sort_new(LayoutWindow *lw);
-void layout_bar_sort_close(LayoutWindow *lw);
 void layout_bar_sort_toggle(LayoutWindow *lw);
 
 void layout_bars_new_image(LayoutWindow *lw);
+void layout_bars_new_selection(LayoutWindow *lw, gint count);
+
 GtkWidget *layout_bars_prepare(LayoutWindow *lw, GtkWidget *image);
 void layout_bars_close(LayoutWindow *lw);
 
