@@ -16,9 +16,12 @@
 #include "cache_maint.h"
 #include "editors.h"
 #include "filelist.h"
+#include "fullscreen.h"
+#include "globals.h"
 #include "img-view.h"
 #include "layout_config.h"
 #include "layout_util.h"
+#include "slideshow.h"
 #include "utilops.h"
 #include "ui_fileops.h"
 #include "ui_tabcomp.h"
@@ -26,6 +29,8 @@
 
 #include "icons/config.xpm"
 
+#include <math.h>
+
 
 #define EDITOR_NAME_MAX_LENGTH 32
 #define EDITOR_COMMAND_MAX_LENGTH 1024
@@ -110,6 +115,10 @@ static gint place_dialogs_under_mouse_c;
 
 static gint black_window_background_c;
 
+static gint fullscreen_screen_c;
+static gint fullscreen_clean_flip_c;
+static gint fullscreen_disable_saver_c;
+
 static gint dupe_custom_threshold_c;
 
 static GtkWidget *configwindow = NULL;
@@ -162,6 +171,11 @@ static void zoom_increment_cb(GtkObject *adj, gpointer data)
 	zoom_increment_c = (gint)(GTK_ADJUSTMENT(adj)->value * 10.0 + 0.01);
 }
 
+static void slideshow_delay_cb(GtkAdjustment *adj, gpointer data)
+{
+	slideshow_delay_c = (gint)(gtk_adjustment_get_value(adj) * (double)SLIDESHOW_SUBSECOND_PRECISION + 0.01);
+}
+
 static void dummy_cancel_cb(GenericDialog *gd, gpointer data)
 {
 	/* no op, only so cancel button appears */
@@ -177,7 +191,8 @@ static void cache_clear_cb(GtkWidget *widget, gpointer data)
 	GenericDialog *gd;
 
 	gd = generic_dialog_new(_("Clear cache"), _("This will remove all thumbnails that have\nbeen saved to disk, continue?"),
-				"GQview", "clear_cache", TRUE, dummy_cancel_cb, NULL);
+				"GQview", "clear_cache", widget, TRUE,
+				dummy_cancel_cb, NULL);
 	generic_dialog_add_stock(gd, _("Ok"), GTK_STOCK_OK, cache_clear_ok_cb, TRUE);
 
 	gtk_widget_show(gd->dialog);
@@ -280,6 +295,10 @@ static void config_window_apply(void)
 		view_window_colors_update();
 		}
 
+	fullscreen_screen = fullscreen_screen_c;
+	fullscreen_clean_flip = fullscreen_clean_flip_c;
+	fullscreen_disable_saver = fullscreen_disable_saver_c;
+
 	update_on_time_change = update_on_time_change_c;
 	exif_rotate_enable = exif_rotate_enable_c;
 
@@ -728,7 +747,8 @@ static void filter_default_cb(GtkWidget *widget, gpointer data)
 	GenericDialog *gd;
 
 	gd = generic_dialog_new(_("Reset filters"), _("This will reset the file filters to the defaults.\nContinue?"),
-				"GQview", "reset_filter", TRUE, dummy_cancel_cb, NULL);
+				"GQview", "reset_filter", widget, TRUE,
+				dummy_cancel_cb, NULL);
 	generic_dialog_add_stock(gd, _("Ok"), GTK_STOCK_OK, filter_default_ok_cb, TRUE);
 	gtk_widget_show(gd->dialog);
 }
@@ -762,7 +782,8 @@ static void editor_default_cb(GtkWidget *widget, gpointer data)
 	GenericDialog *gd;
 
 	gd = generic_dialog_new(_("Reset editors"), _("This will reset the edit commands to the defaults.\nContinue?"),
-				"GQview", "reset_filter", TRUE, dummy_cancel_cb, NULL);
+				"GQview", "reset_filter", widget, TRUE,
+				dummy_cancel_cb, NULL);
 	generic_dialog_add_stock(gd, _("Ok"), GTK_STOCK_OK, editor_default_ok_cb, TRUE);
 	gtk_widget_show(gd->dialog);
 }
@@ -787,7 +808,8 @@ static void safe_delete_clear_cb(GtkWidget* widget, gpointer data)
 	GenericDialog *gd;
 	GtkWidget *entry;
 	gd = generic_dialog_new(_("Clear trash"), _("This will remove the trash contents."),
-				"GQview", "clear_trash", TRUE, dummy_cancel_cb, NULL);
+				"GQview", "clear_trash", widget, TRUE,
+				dummy_cancel_cb, NULL);
 	generic_dialog_add_stock(gd, _("Ok"), GTK_STOCK_OK, safe_delete_clear_ok_cb, TRUE);
 	entry = gtk_entry_new();
 	gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
@@ -874,7 +896,7 @@ static void config_window_create(void)
 	label = gtk_label_new(_("General"));
 	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
 
-	frame = gtk_frame_new(_("Initial directory"));
+	frame = gtk_frame_new(_("Initial folder"));
 	gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
 	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
@@ -884,7 +906,7 @@ static void config_window_create(void)
 	gtk_widget_show(vbox1);
 
 	ct_button = add_check_button(startup_path_enable, &startup_path_enable_c,
-				      _("On startup, change to this directory:"), vbox1);
+				      _("On startup, change to this folder:"), vbox1);
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
@@ -894,6 +916,7 @@ static void config_window_create(void)
 	gtk_widget_realize(configwindow);
 
 	tabcomp = tab_completion_new(&startup_path_entry, startup_path, NULL, NULL);
+	tab_completion_add_select_button(startup_path_entry, NULL, TRUE);
 	gtk_box_pack_start(GTK_BOX(hbox), tabcomp, TRUE, TRUE, 0);
 	gtk_widget_show(tabcomp);
 
@@ -944,9 +967,24 @@ static void config_window_create(void)
 	gtk_container_add (GTK_CONTAINER(frame),vbox1);
 	gtk_widget_show(vbox1);
 
-	add_spin_button(_("Delay before image change (seconds):"), slideshow_delay, &slideshow_delay_c,
-			1.0, 1200.0, 1.0, 0, vbox1);
-	
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+                                                                                                                         
+	label = gtk_label_new(_("Delay before image change (seconds):"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+	gtk_widget_show(label);
+
+	slideshow_delay_c = slideshow_delay;
+	adj = gtk_adjustment_new(slideshow_delay_c ? (double)slideshow_delay_c / SLIDESHOW_SUBSECOND_PRECISION : 10.0,
+				 SLIDESHOW_MIN_SECONDS, SLIDESHOW_MAX_SECONDS, 0.1, 1.0, 1.0);
+	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, (int)log10(SLIDESHOW_SUBSECOND_PRECISION));
+	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 5);
+	g_signal_connect(G_OBJECT(adj), "value_changed",
+			 G_CALLBACK(slideshow_delay_cb), NULL);
+	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
+	gtk_widget_show(spin);
+
 	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
@@ -1275,6 +1313,16 @@ static void config_window_create(void)
 	label = gtk_label_new(_("Advanced"));
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);
 
+	fullscreen_screen_c = fullscreen_screen;
+	hbox = fullscreen_prefs_selection_new(_("Full screen size:"), &fullscreen_screen_c);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	add_check_button(fullscreen_clean_flip, &fullscreen_clean_flip_c,
+                         _("Smooth image flip in full screen"), vbox);
+	add_check_button(fullscreen_disable_saver, &fullscreen_disable_saver_c,
+			_("Disable screen saver in full screen"), vbox);
+
 	frame = gtk_frame_new(NULL);
 	gtk_container_set_border_width(GTK_CONTAINER (frame), 5);
 	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
@@ -1302,9 +1350,18 @@ static void config_window_create(void)
 	gtk_widget_show(label);
 
 	tabcomp = tab_completion_new(&safe_delete_path_entry, safe_delete_path, NULL, NULL);
+	tab_completion_add_select_button(safe_delete_path_entry, NULL, TRUE);
 	gtk_box_pack_start(GTK_BOX(hbox), tabcomp, TRUE, TRUE, 0);
 	gtk_widget_show(tabcomp);
 
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+	add_check_button_sensitivity(ct_button, hbox);
+
+	button = add_spin_button(_("Maximum size (MB)"), safe_delete_size, &safe_delete_size_c,
+				 1.0, 500.0, 1.0, 0, hbox);
+
 	button = gtk_button_new_with_label (_("View"));
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(safe_delete_view_cb), NULL);
@@ -1317,10 +1374,6 @@ static void config_window_create(void)
 	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
-	button = add_spin_button(_("Maximum size (MB)"), safe_delete_size, &safe_delete_size_c,
-				 1.0, 500.0, 1.0, 0, vbox1);
-	add_check_button_sensitivity(ct_button, button);
-
 	add_check_button(exif_rotate_enable, &exif_rotate_enable_c,
 			 _("Auto rotate image using Exif information"), vbox);
 
