@@ -14,6 +14,7 @@
 #include "rcfile.h"
 
 #include "filelist.h"
+#include "slideshow.h"
 #include "ui_fileops.h"
 
 
@@ -89,7 +90,53 @@ static gint read_int_option(FILE *f, gchar *option, gchar *label, gchar *value,
 {
 	if (strcasecmp(option, label) == 0)
 		{
-		n = strtol(value,NULL,0);
+		n = strtol(value, NULL, 10);
+		}
+	return n;
+}
+
+static void write_int_unit_option(FILE *f, gchar *label, gint n, gint subunits)
+{
+	gint l, r;
+
+	if (subunits > 0)
+		{
+		l = n / subunits;
+		r = n % subunits;
+		}
+	else
+		{
+		l = n;
+		r = 0;
+		}
+
+	fprintf(f,"%s: %d.%d\n\n", label, l, r);
+}
+
+static gint read_int_unit_option(FILE *f, gchar *option, gchar *label, gchar *value, gint n, gint subunits)
+{
+	if (strcasecmp(option, label) == 0)
+		{
+		gint l, r;
+		gchar *ptr;
+
+		ptr = value;
+		while (*ptr != '\0' && *ptr != '.') ptr++;
+		if (*ptr == '.')
+			{
+			*ptr = '\0';
+			l = strtol(value, NULL, 10);
+			*ptr = '.';
+			ptr++;
+			r = strtol(ptr, NULL, 10);
+			}
+		else
+			{
+			l = strtol(value, NULL, 10);
+			r = 0;
+			}
+
+		n = l * subunits + r;
 		}
 	return n;
 }
@@ -145,10 +192,6 @@ void save_options(void)
 
 	fprintf(f,"######################################################################\n");
 	fprintf(f,"#                         GQview config file         version %7s #\n", VERSION);
-	fprintf(f,"#                                                                    #\n");
-	fprintf(f,"#  Everything in this file can be changed in the option dialogs.     #\n");
-	fprintf(f,"#      (so there should be no need to edit this file by hand)        #\n");
-	fprintf(f,"#                                                                    #\n");
 	fprintf(f,"######################################################################\n");
 	fprintf(f,"\n");
 	fprintf(f,"##### General Options #####\n\n");
@@ -187,6 +230,7 @@ void save_options(void)
 	fprintf(f,"\n");
 
 	write_bool_option(f, "progressive_keyboard_scrolling", progressive_key_scrolling);
+	write_int_option(f, "scroll_reset_method", scroll_reset_method);
 	fprintf(f,"\n");
 
 	write_bool_option(f, "enable_thumbnails", thumbnails_enabled);
@@ -233,11 +277,15 @@ void save_options(void)
 
 	write_bool_option(f, "black_window_background", black_window_background);
 
+	write_int_option(f, "fullscreen_screen", fullscreen_screen);
+	write_bool_option(f, "fullscreen_clean_flip", fullscreen_clean_flip);
+	write_bool_option(f, "fullscreen_disable_saver", fullscreen_disable_saver);
+
 	write_int_option(f, "custom_similarity_threshold", dupe_custom_threshold);
 
 	fprintf(f,"\n##### Slideshow Options #####\n\n");
 
-	write_int_option(f, "slideshow_delay", slideshow_delay);
+	write_int_unit_option(f, "slideshow_delay", slideshow_delay, SLIDESHOW_SUBSECOND_PRECISION);
 
 	write_bool_option(f, "slideshow_random", slideshow_random);
 	write_bool_option(f, "slideshow_repeat", slideshow_repeat);
@@ -273,6 +321,7 @@ void save_options(void)
 	write_int_option(f, "main_window_y", main_window_y);
 	write_int_option(f, "main_window_width", main_window_w);
 	write_int_option(f, "main_window_height", main_window_h);
+	write_bool_option(f, "main_window_maximized", main_window_maximized);
 	write_int_option(f, "float_window_x", float_window_x);
 	write_int_option(f, "float_window_y", float_window_y);
 	write_int_option(f, "float_window_width", float_window_w);
@@ -383,6 +432,8 @@ void load_options(void)
 			"max_window_size", value, max_window_size);
 		progressive_key_scrolling = read_bool_option(f, option,
 			"progressive_keyboard_scrolling", value, progressive_key_scrolling);
+		scroll_reset_method = read_int_option(f, option,
+			"scroll_reset_method", value, scroll_reset_method);
 
 		thumbnails_enabled = read_bool_option(f, option,
 			"enable_thumbnails", value, thumbnails_enabled);
@@ -455,13 +506,20 @@ void load_options(void)
 		black_window_background = read_bool_option(f, option,
 			"black_window_background", value, black_window_background);
 
+		fullscreen_screen = read_int_option(f, option,
+			"fullscreen_screen", value, fullscreen_screen);
+		fullscreen_clean_flip = read_bool_option(f, option,
+			"fullscreen_clean_flip", value, fullscreen_clean_flip);
+		fullscreen_disable_saver = read_bool_option(f, option,
+			"fullscreen_disable_saver", value, fullscreen_disable_saver);
+
 		dupe_custom_threshold = read_int_option(f, option,
 			"custom_similarity_threshold", value, dupe_custom_threshold);
 
 		/* slideshow options */
 
-		slideshow_delay = read_int_option(f, option,
-			"slideshow_delay", value, slideshow_delay);
+		slideshow_delay = read_int_unit_option(f, option,
+			"slideshow_delay", value, slideshow_delay, SLIDESHOW_SUBSECOND_PRECISION);
 		slideshow_random = read_bool_option(f, option,
 			"slideshow_random", value, slideshow_random);
 		slideshow_repeat = read_bool_option(f, option,
@@ -545,6 +603,8 @@ void load_options(void)
 			"main_window_width", value, main_window_w);
 		main_window_h = read_int_option(f, option,
 			"main_window_height", value, main_window_h);
+		main_window_maximized = read_bool_option(f, option,
+			"main_window_maximized", value, main_window_maximized);
 		float_window_x = read_int_option(f, option,
 			"float_window_x", value, float_window_x);
 		float_window_y = read_int_option(f, option,
