@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2001 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -35,5 +35,10 @@ gfloat image_sim_compare(ImageSimilarityData *a, ImageSimilarityData *b);
 gfloat image_sim_compare_fast(ImageSimilarityData *a, ImageSimilarityData *b, gfloat min);
 
 
+void image_sim_alternate_set(gint enable);
+gint image_sim_alternate_enabled(void);
+void image_sim_alternate_processing(ImageSimilarityData *sd);
+
+
 #endif
 
