@@ -269,8 +269,11 @@ static void slideshow_timer_reset(SlideShowData *ss, gint reset)
 {
 	if (reset)
 		{
+		if (slideshow_delay < 1) slideshow_delay = 1;
+
 		if (ss->timeout_id != -1) gtk_timeout_remove(ss->timeout_id);
-		ss->timeout_id = gtk_timeout_add(slideshow_delay * 1000, slideshow_loop_cb, ss);
+		ss->timeout_id = gtk_timeout_add(slideshow_delay * 1000 / SLIDESHOW_SUBSECOND_PRECISION,
+						 slideshow_loop_cb, ss);
 		}
 	else if (ss->timeout_id != -1)
 		{
