@@ -14,6 +14,7 @@
 #include "thumb.h"
 
 #include "cache.h"
+#include "globals.h"
 #include "image-load.h"
 #include "pixbuf_util.h"
 #include "ui_fileops.h"
@@ -42,7 +43,7 @@ static gint thumb_loader_save_to_cache(ThumbLoader *tl)
 
 	if (!tl || !tl->pixbuf) return FALSE;
 
-	cache_dir = cache_get_location(tl->path, FALSE, NULL, &mode);
+	cache_dir = cache_get_location(CACHE_TYPE_THUMB, tl->path, FALSE, &mode);
 
 	if (cache_ensure_dir_exists(cache_dir, mode))
 		{
@@ -50,7 +51,7 @@ static gint thumb_loader_save_to_cache(ThumbLoader *tl)
 		gchar *pathl;
 
 		cache_path = g_strconcat(cache_dir, "/", filename_from_path(tl->path),
-					 GQVIEW_CACHE_THUMB_EXT, NULL);
+					 GQVIEW_CACHE_EXT_THUMB, NULL);
 
 		if (debug) printf("Saving thumb: %s\n", cache_path);
 
@@ -89,7 +90,7 @@ static gint thumb_loader_mark_failure(ThumbLoader *tl)
 	
 	if (!tl) return FALSE;
 
-	cache_dir = cache_get_location(tl->path, FALSE, NULL, &mode);
+	cache_dir = cache_get_location(CACHE_TYPE_THUMB, tl->path, FALSE, &mode);
 
 	if (cache_ensure_dir_exists(cache_dir, mode))
 		{
@@ -98,7 +99,7 @@ static gint thumb_loader_mark_failure(ThumbLoader *tl)
 		FILE *f;
 
 		cache_path = g_strconcat(cache_dir, "/", filename_from_path(tl->path),
-					 GQVIEW_CACHE_THUMB_EXT, NULL);
+					 GQVIEW_CACHE_EXT_THUMB, NULL);
 
 		if (debug) printf("marking thumb failure: %s\n", cache_path);
 
@@ -160,11 +161,13 @@ static void thumb_loader_done_cb(ImageLoader *il, gpointer data)
 		if (debug) printf("thumbnail size mismatch, regenerating: %s\n", tl->path);
 		tl->from_cache = FALSE;
 
-		image_loader_free(tl->il);
 		thumb_loader_setup(tl, tl->path);
 
 		if (!image_loader_start(tl->il, thumb_loader_done_cb, tl))
 			{
+			image_loader_free(tl->il);
+			tl->il = NULL;
+
 			if (debug) printf("regeneration failure: %s\n", tl->path);
 			thumb_loader_error_cb(tl->il, tl);
 			}
@@ -211,22 +214,21 @@ static void thumb_loader_done_cb(ImageLoader *il, gpointer data)
 
 static void thumb_loader_error_cb(ImageLoader *il, gpointer data)
 {
-	/* at least some of the image must be available, go to done_cb
-	 * the error handling should be settable to this or the #if 0 below, I guess
-	 */
-
-	thumb_loader_done_cb(il, data);
-
-#if 0
 	ThumbLoader *tl = data;
 
+	/* if at least some of the image is available, go to done_cb */
+	if (image_loader_get_pixbuf(tl->il) != NULL)
+		{
+		thumb_loader_done_cb(il, data);
+		return;
+		}
+
 	if (debug) printf("thumb error: %s\n", tl->path);
 
 	image_loader_free(tl->il);
 	tl->il = NULL;
 
 	if (tl->func_error) tl->func_error(tl, tl->data_error);
-#endif
 }
 
 static gint thumb_loader_done_delay_cb(gpointer data)
@@ -247,6 +249,7 @@ static void thumb_loader_delay_done(ThumbLoader *tl)
 
 static void thumb_loader_setup(ThumbLoader *tl, gchar *path)
 {
+	image_loader_free(tl->il);
 	tl->il = image_loader_new(path);
 
 	image_loader_set_error_func(tl->il, thumb_loader_error_cb, tl);
@@ -284,7 +287,7 @@ gint thumb_loader_start(ThumbLoader *tl, void (*func_done)(ThumbLoader *, gpoint
 
 	if (enable_thumb_caching)
 		{
-		cache_path = cache_find_location(tl->path, GQVIEW_CACHE_THUMB_EXT);
+		cache_path = cache_find_location(CACHE_TYPE_THUMB, tl->path);
 
 		if (cache_path)
 			{
@@ -337,7 +340,7 @@ gint thumb_loader_start(ThumbLoader *tl, void (*func_done)(ThumbLoader *, gpoint
 			{
 			tl->from_cache = FALSE;
 			print_term(_("Thumbnail image in cache failed to load, trying to recreate.\n"));
-			image_loader_free(tl->il);
+
 			thumb_loader_setup(tl, tl->path);
 			if (image_loader_start(tl->il, thumb_loader_done_cb, tl)) return TRUE;
 			}
