@@ -271,9 +271,11 @@ struct _ImageWindow
 
 	void (*func_update)(ImageWindow *, gpointer);
 	void (*func_complete)(ImageWindow *, gint preload, gpointer);
+	void (*func_new)(ImageWindow *, gpointer);
 
 	gpointer data_update;
 	gpointer data_complete;
+	gpointer data_new;
 
 	/* button, scroll functions */
 	void (*func_button)(ImageWindow *, gint button,
@@ -299,6 +301,19 @@ struct _ImageWindow
 
 	gint auto_refresh_id;
 	gint auto_refresh_interval;
+
+	gint delay_flip;
+
+	gint scroller_id;
+	gint scroller_overlay;
+	gint scroller_x;
+	gint scroller_y;
+	gint scroller_xpos;
+	gint scroller_ypos;
+	gint scroller_xinc;
+	gint scroller_yinc;
+
+	GList *overlay_list;
 };
 
 struct _FileData {
@@ -393,6 +408,8 @@ struct _LayoutWindow
 	/* full screen */
 
 	FullScreenData *full_screen;
+	gint full_screen_overlay_id;
+	gint full_screen_overlay_on;
 
 	/* dividers */
 
@@ -410,9 +427,11 @@ struct _LayoutWindow
 	GtkWidget *utility_box;
 	GtkWidget *bar_sort;
 	GtkWidget *bar_exif;
+	GtkWidget *bar_info;
 
 	gint bar_sort_enabled;
 	gint bar_exif_enabled;
+	gint bar_info_enabled;
 
 	gint bar_exif_size;
 	gint bar_exif_advanced;
@@ -594,6 +613,10 @@ struct _FullScreenData
 	ImageWindow *normal_imd;
 
 	gint hide_mouse_id;
+	gint busy_mouse_id;
+	gint cursor_state;
+
+	gint saver_block_id;
 
 	void (*stop_func)(FullScreenData *, gpointer);
 	gpointer stop_data;
