@@ -568,7 +568,7 @@ static void bookmark_edit_ok_cb(GenericDialog *gd, gpointer data)
 
 /* simply pass NULL for text to turn this into a 'new bookmark' dialog */
 
-static void bookmark_edit(const gchar *key, const gchar *text)
+static void bookmark_edit(const gchar *key, const gchar *text, GtkWidget *parent)
 {
 	BookPropData *p;
 	GenericDialog *gd;
@@ -583,7 +583,8 @@ static void bookmark_edit(const gchar *key, const gchar *text)
 	p->bb = bookmark_from_string(text);
 	p->bb->parent = g_strdup(key);
 
-	gd = generic_dialog_new(_("Edit Bookmark"), NULL, "SLIK", "bookmark_edit", TRUE,
+	gd = generic_dialog_new(_("Edit Bookmark"), NULL, "SLIK", "bookmark_edit",
+				parent, TRUE,
 				bookmark_edit_cancel_cb, p);
 	g_signal_connect(G_OBJECT(gd->dialog), "destroy",
 			 G_CALLBACK(bookmark_edit_destroy_cb), p);
@@ -614,6 +615,7 @@ static void bookmark_edit(const gchar *key, const gchar *text)
 	gtk_widget_show(label);
 
 	label = tab_completion_new(&p->path_entry, p->bb->path, NULL, NULL);
+	tab_completion_add_select_button(p->path_entry, NULL, TRUE);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 1, 2);
 	generic_dialog_attach_default(gd, p->path_entry);
 	gtk_widget_show(label);
@@ -627,6 +629,7 @@ static void bookmark_edit(const gchar *key, const gchar *text)
 	if (!icon) icon = "";
 	label = tab_completion_new_with_history(&p->icon_entry, icon,
 		       				"bookmark_icons", -1, NULL, NULL);
+	tab_completion_add_select_button(p->icon_entry, _("Select icon"), FALSE);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 2, 3);
 	generic_dialog_attach_default(gd, p->icon_entry);
 	gtk_widget_show(label);
@@ -664,7 +667,8 @@ static void bookmark_menu_prop_cb(GtkWidget *widget, gpointer data)
 	BookMarkData *bm = data;
 
 	if (!bm->active_button) return;
-	bookmark_edit(bm->key, bm->active_button->key);
+
+	bookmark_edit(bm->key, bm->active_button->key, widget);
 }
 
 static void bookmark_menu_move(BookMarkData *bm, gint direction)
