@@ -110,7 +110,7 @@ static void encoding_dialog(const gchar *path)
 		}
 
 	gd = generic_dialog_new("Filename encoding locale mismatch", string->str,
-				PACKAGE, "locale warning", TRUE, NULL, NULL);
+				PACKAGE, "locale warning", NULL, TRUE, NULL, NULL);
 	generic_dialog_add_stock(gd, NULL, GTK_STOCK_CLOSE, NULL, TRUE);
 	gtk_widget_show(gd->dialog);
 
