@@ -416,13 +416,13 @@ static gint dest_view_rename_cb(TreeEditData *ted, const gchar *old, const gchar
 	if (isname(new_path))
 		{
 		buf = g_strdup_printf(_("A file with name %s already exists."), new);
-		warning_dialog("Rename failed", buf);
+		warning_dialog("Rename failed", buf, dd->entry);
 		g_free(buf);
 		}
 	else if (!rename_file(old_path, new_path))
 		{
 		buf = g_strdup_printf(_("Failed to rename %s to %s."), old, new);
-		warning_dialog("Rename failed", buf);
+		warning_dialog("Rename failed", buf, dd->entry);
 		g_free(buf);
 		}
 	else
@@ -478,7 +478,7 @@ static void dest_view_delete_dlg_ok_cb(GenericDialog *gd, gpointer data)
 	if (!unlink_file(dl->path))
 		{
 		gchar *text = g_strdup_printf(_("Unable to delete file:\n%s"), dl->path);
-		warning_dialog(_("File deletion failed"), text);
+		warning_dialog(_("File deletion failed"), text, dl->dd->entry);
 		g_free(text);
 		}
 	else if (dl->dd->path)
@@ -521,7 +521,8 @@ static void dest_view_delete(Dest_Data *dd, GtkTreeView *view)
 		}
 
 	text = g_strdup_printf(_("About to delete the file:\n %s"), path);
-	dd->gd = generic_dialog_new(_("Delete file"), text, "SLIK", "dlg_confirm", TRUE,
+	dd->gd = generic_dialog_new(_("Delete file"), text, "SLIK", "dlg_confirm",
+				    dd->entry, TRUE,
 				    dest_view_delete_dlg_cancel, dl);
 	g_free(text);
 
@@ -727,7 +728,7 @@ static void dest_new_dir_cb(GtkWidget *widget, gpointer data)
 		}
 	else
 		{
-		buf = concat_dir_and_file(dd->path, _("New Directory"));
+		buf = concat_dir_and_file(dd->path, _("New folder"));
 		path = unique_filename(buf, NULL, " ", FALSE);
 		g_free(buf);
 		}
@@ -737,8 +738,8 @@ static void dest_new_dir_cb(GtkWidget *widget, gpointer data)
 		/* failed */
 		gchar *text;
 
-		text = g_strdup_printf(_("Unable to create directory:\n%s"), filename_from_path(path));
-		warning_dialog(_("Error creating directory"), text);
+		text = g_strdup_printf(_("Unable to create folder:\n%s"), filename_from_path(path));
+		warning_dialog(_("Error creating folder"), text, dd->entry);
 		g_free(text);
 		}
 	else
@@ -1031,7 +1032,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
-	button = gtk_button_new_with_label(_("New Directory"));
+	button = gtk_button_new_with_label(_("New folder"));
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(dest_new_dir_cb), dd);
 	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
@@ -1068,6 +1069,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	dd->bookmark_list = scrolled;
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	gtk_box_pack_start(GTK_BOX(hbox2), scrolled, TRUE, TRUE, 0);
@@ -1134,6 +1136,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 		gtk_widget_show(dd->filter_combo);
 
 		scrolled = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 		if (paned)
