@@ -30,6 +30,7 @@
 #include "ui_bookmark.h"
 #include "ui_fileops.h"
 #include "ui_spinner.h"
+#include "ui_utildlg.h"
 
 #include <gdk/gdkkeysyms.h> /* for key values */
 
@@ -66,8 +67,16 @@ struct _TabCompData
 	gint has_history;
 	gchar *history_key;
 	gint history_levels;
+
+	FileDialog *fd;
+	gchar *fd_title;
+	gint fd_folders_only;
+	GtkWidget *fd_button;
 };
 
+
+static void tab_completion_select_show(TabCompData *td);
+
 static void tab_completion_free_list(TabCompData *td)
 {
 	GList *list;
@@ -125,8 +134,13 @@ static void tab_completion_read_dir(TabCompData *td, const gchar *path)
 static void tab_completion_destroy(GtkWidget *widget, gpointer data)
 {
 	TabCompData *td = data;
+
 	tab_completion_free_list(td);
 	g_free(td->history_key);
+
+	if (td->fd) file_dialog_close(td->fd);
+	g_free(td->fd_title);
+
 	g_free(td);
 }
 
@@ -528,7 +542,13 @@ static gint tab_completion_key_pressed(GtkWidget *widget, GdkEventKey *event, gp
 				}
 			break;
 		case GDK_Return: case GDK_KP_Enter:
-			if (tab_completion_emit_enter_signal(td))
+			if (td->fd_button &&
+			    (event->state & GDK_CONTROL_MASK))
+				{
+				tab_completion_select_show(td);
+				stop_signal = TRUE;
+				}
+			else if (tab_completion_emit_enter_signal(td))
 				{
 				stop_signal = TRUE;
 				}
@@ -562,6 +582,22 @@ static void tab_completion_button_pressed(GtkWidget *widget, gpointer data)
 		}
 }
 
+static void tab_completion_button_size_allocate(GtkWidget *button, GtkAllocation *allocation, gpointer data)
+{
+	GtkWidget *entry = data;
+
+	if (allocation->height > entry->requisition.height)
+		{
+		GtkAllocation button_allocation;
+
+		button_allocation = button->allocation;
+		button_allocation.height = entry->requisition.height;
+		button_allocation.y = entry->allocation.y +
+			(entry->allocation.height - entry->requisition.height) / 2;
+		gtk_widget_size_allocate(button, &button_allocation);
+		}
+}
+
 static GtkWidget *tab_completion_create_complete_button(GtkWidget *entry)
 {
 	GtkWidget *button;
@@ -570,6 +606,8 @@ static GtkWidget *tab_completion_create_complete_button(GtkWidget *entry)
 
 	button = gtk_button_new();
 	GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+	g_signal_connect(G_OBJECT(button), "size_allocate",
+			 G_CALLBACK(tab_completion_button_size_allocate), entry);
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(tab_completion_button_pressed), entry);
 
@@ -746,3 +784,96 @@ gchar *remove_trailing_slash(const gchar *path)
 	return ret;
 }
 
+static void tab_completion_select_cancel_cb(FileDialog *fd, gpointer data)
+{
+	TabCompData *td = data;
+
+#if 0
+	gtk_widget_set_sensitive(td->fd_button, TRUE);
+#endif
+
+	td->fd = NULL;
+	file_dialog_close(fd);
+}
+
+static void tab_completion_select_ok_cb(FileDialog *fd, gpointer data)
+{
+	TabCompData *td = data;
+
+	gtk_entry_set_text(GTK_ENTRY(td->entry), gtk_entry_get_text(GTK_ENTRY(fd->entry)));
+
+	tab_completion_select_cancel_cb(fd, data);
+}
+
+static void tab_completion_select_show(TabCompData *td)
+{
+	const gchar *title;
+	const gchar *path;
+
+	if (td->fd)
+		{
+		gtk_window_present(GTK_WINDOW(GENERIC_DIALOG(td->fd)->dialog));
+		return;
+		}
+
+#if 0
+	gtk_widget_set_sensitive(td->fd_button, FALSE);
+#endif
+
+	title = (td->fd_title) ? td->fd_title : _("Select path");
+	td->fd = file_dialog_new(title, NULL, "SLIK", "select_path", td->entry,
+				 tab_completion_select_cancel_cb, td);
+	file_dialog_add_button_stock(td->fd, NULL, GTK_STOCK_OK,
+				 tab_completion_select_ok_cb, TRUE);
+
+	path = gtk_entry_get_text(GTK_ENTRY(td->entry));
+	if (strlen(path) == 0) path = NULL;
+	if (td->fd_folders_only)
+		{
+		file_dialog_add_path_widgets(td->fd, NULL, path, td->history_key, NULL, NULL);
+		}
+	else
+		{
+		file_dialog_add_path_widgets(td->fd, NULL, path, td->history_key, "*", _("All files"));
+		}
+
+	gtk_widget_show(GENERIC_DIALOG(td->fd)->dialog);
+}
+
+static void tab_completion_select_pressed(GtkWidget *widget, gpointer data)
+{
+	TabCompData *td = data;
+
+	tab_completion_select_show(td);
+}
+
+void tab_completion_add_select_button(GtkWidget *entry, const gchar *title, gint folders_only)
+{
+	TabCompData *td = g_object_get_data(G_OBJECT(entry), "tab_completion_data");
+	GtkWidget *hbox;
+
+	if (!td) return;
+
+	g_free(td->fd_title);
+	td->fd_title = g_strdup(title);
+	td->fd_folders_only = folders_only;
+
+	if (td->fd_button) return;
+
+	hbox = gtk_widget_get_parent(td->entry);
+	if (!GTK_IS_BOX(hbox)) return;
+
+	td->fd_button = gtk_button_new_with_label("...");
+	g_signal_connect(G_OBJECT(td->fd_button), "size_allocate",
+			 G_CALLBACK(tab_completion_button_size_allocate), td->entry);
+	g_signal_connect(G_OBJECT(td->fd_button), "clicked",
+			 G_CALLBACK(tab_completion_select_pressed), td);
+
+	gtk_box_pack_start(GTK_BOX(hbox), td->fd_button, FALSE, FALSE, 0);
+#if 0
+	gtk_box_reorder_child(GTK_BOX(hbox), td->fd_button, 0);
+#endif
+
+	gtk_widget_show(td->fd_button);
+}
+
