@@ -188,7 +188,8 @@ GtkWidget *generic_dialog_add(GenericDialog *gd, const gchar *text,
 
 static void generic_dialog_setup(GenericDialog *gd,
 				 const gchar *title, const gchar *message,
-				 const gchar *wmclass, const gchar *wmsubclass, gint auto_close,
+				 const gchar *wmclass, const gchar *wmsubclass,
+				 GtkWidget *parent, gint auto_close,
 				 void (*cancel_cb)(GenericDialog *, gpointer), gpointer data)
 {
 	GtkWidget *vbox;
@@ -201,10 +202,30 @@ static void generic_dialog_setup(GenericDialog *gd,
 	gd->dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_type_hint(GTK_WINDOW(gd->dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_wmclass(GTK_WINDOW(gd->dialog), wmsubclass, wmclass);
+	if (parent)
+		{
+		GtkWindow *window = NULL;
+
+		if (GTK_IS_WINDOW(parent))
+			{
+			window = GTK_WINDOW(parent);
+			}
+		else
+			{
+			GtkWidget *top;
+
+			top = gtk_widget_get_toplevel(parent);
+			if (GTK_IS_WINDOW(top) && GTK_WIDGET_TOPLEVEL(top)) window = GTK_WINDOW(top);
+			}
+
+		if (window) gtk_window_set_transient_for(GTK_WINDOW(gd->dialog), window);
+		}
+
 	g_signal_connect(G_OBJECT(gd->dialog), "delete_event",
 			 G_CALLBACK(generic_dialog_delete_cb), gd);
 	g_signal_connect(G_OBJECT(gd->dialog), "key_press_event",
 			 G_CALLBACK(generic_dialog_key_press_cb), gd);
+
 	gtk_window_set_resizable(GTK_WINDOW(gd->dialog), TRUE);
 	gtk_window_set_title(GTK_WINDOW (gd->dialog), title);
 	gtk_container_set_border_width(GTK_CONTAINER (gd->dialog), 5);
@@ -246,13 +267,15 @@ static void generic_dialog_setup(GenericDialog *gd,
 }
 
 GenericDialog *generic_dialog_new(const gchar *title, const gchar *message,
-				  const gchar *wmclass, const gchar *wmsubclass, gint auto_close,
+				  const gchar *wmclass, const gchar *wmsubclass,
+				  GtkWidget *parent, gint auto_close,
 				  void (*cancel_cb)(GenericDialog *, gpointer), gpointer data)
 {
 	GenericDialog *gd;
 
 	gd = g_new0(GenericDialog, 1);
-	generic_dialog_setup(gd, title, message, wmclass, wmsubclass, auto_close, cancel_cb, data);
+	generic_dialog_setup(gd, title, message, wmclass, wmsubclass,
+			     parent, auto_close, cancel_cb, data);
 	return gd;
 }
 /*
@@ -266,11 +289,11 @@ static void warning_dialog_ok_cb(GenericDialog *gd, gpointer data)
 	/* no op */
 }
 
-GenericDialog *warning_dialog(const gchar *title, const gchar *message)
+GenericDialog *warning_dialog(const gchar *title, const gchar *message, GtkWidget *parent)
 {
 	GenericDialog *gd;
 
-	gd = generic_dialog_new(title, message, "SLIK", "warning", TRUE, NULL, NULL);
+	gd = generic_dialog_new(title, message, "SLIK", "warning", parent, TRUE, NULL, NULL);
 	generic_dialog_add_stock(gd, NULL, GTK_STOCK_OK, warning_dialog_ok_cb, TRUE);
 
 	gtk_widget_show(gd->dialog);
@@ -295,6 +318,7 @@ void file_dialog_close(FileDialog *fd)
 
 FileDialog *file_dialog_new(const gchar *title, const gchar *message,
 			    const gchar *wmclass, const gchar *wmsubclass,
+			    GtkWidget *parent,
 			    void (*cancel_cb)(FileDialog *, gpointer), gpointer data)
 {
 	FileDialog *fd = NULL;
@@ -302,7 +326,7 @@ FileDialog *file_dialog_new(const gchar *title, const gchar *message,
 	fd = g_new0(FileDialog, 1);
 
 	generic_dialog_setup(GENERIC_DIALOG(fd), title, message,
-			     wmclass, wmsubclass, FALSE,
+			     wmclass, wmsubclass, parent, FALSE,
 			     (void *)cancel_cb, data);
 
 	return fd;
