@@ -21,6 +21,7 @@
 #include "image.h"
 #include "img-view.h"
 #include "layout.h"
+#include "search.h"
 #include "ui_bookmark.h"
 #include "ui_fileops.h"
 #include "ui_tabcomp.h"
@@ -39,6 +40,7 @@ void file_maint_renamed(const gchar *source, const gchar *dest)
 	view_window_maint_moved(source, dest);
 	collection_maint_renamed(source, dest);
 	dupe_maint_renamed(source, dest);
+	search_maint_renamed(source, dest);
 	cache_maint_moved(source, dest);
 }
 
@@ -49,6 +51,7 @@ void file_maint_removed(const gchar *path, GList *ignore_list)
 	view_window_maint_removed(path, ignore_list);
 	collection_maint_removed(path);
 	dupe_maint_removed(path);
+	search_maint_removed(path);
 	cache_maint_removed(path);
 }
 
@@ -59,9 +62,15 @@ void file_maint_moved(const gchar *source, const gchar *dest, GList *ignore_list
 	view_window_maint_moved(source, dest);
 	collection_maint_renamed(source, dest);
 	dupe_maint_renamed(source, dest);
+	search_maint_renamed(source, dest);
 	cache_maint_moved(source, dest);
 }
 
+void file_maint_copied(const gchar *source, const gchar *dest)
+{
+	cache_maint_copied(source, dest);
+}
+
 /*
  *--------------------------------------------------------------------------
  * The file manipulation dialogs
@@ -209,12 +218,13 @@ static void generic_dialog_image_set(GenericDialog *gd, const gchar *path)
  */
 
 GenericDialog *file_util_gen_dlg(const gchar *title, const gchar *message,
-				 const gchar *wmclass, const gchar *wmsubclass, gint auto_close,
+				 const gchar *wmclass, const gchar *wmsubclass,
+				 GtkWidget *parent, gint auto_close,
 				 void (*cancel_cb)(GenericDialog *, gpointer), gpointer data)
 {
 	GenericDialog *gd;
 
-	gd = generic_dialog_new(title, message, wmclass, wmsubclass, auto_close, cancel_cb, data);
+	gd = generic_dialog_new(title, message, wmclass, wmsubclass, parent, auto_close, cancel_cb, data);
 	if (place_dialogs_under_mouse)
 		{
 		gtk_window_set_position(GTK_WINDOW(gd->dialog), GTK_WIN_POS_MOUSE);
@@ -225,11 +235,12 @@ GenericDialog *file_util_gen_dlg(const gchar *title, const gchar *message,
 
 FileDialog *file_util_file_dlg(const gchar *title, const gchar *message,
 			       const gchar *wmclass, const gchar *wmsubclass,
+			       GtkWidget *parent,
 			       void (*cancel_cb)(FileDialog *, gpointer), gpointer data)
 {
 	FileDialog *fd;
 
-	fd = file_dialog_new(title, message, wmclass, wmsubclass, cancel_cb, data);
+	fd = file_dialog_new(title, message, wmclass, wmsubclass, parent, cancel_cb, data);
 	if (place_dialogs_under_mouse)
 		{
 		gtk_window_set_position(GTK_WINDOW(GENERIC_DIALOG(fd)->dialog), GTK_WIN_POS_MOUSE);
@@ -247,11 +258,11 @@ static void file_util_warning_dialog_ok_cb(GenericDialog *gd, gpointer data)
 	/* no op */
 }
 
-GenericDialog *file_util_warning_dialog(const gchar *title, const gchar *message)
+GenericDialog *file_util_warning_dialog(const gchar *title, const gchar *message, GtkWidget *parent)
 {
 	GenericDialog *gd;
 
-	gd = file_util_gen_dlg(title, message, "GQview", "warning", TRUE, NULL, NULL);
+	gd = file_util_gen_dlg(title, message, "GQview", "warning", parent, TRUE, NULL, NULL);
 	generic_dialog_add_stock(gd, _("Ok"), GTK_STOCK_OK, file_util_warning_dialog_ok_cb, TRUE);
 	if (place_dialogs_under_mouse)
 		{
@@ -431,7 +442,8 @@ static GenericDialog *file_util_move_multiple_confirm_dialog(FileDataMult *fdm)
 	GtkWidget *hbox;
 
 	text = g_strdup_printf(_("Overwrite file:\n %s\n with:\n %s"), fdm->dest, fdm->source);
-	gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
+	gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm",
+				NULL, TRUE,
 				file_util_move_multiple_cancel_cb, fdm);
 	g_free(text);
 
@@ -512,7 +524,8 @@ static void file_util_move_multiple(FileDataMult *fdm)
 					text = g_strdup_printf(_("Unable to move file:\n%s\nto itself."), fdm->dest);
 					}
 
-				gd = file_util_gen_dlg(title, text, "GQview", "dlg_confirm", TRUE,
+				gd = file_util_gen_dlg(title, text, "GQview", "dlg_confirm",
+							NULL, TRUE,
 							file_util_move_multiple_cancel_cb, fdm);
 				g_free(text);
 				generic_dialog_add_stock(gd, _("Continue"), GTK_STOCK_GO_FORWARD,
@@ -563,7 +576,11 @@ static void file_util_move_multiple(FileDataMult *fdm)
 				{
 				if (fdm->copy)
 					{
-					success = copy_file(fdm->source, fdm->dest);
+					if (copy_file(fdm->source, fdm->dest));
+						{
+						success = TRUE;
+						file_maint_copied(fdm->source, fdm->dest);
+						}
 					}
 				else
 					{
@@ -595,7 +612,8 @@ static void file_util_move_multiple(FileDataMult *fdm)
 				title = _("Error moving file");
 				text = g_strdup_printf(_("Unable to move file:\n%s\nto:\n%s\nduring multiple file move."), fdm->source, fdm->dest);
 				}
-			gd = file_util_gen_dlg(title, text, "GQview", "dlg_confirm", TRUE,
+			gd = file_util_gen_dlg(title, text, "GQview", "dlg_confirm",
+						NULL, TRUE,
 						file_util_move_multiple_cancel_cb, fdm);
 			g_free(text);
 
@@ -725,7 +743,7 @@ static void file_util_move_single(FileDataSingle *fds)
 	if (fds->dest && fds->source && strcmp(fds->dest, fds->source) == 0)
 		{
 		file_util_warning_dialog(_("Source matches destination"),
-					 _("Source and destination are the same, operation cancelled."));
+					 _("Source and destination are the same, operation cancelled."), NULL);
 		}
 	else if (isfile(fds->dest) && !fds->confirmed)
 		{
@@ -735,7 +753,8 @@ static void file_util_move_single(FileDataSingle *fds)
 		GtkWidget *hbox;
 
 		text = g_strdup_printf(_("Overwrite file:\n%s\n with:\n%s"), fds->dest, fds->source);
-		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
+		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm",
+					NULL, TRUE,
 					file_util_move_single_cancel_cb, fds);
 		g_free(text);
 
@@ -781,7 +800,11 @@ static void file_util_move_single(FileDataSingle *fds)
 		gint success = FALSE;
 		if (fds->copy)
 			{
-			success = copy_file(fds->source, fds->dest);
+			if(copy_file(fds->source, fds->dest));
+				{
+				success = TRUE;
+				file_maint_copied(fds->source, fds->dest);
+				}
 			}
 		else
 			{
@@ -805,7 +828,7 @@ static void file_util_move_single(FileDataSingle *fds)
 				title = _("Error moving file");
 				text = g_strdup_printf(_("Unable to move file:\n%s\nto:\n%s"), fds->source, fds->dest);
 				}
-			file_util_warning_dialog(title, text);
+			file_util_warning_dialog(title, text, NULL);
 			g_free(text);
 			}
 		}
@@ -847,11 +870,12 @@ static void file_util_move_check(FileDialog *fd)
 		if (isfile(fd->dest_path))
 			{
 			file_util_warning_dialog(_("Invalid destination"),
-						 _("When operating with multiple files, please select\na directory, not a file."));
+						 _("When operating with multiple files, please select\na folder, not a file."),
+						 NULL);
 			}
 		else
-			file_util_warning_dialog(_("Invalid directory"),
-						 _("Please select an existing directory"));
+			file_util_warning_dialog(_("Invalid folder"),
+						 _("Please select an existing folder."), NULL);
 		return;
 		}
 
@@ -868,7 +892,8 @@ static void file_util_move_cancel_cb(FileDialog *fd, gpointer data)
 	file_dialog_close(fd);
 }
 
-static void real_file_util_move(const gchar *source_path, GList *source_list, const gchar *dest_path, gint copy)
+static void real_file_util_move(const gchar *source_path, GList *source_list,
+				const gchar *dest_path, gint copy, GtkWidget *parent)
 {
 	FileDialog *fd;
 	gchar *path = NULL;
@@ -899,7 +924,7 @@ static void real_file_util_move(const gchar *source_path, GList *source_list, co
 
 	if (copy)
 		{
-		title = _("GQview - copy");
+		title = _("Copy - GQview");
 		op_text = _("Copy");
 		if (path)
 			text = g_strdup_printf(_("Copy file:\n%s\nto:"), path);
@@ -909,7 +934,7 @@ static void real_file_util_move(const gchar *source_path, GList *source_list, co
 		}
 	else
 		{
-		title = _("GQview - move");
+		title = _("Move - GQview");
 		op_text = _("Move");
 		if (path)
 			text = g_strdup_printf(_("Move file:\n%s\nto:"), path);
@@ -918,7 +943,8 @@ static void real_file_util_move(const gchar *source_path, GList *source_list, co
 		stock_id = GTK_STOCK_OK;
 		}
 
-	fd = file_util_file_dlg(title, text, "GQview", "dlg_copymove", file_util_move_cancel_cb, NULL);
+	fd = file_util_file_dlg(title, text, "GQview", "dlg_copymove", parent,
+				file_util_move_cancel_cb, NULL);
 	g_free(text);
 
 	file_dialog_add_button_stock(fd, op_text, stock_id, file_util_move_cb, TRUE);
@@ -933,14 +959,14 @@ static void real_file_util_move(const gchar *source_path, GList *source_list, co
 	gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
 }
 
-void file_util_move(const gchar *source_path, GList *source_list, const gchar *dest_path)
+void file_util_move(const gchar *source_path, GList *source_list, const gchar *dest_path, GtkWidget *parent)
 {
-	real_file_util_move(source_path, source_list, dest_path, FALSE);
+	real_file_util_move(source_path, source_list, dest_path, FALSE, parent);
 }
 
-void file_util_copy(const gchar *source_path, GList *source_list, const gchar *dest_path)
+void file_util_copy(const gchar *source_path, GList *source_list, const gchar *dest_path, GtkWidget *parent)
 {
-	real_file_util_move(source_path, source_list, dest_path, TRUE);
+	real_file_util_move(source_path, source_list, dest_path, TRUE, parent);
 }
 
 void file_util_move_simple(GList *list, const gchar *dest_path)
@@ -1044,7 +1070,8 @@ static gint file_util_safe_number(gint64 free_space)
 		if (debug) printf("expunging from trash for space: %s\n", fd->name);
 		if (!unlink_file(fd->path) && !warned)
 			{
-			file_util_warning_dialog(_("Delete failed"), _("Unable to remove old file from trash folder"));
+			file_util_warning_dialog(_("Delete failed"),
+						 _("Unable to remove old file from trash folder"), NULL);
 			warned = TRUE;
 			}
 		total -= fd->size;
@@ -1099,7 +1126,7 @@ static gint file_util_unlink(const gchar *path)
 		if (debug) printf("creating trash: %s\n", safe_delete_path);
 		if (!safe_delete_path || !mkdir_utf8(safe_delete_path, 0755))
 			{
-			result = _("Could not create directory");
+			result = _("Could not create folder");
 			success = FALSE;
 			}
 		}
@@ -1132,7 +1159,8 @@ static gint file_util_unlink(const gchar *path)
 		gchar *buf;
 
 		buf = g_strdup_printf(_("Unable to access or create the trash folder.\n\"%s\""), safe_delete_path);
-		gd = file_util_warning_dialog(result, buf);
+		gd = file_util_warning_dialog(result, buf, NULL);
+		g_free(buf);
 
 		button = gtk_check_button_new_with_label(_("Turn off safe delete"));
 		g_signal_connect(G_OBJECT(button), "toggled",
@@ -1182,7 +1210,7 @@ static void file_util_delete_multiple_ok_cb(GenericDialog *gd, gpointer data)
 {
 	GList *source_list = data;
 
-	while(source_list)
+	while (source_list)
 		{
 		gchar *path = source_list->data;
 
@@ -1196,8 +1224,9 @@ static void file_util_delete_multiple_ok_cb(GenericDialog *gd, gpointer data)
 				gchar *text;
 
 				text = g_strdup_printf(_("Unable to delete file:\n %s\n Continue multiple delete operation?"), path);
-				d = file_util_gen_dlg(_("Delete failed"), text, "GQview", "dlg_confirm", TRUE,
-						       file_util_delete_multiple_cancel_cb, source_list);
+				d = file_util_gen_dlg(_("Delete failed"), text, "GQview", "dlg_confirm",
+						      NULL, TRUE,
+						      file_util_delete_multiple_cancel_cb, source_list);
 				g_free(text);
 
 				generic_dialog_add_stock(d, _("Continue"), GTK_STOCK_GO_FORWARD, file_util_delete_multiple_ok_cb, TRUE);
@@ -1208,7 +1237,7 @@ static void file_util_delete_multiple_ok_cb(GenericDialog *gd, gpointer data)
 				gchar *text;
 				
 				text = g_strdup_printf(_("Unable to delete file:\n%s"), path);
-				file_util_warning_dialog(_("Delete failed"), text);
+				file_util_warning_dialog(_("Delete failed"), text, NULL);
 				g_free(text);
 				}
 			g_free(path);
@@ -1320,7 +1349,7 @@ static void file_util_delete_multiple_review_scroll_cb(ImageWindow *imd, GdkScro
 		}
 }
 
-static void file_util_delete_multiple(GList *source_list)
+static void file_util_delete_multiple(GList *source_list, GtkWidget *parent)
 {
 	if (!confirm_delete)
 		{
@@ -1336,8 +1365,8 @@ static void file_util_delete_multiple(GList *source_list)
 		ImageWindow *imd;
 		gchar *buf;
 
-		gd = file_util_gen_dlg(_("Delete files"), _("About to delete multiple files..."),
-					"GQview", "dlg_confirm", TRUE,
+		gd = file_util_gen_dlg(_("Delete files - GQview"), _("About to delete multiple files..."),
+					"GQview", "dlg_confirm", parent, TRUE,
 					file_util_delete_multiple_cancel_cb, source_list);
 
 		generic_dialog_image_add(gd, NULL, NULL);
@@ -1396,7 +1425,7 @@ static void file_util_delete_ok_cb(GenericDialog *gd, gpointer data)
 	if (!file_util_unlink(path))
 		{
 		gchar *text = g_strdup_printf(_("Unable to delete file:\n%s"), path);
-		file_util_warning_dialog(_("File deletion failed"), text);
+		file_util_warning_dialog(_("File deletion failed"), text, NULL);
 		g_free(text);
 		}
 	else
@@ -1414,7 +1443,7 @@ static void file_util_delete_cancel_cb(GenericDialog *gd, gpointer data)
 	g_free(path);
 }
 
-static void file_util_delete_single(const gchar *path)
+static void file_util_delete_single(const gchar *path, GtkWidget *parent)
 {
 	gchar *buf = g_strdup(path);
 
@@ -1428,7 +1457,8 @@ static void file_util_delete_single(const gchar *path)
 		gchar *text;
 
 		text = g_strdup_printf(_("About to delete the file:\n %s"), buf);
-		gd = file_util_gen_dlg(_("Delete file"), text, "GQview", "dlg_confirm", TRUE,
+		gd = file_util_gen_dlg(_("Delete file - GQview"), text, "GQview", "dlg_confirm",
+					parent, TRUE,
 					file_util_delete_cancel_cb, buf);
 		g_free(text);
 
@@ -1442,22 +1472,22 @@ static void file_util_delete_single(const gchar *path)
 		}
 }
 
-void file_util_delete(const gchar *source_path, GList *source_list)
+void file_util_delete(const gchar *source_path, GList *source_list, GtkWidget *parent)
 {
 	if (!source_path && !source_list) return;
 
 	if (source_path)
 		{
-		file_util_delete_single(source_path);
+		file_util_delete_single(source_path, parent);
 		}
 	else if (!source_list->next)
 		{
-		file_util_delete_single(source_list->data);
+		file_util_delete_single(source_list->data, parent);
 		path_list_free(source_list);
 		}
 	else
 		{
-		file_util_delete_multiple(source_list);
+		file_util_delete_multiple(source_list, parent);
 		}
 }
 
@@ -1563,7 +1593,8 @@ static void file_util_rename_multiple(RenameDataMult *rd)
 		gchar *text;
 
 		text = g_strdup_printf(_("Overwrite file:\n%s\nby renaming:\n%s"), fd->dest_path, fd->source_path);
-		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
+		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm",
+					NULL, TRUE,
 					file_util_rename_multiple_cancel_cb, rd);
 		g_free(text);
 
@@ -1582,7 +1613,7 @@ static void file_util_rename_multiple(RenameDataMult *rd)
 			gchar *text = g_strdup_printf(_("Unable to rename file:\n%s\n to:\n%s"),
 						      filename_from_path(fd->source_path),
 						      filename_from_path(fd->dest_path));
-			file_util_warning_dialog(_("Error renaming file"), text);
+			file_util_warning_dialog(_("Error renaming file"), text, NULL);
 			g_free(text);
 			}
 		else
@@ -1664,7 +1695,9 @@ static void file_util_rename_multiple_auto(RenameDataMult *rd)
 
 	if (!success)
 		{
-		warning_dialog(_("Auto rename"), _("Can not auto rename with the selected\nnumber set, one or more files exist that\nmatch the resulting name list.\n"));
+		warning_dialog(_("Auto rename"),
+			       _("Can not auto rename with the selected\nnumber set, one or more files exist that\nmatch the resulting name list.\n"),
+			       NULL);
 		return;
 		}
 
@@ -1726,7 +1759,7 @@ static void file_util_rename_multiple_auto(RenameDataMult *rd)
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(rd->auto_spin_start), (float)n);
 
 		buf = g_strdup_printf(_("Failed to rename\n%s\nThe number was %d."), filename_from_path(rd->fd->source_path), n);
-		warning_dialog(_("Auto rename"), buf);
+		warning_dialog(_("Auto rename"), buf, NULL);
 		g_free(buf);
 
 		return;
@@ -1946,7 +1979,7 @@ static void file_util_rename_multiple_destroy_cb(GtkWidget *widget, gpointer dat
 	g_free(rd);
 }
 
-static void file_util_rename_multiple_do(GList *source_list)
+static void file_util_rename_multiple_do(GList *source_list, GtkWidget *parent)
 {
 	RenameDataMult *rd;
 	GtkWidget *pane;
@@ -1964,7 +1997,8 @@ static void file_util_rename_multiple_do(GList *source_list)
 
 	rd = g_new0(RenameDataMult, 1);
 
-	rd->fd = file_util_file_dlg(_("GQview - rename"), _("Rename multiple files:"), "GQview", "dlg_rename",
+	rd->fd = file_util_file_dlg(_("Rename - GQview"), _("Rename multiple files:"),
+				    "GQview", "dlg_rename", parent,
 				    file_util_rename_multiple_close_cb, rd);
 	file_dialog_add_button_stock(rd->fd, _("Rename"), GTK_STOCK_OK, file_util_rename_multiple_cb, TRUE);
 
@@ -2133,7 +2167,8 @@ static void file_util_rename_single(FileDataSingle *fds)
 		gchar *text;
 
 		text = g_strdup_printf(_("Overwrite file:\n%s\nby renaming:\n%s"), fds->dest, fds->source);
-		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
+		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm",
+					NULL, TRUE,
 					file_util_rename_single_cancel_cb, fds);
 		g_free(text);
 
@@ -2149,7 +2184,7 @@ static void file_util_rename_single(FileDataSingle *fds)
 		if (!rename_file(fds->source, fds->dest))
 			{
 			gchar *text = g_strdup_printf(_("Unable to rename file:\n%s\nto:\n%s"), filename_from_path(fds->source), filename_from_path(fds->dest));
-			file_util_warning_dialog(_("Error renaming file"), text);
+			file_util_warning_dialog(_("Error renaming file"), text, NULL);
 			g_free(text);
 			}
 		else
@@ -2185,11 +2220,11 @@ static void file_util_rename_single_close_cb(FileDialog *fd, gpointer data)
 	file_dialog_close(fd);
 }
 
-static void file_util_rename_single_do(const gchar *source_path)
+static void file_util_rename_single_do(const gchar *source_path, GtkWidget *parent)
 {
 	FileDialog *fd;
 
-	fd = file_util_file_dlg(_("GQview - rename"), _("Rename file:"), "GQview", "dlg_rename",
+	fd = file_util_file_dlg(_("Rename - GQview"), _("Rename file:"), "GQview", "dlg_rename", parent,
 			     file_util_rename_single_close_cb, NULL);
 
 	generic_dialog_image_add(GENERIC_DIALOG(fd), source_path, NULL);
@@ -2210,22 +2245,22 @@ static void file_util_rename_single_do(const gchar *source_path)
 	gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
 }
 
-void file_util_rename(const gchar *source_path, GList *source_list)
+void file_util_rename(const gchar *source_path, GList *source_list, GtkWidget *parent)
 {
 	if (!source_path && !source_list) return;
 
 	if (source_path)
 		{
-		file_util_rename_single_do(source_path);
+		file_util_rename_single_do(source_path, parent);
 		}
 	else if (!source_list->next)
 		{
-		file_util_rename_single_do(source_list->data);
+		file_util_rename_single_do(source_list->data, parent);
 		path_list_free(source_list);
 		}
 	else
 		{
-		file_util_rename_multiple_do(source_list);
+		file_util_rename_multiple_do(source_list, parent);
 		}
 }
 
@@ -2243,35 +2278,24 @@ static void file_util_create_dir_do(const gchar *base, const gchar *name)
 
 	if (isdir(path))
 		{
-		gchar *text = g_strdup_printf(_("The directory:\n%s\nalready exists."), name);
-		file_util_warning_dialog(_("Directory exists"), text);
+		gchar *text = g_strdup_printf(_("The folder:\n%s\nalready exists."), name);
+		file_util_warning_dialog(_("Folder exists"), text, NULL);
 		g_free(text);
 		}
 	else if (isname(path))
 		{
 		gchar *text = g_strdup_printf(_("The path:\n%s\nalready exists as a file."), name);
-		file_util_warning_dialog(_("Could not create directory"), text);
+		file_util_warning_dialog(_("Could not create folder"), text, NULL);
 		g_free(text);
 		}
 	else
 		{
 		if (!mkdir_utf8(path, 0755))
 			{
-			gchar *text = g_strdup_printf(_("Unable to create directory:\n%s"), name);
-			file_util_warning_dialog(_("Error creating directory"), text);
+			gchar *text = g_strdup_printf(_("Unable to create folder:\n%s"), name);
+			file_util_warning_dialog(_("Error creating folder"), text, NULL);
 			g_free(text);
 			}
-		else
-			{
-#if 0
-			if (strcmp(base, current_path) == 0)
-				{
-				gchar *buf = g_strdup(current_path);
-				filelist_change_to(buf);
-				g_free(buf);
-				}
-#endif
-			}
 		}
 
 	g_free(path);
@@ -2305,15 +2329,15 @@ static void file_util_create_dir_close_cb(FileDialog *fd, gpointer data)
 	file_dialog_close(fd);
 }
 
-void file_util_create_dir(const gchar *path)
+void file_util_create_dir(const gchar *path, GtkWidget *parent)
 {
 	FileDialog *fd;
 	gchar *text;
 
 	if (!isdir(path)) return;
 
-	text = g_strdup_printf(_("Create directory in:\n%s\nnamed:"), path);
-	fd = file_util_file_dlg(_("GQview - new directory"), text, "GQview", "dlg_newdir",
+	text = g_strdup_printf(_("Create folder in:\n%s\nnamed:"), path);
+	fd = file_util_file_dlg(_("New folder - GQview"), text, "GQview", "dlg_newdir", parent,
 			     file_util_create_dir_close_cb, NULL);
 	g_free(text);
 
