@@ -95,7 +95,7 @@ static gint vdlist_rename_row_cb(TreeEditData *td, const gchar *old, const gchar
 		gchar *buf;
 
 		buf = g_strdup_printf(_("Failed to rename %s to %s."), old, new);
-		warning_dialog("Rename failed", buf);
+		warning_dialog("Rename failed", buf, vdl->listview);
 		g_free(buf);
 		}
 	else
@@ -335,8 +335,8 @@ static void vdlist_pop_menu_new_cb(GtkWidget *widget, gpointer data)
 		{
 		gchar *text;
 
-		text = g_strdup_printf(_("Unable to create directory:\n%s"), new_path);
-		file_util_warning_dialog(_("Error creating directory"), text);
+		text = g_strdup_printf(_("Unable to create folder:\n%s"), new_path);
+		file_util_warning_dialog(_("Error creating folder"), text, vdl->listview);
 		g_free(text);
 		}
 	else
