@@ -202,7 +202,7 @@ static gint vdtree_rename_row_cb(TreeEditData *td, const gchar *old, const gchar
 		gchar *buf;
 
 		buf = g_strdup_printf(_("Failed to rename %s to %s."), old, new);
-		warning_dialog("Rename failed", buf);
+		warning_dialog("Rename failed", buf, vdt->treeview);
 		g_free(buf);
 		}
 	else
@@ -431,8 +431,8 @@ static void vdtree_pop_menu_new_cb(GtkWidget *widget, gpointer data)
 		{
 		gchar *text;
 
-		text = g_strdup_printf(_("Unable to create directory:\n%s"), new_path);
-		file_util_warning_dialog(_("Error creating directory"), text);
+		text = g_strdup_printf(_("Unable to create folder:\n%s"), new_path);
+		file_util_warning_dialog(_("Error creating folder"), text, vdt->treeview);
 		g_free(text);
 		}
 	else
