@@ -176,28 +176,28 @@ static void vficon_pop_menu_copy_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileIcon *vfi = data;
 
-	file_util_copy(NULL, vficon_pop_menu_file_list(vfi), NULL);
+	file_util_copy(NULL, vficon_pop_menu_file_list(vfi), NULL, vfi->listview);
 }
 
 static void vficon_pop_menu_move_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileIcon *vfi = data;
 
-	file_util_move(NULL, vficon_pop_menu_file_list(vfi), NULL);
+	file_util_move(NULL, vficon_pop_menu_file_list(vfi), NULL, vfi->listview);
 }
 
 static void vficon_pop_menu_rename_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileIcon *vfi = data;
 
-	file_util_rename(NULL, vficon_pop_menu_file_list(vfi));
+	file_util_rename(NULL, vficon_pop_menu_file_list(vfi), vfi->listview);
 }
 
 static void vficon_pop_menu_delete_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileIcon *vfi = data;
 
-	file_util_delete(NULL, vficon_pop_menu_file_list(vfi));
+	file_util_delete(NULL, vficon_pop_menu_file_list(vfi), vfi->listview);
 }
 
 static void vficon_pop_menu_sort_cb(GtkWidget *widget, gpointer data)
