@@ -25,7 +25,7 @@ void vficon_set_thumb_status_func(ViewFileIcon *vfi,
 void vficon_set_layout(ViewFileIcon *vfi, LayoutWindow *layout);
 
 gint vficon_set_path(ViewFileIcon *vfi, const gchar *path);
-gint vficon_refresh(ViewFileIcon *vfi);;
+gint vficon_refresh(ViewFileIcon *vfi);
 
 void vficon_sort_set(ViewFileIcon *vfi, SortType type, gint ascend);
 
