@@ -253,14 +253,14 @@ static void vflist_pop_menu_copy_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileList *vfl = data;
 
-	file_util_copy(NULL, vflist_pop_menu_file_list(vfl), NULL);
+	file_util_copy(NULL, vflist_pop_menu_file_list(vfl), NULL, vfl->listview);
 }
 
 static void vflist_pop_menu_move_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileList *vfl = data;
 
-	file_util_move(NULL, vflist_pop_menu_file_list(vfl), NULL);
+	file_util_move(NULL, vflist_pop_menu_file_list(vfl), NULL, vfl->listview);
 }
 
 static void vflist_pop_menu_rename_cb(GtkWidget *widget, gpointer data)
@@ -291,14 +291,14 @@ static void vflist_pop_menu_rename_cb(GtkWidget *widget, gpointer data)
 		return;
 		}
 
-	file_util_rename(NULL, list);
+	file_util_rename(NULL, list, vfl->listview);
 }
 
 static void vflist_pop_menu_delete_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileList *vfl = data;
 
-	file_util_delete(NULL, vflist_pop_menu_file_list(vfl));
+	file_util_delete(NULL, vflist_pop_menu_file_list(vfl), vfl->listview);
 }
 
 static void vflist_pop_menu_sort_cb(GtkWidget *widget, gpointer data)
@@ -445,19 +445,19 @@ static gint vflist_row_rename_cb(TreeEditData *td, const gchar *old, const gchar
 	if (strchr(new, '/') != NULL)
 		{
 		gchar *text = g_strdup_printf(_("Invalid file name:\n%s"), new);
-		file_util_warning_dialog(_("Error renaming file"), text);
+		file_util_warning_dialog(_("Error renaming file"), text, vfl->listview);
 		g_free(text);
 		}
 	else if (isfile(new_path))
 		{
 		gchar *text = g_strdup_printf(_("A file with name %s already exists."), new);
-		file_util_warning_dialog(_("Error renaming file"), text);
+		file_util_warning_dialog(_("Error renaming file"), text, vfl->listview);
 		g_free(text);
 		}
 	else if (!rename_file(old_path, new_path))
 		{
 		gchar *text = g_strdup_printf(_("Unable to rename file:\n%s\nto:\n%s"), old, new);
-		file_util_warning_dialog(_("Error renaming file"), text);
+		file_util_warning_dialog(_("Error renaming file"), text, vfl->listview);
 		g_free(text);
 		}
 	else
